@@ -1,11 +1,18 @@
 /* App.m
  * Application class of Cenon
  *
- * Copyright (C) 1995-2011 by vhf interservice GmbH
+ * Copyright (C) 1995-2013 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2010-07-04 (svg stuff added)
+ * modified: 2013-06-14 (-listFromPSFile: AI with header will be loaded as UTF8 and lossy)
+ *           2012-06-29 (-terminate:, terminate sub processes)
+ *           2012-06-22 (-openFile:, remove i-cut Import Stuff)
+ *           2012-02-06 (systemLibrary: return nil on Apple)
+ *           2011-12-02 (i-cut Import Stuff added)
+ *           2011-04-05 (Vectorizer)
+ *           2011-03-06 (-applicationDidFinishLaunching: auto check for updates, -importASCII: removed)
+ *           2010-07-04 (svg stuff added)
  *           2010-06-30 (-changeSafeType: TIFF added)
  *           2010-04-10 (displayInfo: get version from plist and date from __DATE__)
  *           2010-01-12 (take snapshot of open documents and their positions)
@@ -40,7 +47,7 @@
 #include "locations.h"
 #include "App.h"
 #include "CenonModuleMethods.h"
-#include "MyPageLayout.h"
+//#include "MyPageLayout.h"
 #include "TilePanel.h"
 #include "GridPanel.h"
 #include "WorkingAreaPanel.h"
@@ -50,6 +57,7 @@
 #include "DocWindow.h"
 #include "PreferencesMacros.h"
 #include "UpdateController.h"
+#include "Vectorizer.h"
 
 #include "DINImportSub.h"
 #include "DXFImportSub.h"
@@ -57,7 +65,8 @@
 #include "HPGLImportSub.h"
 #include "PSImportSub.h"
 #include "Type1ImportSub.h"
-//#include "SVGImportSub.h"
+#include "SVGImportSub.h"
+#include "ICUTImportSub.h"
 
 @implementation App
 
@@ -89,6 +98,8 @@
     [registrationDict setObject:@"NO"        forKey:@"psPreferArcs"];
     [registrationDict setObject:@"NO"        forKey:@"colorToLayer"];
     [registrationDict setObject:@"NO"        forKey:@"fillObjects"];
+    //[registrationDict setObject:@"NO"        forKey:@"icutFillClosedPaths"];
+    //[registrationDict setObject:@"NO"        forKey:@"icutOriginUL"];
 
     /* Export preferences defaults */
     [registrationDict setObject:@"NO"        forKey:@"exportFlattenText"];
@@ -208,59 +219,53 @@
 }
 
 /* created:  2002-07-18
- * modified: 2005-09-05 (load module only once)
+ * modified: 2012-02-06 (test for path == nil)
+ *           2011-06-08 (add CAM.bundle to loadedFiles)
+ *           2005-09-05 (load module only once)
+ *
+ * FIXME: Each module should give info about other modules needed before
+ *        Each module should give info about modules to be in conflict
+ *        Available modules should be selectable by switches in preferences
  */
 - (void)loadModules
 {   NSBundle        *mainBundle = [NSBundle mainBundle], *bundle;
     Class           bundleClass;
-    NSString        *path;
+    NSString        *path = nil;
     int             i, j;
     NSFileManager   *fileManager = [NSFileManager defaultManager];
-    BOOL            camModuleLoaded = NO;
     NSMutableArray  *loadedFiles = [NSMutableArray array];
 
-    /* load modules from application bundle
-     * FIXME: The CAM module should be integrated below (case 0)
-     *        Each module should give info about other modules needed before
-     *        Each module should give info about modules to be in conflict
-     *        Available modules should be selectable by switches in preferences
-     */
-    path = [mainBundle pathForResource:@"CAM" ofType:@"bundle"];
-    if ( (bundle = [NSBundle bundleWithPath:path]) )
-    {
-        bundleClass = [bundle principalClass];  // controller
-        [modules addObject:bundle];             // our loaded modules
-        [bundleClass instance];                 // create instance
-        camModuleLoaded = YES;
-    }
-
-    /* load additional modules */
-    for (i=0; i<5; i++)
+    /* load modules */
+    for (i=0; i <= 5; i++)
     {   NSArray	*files;
 
         switch (i)
         {
-            //case 0: path = [[mainBundle resourcePath] stringByAppendingPathComponent:BUNDLEFOLDER]; break;
-            //case 0: path = [mainBundle resourcePath]; break;
-            case 0: path = [userLibrary()  stringByAppendingPathComponent:@"Bundles"]; break;
-            case 1: path = [localLibrary() stringByAppendingPathComponent:@"Bundles"]; break;
-            case 2: path = userBundlePath(); break;
-            case 3: path = localBundlePath(); break;
-            case 4: path = systemBundlePath(); break;
+            default:
+            //case 0: path = [[mainBundle resourcePath] stringByAppendingPathComponent:@"Modules"]; break;
+            case 0: path = [mainBundle resourcePath]; break;    // load modules from main bundle
+            case 1: path = [userLibrary()  stringByAppendingPathComponent:@"Bundles"]; break;
+            case 2: path = [localLibrary() stringByAppendingPathComponent:@"Bundles"]; break;
+            case 3: path = userBundlePath();   break;           // load modules from Home Library/Cenon of user
+            case 4: path = localBundlePath();  break;           // load modules from /Library/Extensions/Cenon
+            case 5: path = systemBundlePath(); break;           // load modules from /System/Library/Bundles/Cenon (GNUstep only)
         }
+        if ( !path )
+            continue;
 
         /* we load the bundles in alphabetic order */
         files = [[fileManager directoryContentsAtPath:path] sortedArrayUsingSelector:@selector(compare:)];
         for (j=0; j<(int)[files count]; j++)
         {   NSString	*file = [files objectAtIndex:j];
 
-            if (camModuleLoaded && [file hasPrefix:@"Cut."])	// either CAM or Cut, not both
+            if ([loadedFiles containsObject:@"CAM.bundle"] && [file hasPrefix:@"Cut."]) // either CAM or Cut, not both
                 continue;
             if ( [file hasSuffix:@".bundle"] &&
                  ![loadedFiles containsObject:file] &&  // already loaded ?
                  (bundle = [NSBundle bundleWithPath:[path stringByAppendingPathComponent:file]]) )
             {
-                bundleClass = [bundle principalClass];  // controller
+                NSLog(@"Load Module: %@/%@\n", path, file);
+                bundleClass = [bundle principalClass];  // controller (XYZPrincipal)
                 [modules addObject:bundle];             // our loaded modules
                 [bundleClass instance];                 // create instance
                 [loadedFiles addObject:file];
@@ -274,7 +279,7 @@
 }
 
 /* created:  1993-01-??
- * modified: 2005-11-18
+ * modified: 2012-03-09 (Apple: copy existing Library to location in "Application Support")
  */
 - (void)applicationDidFinishLaunching:(NSNotification *)notification
 {   NSApplication	*theApplication = [notification object];
@@ -283,10 +288,37 @@
 
     [[NSUserDefaults standardUserDefaults] setObject:@"." forKey:@"NSDecimalSeparator"];
 
-    /* create HOME-Library if it don't exist
+    /* Apple: if Home-Library exists in old location, move to new location */
+#   ifdef __APPLE__ // keep things working through transition to new Library-location
+    {   NSString    *oldPath;
+
+        oldPath = [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) objectAtIndex:0]
+                   stringByAppendingPathComponent:APPNAME];
+        path    = vhfPathWithPathComponents(
+                  [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) objectAtIndex:0],
+                  @"Application Support", APPNAME, nil );
+        if ( [fileManager fileExistsAtPath:oldPath] && ![fileManager fileExistsAtPath:path] )
+        {
+            if ( NSRunAlertPanel(@"", NSLocalizedString(@"Cenon User-Library will be moved from old location\n \"%@\"\nto new location\n \"%@\".", NULL),
+                                 OK_STRING, CANCEL_STRING, nil, oldPath, path) == NSAlertDefaultReturn )
+            {
+                if ( [fileManager respondsToSelector:@selector(moveItemAtPath:toPath:error:)] ) // >= 10.5
+                    [fileManager moveItemAtPath:oldPath toPath:path error:NULL];    // mv oldPath -> path
+                else                                                                // <  10.5
+                    [fileManager movePath:oldPath toPath:path handler:nil];
+            }
+        }
+        else if ( [fileManager fileExistsAtPath:oldPath] )  // it's in both locations
+            NSRunAlertPanel(@"", NSLocalizedString(@"Cenon User-Library exists in old location\n \"%@\"\nand new location\n \"%@\".\nOnly the new location is used !", NULL),
+                            OK_STRING, nil, nil, oldPath, path);
+    }
+#   endif
+
+    /* create Cenon HOME-Library. If it doesn't exist, we copy the frame
+     * FIXME: on Apple, we have to prepare this in the HOME/Documents folder now !
      */
-    path = userLibrary();
-    if ( ![fileManager fileExistsAtPath:path] )
+    path = vhfUserLibrary(APPNAME); // vhfUserLibrary(APPNAME) or vhfUserDocuments(APPNAME)
+    if ( ! [fileManager fileExistsAtPath:path] )    // no Cenon Home-Library
     {
         if ( ![fileManager fileExistsAtPath:localLibrary()] )
             NSRunAlertPanel(@"", CANTFINDLIB_STRING, OK_STRING, nil, nil, NULL);
@@ -301,11 +333,11 @@
                                     attributes:nil];
             /* copy Cenon/Projects/.dir.tiff */
             from = vhfPathWithPathComponents(localLibrary(), @"Projects", @".dir.tiff", nil);
-            to = vhfPathWithPathComponents(path, @"Projects", @".dir.tiff", nil);
+            to   = vhfPathWithPathComponents(path,           @"Projects", @".dir.tiff", nil);
             [fileManager copyPath:from toPath:to handler:nil];
             /* copy Cenon/dir.tiff */
             from = vhfPathWithPathComponents(localLibrary(), @".dir.tiff", nil);
-            to = vhfPathWithPathComponents(path, @".dir.tiff", nil);
+            to   = vhfPathWithPathComponents(path,           @".dir.tiff", nil);
             [fileManager copyPath:from toPath:to handler:nil];
         }
     }
@@ -328,6 +360,22 @@
 
     /* Load Document Snapshot */
     [self restoreSnapshot:self];
+
+#ifdef __APPLE__
+    /* automatically check for updates */
+    [[UpdateController sharedInstance] checkForUpdates:self];
+    /*[[NSRunLoop currentRunLoop] performSelector:@selector(checkForUpdates:)
+                                         target:[UpdateController sharedInstance]
+                                            argument:self order:10 modes:NSDefaultRunLoopMode];*/
+#endif
+
+#ifdef GNUSTEP_BASE_VERSION
+    /* menu in window */
+    if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) == NSWindows95InterfaceStyle)
+    {
+        [self new:self];
+    }
+#endif
 }
 
 
@@ -344,7 +392,8 @@
     document = [Document new];
 }
 
-/* modified: 2010-09-10 (defaultManager renamed to fileManager)
+/* modified: 2011-09-16 (icut format added)
+ *           2010-09-10 (defaultManager renamed to fileManager)
  *           2009-02-25 (fileNameLC)
  */
 - (id)listFromFile:(NSString*)fileName
@@ -360,9 +409,19 @@
         NSData	*data = [NSData dataWithContentsOfFile:fileName];	// get data object
 
         [dxfImport setRes:Prefs_DXFRes];
-        list = [[dxfImport importDXF:data] retain];	// get list of graphic-objects from import
+        list = [[dxfImport importDXF:data] retain];	// get list of graphic objects from import
         [dxfImport release];
     }
+    else if ( [fileNameLC hasSuffix:@".cut"] || [fileNameLC hasSuffix:@".icut"] )
+    {	id      icutImport = [[ICUTImportSub allocWithZone:[self zone]] init];
+        NSData  *data = [NSData dataWithContentsOfFile:fileName];   // get data object
+
+        // FIXME: should come from Somewher with the possibility to change
+        //        [icutImport fillClosedPaths:Prefs_ICUTFillClosedPaths];
+        //        [icutImport originUL:Prefs_ICUTOriginUL];
+        list = [[icutImport importICUT:data] retain];	// get list of graphic objects from import
+        [icutImport release];
+    }
     else if ( [fileNameLC hasSuffix:@".hpgl"] || [fileNameLC hasSuffix:@".hgl"] ||
               [fileNameLC hasSuffix:@".plt"] )
     {	id      hpglImport;
@@ -389,7 +448,7 @@
             return nil;
         }
         data = [NSData dataWithContentsOfFile:fileName];	// get file-stream
-        list = [[hpglImport importHPGL:data] retain];		// get list of graphic-objects from import
+        list = [[hpglImport importHPGL:data] retain];		// get list of graphic objects from import
         [hpglImport release];
     }
     else if ( [fileNameLC hasSuffix:@".ger"] || [fileNameLC hasSuffix:@".gerber"] )
@@ -483,13 +542,13 @@
         list = [[DINImportSub layerListFromGraphicList:list] retain];
         [dinImport release];
     }
-    /*else if ( [fileNameLC hasSuffix:@".svg"] )
+    else if ( [fileNameLC hasSuffix:@".svg"] )
     {	SVGImport   *svgImport = [[SVGImportSub allocWithZone:[self zone]] init];
         NSData      *data = [NSData dataWithContentsOfFile:fileName];
 
         list = [[svgImport importSVG:data] retain]; // get list of graphic-objects from import
         [svgImport release];
-    }*/
+    }
     else if ( [fileNameLC hasSuffix:@".font"] || [fileNameLC hasSuffix:@".pfa"] || [fileNameLC hasSuffix:@".pfb"] )
     {   id          fontObject, type1Import;
         NSData		*data;
@@ -517,6 +576,12 @@
     else if ( [fileNameLC hasSuffix:@".pdf"] || [fileNameLC hasSuffix:@".PDF"])
     {   PSImportSub	*psImport = [[PSImportSub allocWithZone:[self zone]] init];
 
+#ifdef __APPLE__    // check, if gs is installed (Linux has it installed anyway, OpenStep uses DPS)
+        {   path = [psImport gsPath];
+            if ( ! [fileManager fileExistsAtPath:path] )
+                NSRunAlertPanel(@"", PSIMPORT_INSTALLGS_STRING, OK_STRING, nil, nil, nil);
+        }
+#endif
         [psImport preferArcs:Prefs_PSPreferArcs];
         [psImport flattenText:Prefs_PSFlattenText];
         list = [[psImport importPDFFromFile:fileName] retain];
@@ -534,32 +599,34 @@
     return [list autorelease];
 }
 
+/* modified: 2013-06-14 (AI with header will be loaded as UTF8 and lossy for more tolerance)
+ */
 - (NSArray*)listFromPSFile:(NSString*)fileName
 {   PSImportSub		*psImport;
-    NSData		*data = nil;
+    NSData          *data = nil;
     NSString		*path;
-    NSFileManager	*defaultManager = [NSFileManager defaultManager];
+    NSFileManager	*fileManager = [NSFileManager defaultManager];
 
     /* Adobe Illustrator */
     if ( [fileName hasSuffix:@".ai"] )
     {   NSString	*string, *header;
         NSRange		range;
 
-        string = [NSString stringWithContentsOfFile:fileName];	/* get file */
+        string = [NSString stringWithContentsOfFile:fileName];	// get file
         range = [string rangeOfString:@"%%BeginResource"];
-        if ( !range.length )	/* no ps-header */
+        if ( !range.length )	// no ps-header
         {
             path = [[[NSBundle bundleForClass:[PSImport class]] resourcePath]
                     stringByAppendingPathComponent:AI_HEADER];
-            if ( ![defaultManager fileExistsAtPath:path] )
+            if ( ![fileManager fileExistsAtPath:path] )
             {   NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                 return nil;
             }
-            header = [NSString stringWithContentsOfFile:path];	/* get file */
+            header = [NSString stringWithContentsOfFile:path];	// get file
 
             string = [header stringByAppendingString:string];
         }
-        data = [string dataUsingEncoding:NSASCIIStringEncoding];
+        data = [string dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES];
     }
     else
         data = [NSData dataWithContentsOfFile:fileName];	/* get file */
@@ -569,11 +636,17 @@
     //[psImport moveToOrigin:NO];
     [psImport preferArcs:Prefs_PSPreferArcs];
     [psImport flattenText:Prefs_PSFlattenText];
+#ifdef __APPLE__    // check, if gs is installed (Linux has it installed anyway, OpenStep uses DPS)
+    {   path = [psImport gsPath];
+        if ( ! [fileManager fileExistsAtPath:path] )
+            NSRunAlertPanel(@"", PSIMPORT_INSTALLGS_STRING, OK_STRING, nil, nil, nil);
+    }
+#endif
     return [psImport importPS:data];
 }
 
 - (BOOL)openFile:(NSString *)fileName
-{   id              list = 0;
+{   id              list = nil;
     NSString        *path;
     NSFileManager   *fileManager = [NSFileManager defaultManager];
 
@@ -606,7 +679,7 @@
     }
     else if ( (list = [self listFromFile:fileName]) )
     {
-        /* special treatment for fonts */
+        /* Fonts: special treatment for fonts */
         if ( [fileName hasSuffix:@".font"] || [fileName hasSuffix:@".pfa"] || [fileName hasSuffix:@".pfb"] )
         {
             document = [Document new];
@@ -627,7 +700,8 @@
 }
 
 /* Import files
- * modified: 2010-02-24 (allow extension ".tif")
+ * modified: 2011-12-03 ("cut", "icut" added)
+ *           2010-02-24 (allow extension ".tif")
  */
 typedef enum
 {   IMPORTTO_SELECTEDLAYER  = 0,
@@ -638,7 +712,7 @@ typedef enum
 {   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                               @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
                                               @"cenon", @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
-                                              @"font", @"pfa", @"din", @"drl", @"svg", nil];
+                                              @"font", @"pfa", @"din", @"drl", @"svg", @"cut", @"icut", nil];
     NSString    *fileName;
     id          openPanel = [NSOpenPanel openPanel];
     static      NSString	*openDir = @"";
@@ -688,20 +762,22 @@ typedef enum
 
 /* modified: 2005-11-14
  */
+#if 0   // moved to CAM module, can be removed
 - (void)importASCII:sender
-{   NSArray		*fileTypes = [NSArray arrayWithObjects:@"txt", @"asc", @"tab", nil];
-    NSString		*fileName;
-    id			openPanel = [NSOpenPanel openPanel];
-    static NSString	*openDir = @"";
+{   NSArray         *fileTypes = [NSArray arrayWithObjects:@"txt", @"asc", @"tab", nil];
+    NSString        *fileName;
+    id              openPanel = [NSOpenPanel openPanel];
+    static NSString *openDir = @"";
 
     fillPopup(iaaPopup, CHARCONV_FOLDER, DICT_EXT, 1);
     [openPanel setAccessoryView:[importASCIIAccessory retain]];
 
     [openPanel setAllowsMultipleSelection:NO];
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
-    {   int		sort = [iaaRadio selectedColumn];
-        NSString	*tabName, *string;
-        NSDictionary	*conversionDict = nil;
+    {   int             sort = [iaaRadio selectedColumn];
+        NSString        *tabName, *string;
+        NSDictionary    *conversionDict = nil;
+        DocView         *docView = [[self currentDocument] documentView];
 
         if ( [iaaPopup indexOfSelectedItem] >= 1 &&
              (tabName = [NSString stringWithFormat:@"%@%@", [iaaPopup title], DICT_EXT]) )
@@ -709,30 +785,38 @@ typedef enum
         [openDir release];
         openDir = [[[openPanel filename] stringByDeletingLastPathComponent] retain];
 
-	fileName = [openPanel filename];
+        fileName = [openPanel filename];
         string = [NSString stringWithContentsOfFile:fileName];
-        [[[self currentDocument] documentView] importASCII:stringWithConvertedChars(string, conversionDict)
-                                                      sort:sort];
+        [docView importASCII:stringWithConvertedChars(string, conversionDict) sort:sort];
     }
     [openPanel setAccessoryView:nil];
 }
+#endif
 
 /*
  * openDocument gets a file name from the user, creates a new document window,
  * and loads the specified file into it.
+ * modified: 2012-03-13 (set openPanelAccessory)
+ *           2011-12-03 ("cut", "icut" added)
  */
 - (void)openDocument:sender
 {   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                   @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
                                   @"cenon", @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
-                                  @"font", @"pfa", @"din", @"drl", @"svg", nil];
+                                  @"font", @"pfa", @"din", @"drl", @"svg", @"cut", @"icut", nil];
     NSArray         *fileNames;
     id              openPanel = [NSOpenPanel openPanel];
     int             i, cnt;
     static NSString *openDir = @"";
 
+    if (openPanelAccessory)
+        [openPanel setAccessoryView:[openPanelAccessory retain]];
     [openPanel setAllowsMultipleSelection:YES];
+    [openPanel setDirectory:openDir];
+    [openPanel setAllowedFileTypes:fileTypes];  // FIXME, Apple: this method doesn't work, we stay with the deprecated modal method (2012-03-13)
+
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
+    //if ( [openPanel runModal] )
     {
         [openDir release];
         openDir = [[[openPanel filename] stringByDeletingLastPathComponent] retain];
@@ -743,6 +827,30 @@ typedef enum
             haveOpenedDocument = [self openFile:[fileNames objectAtIndex:i]] || haveOpenedDocument;
     }
 }
+/* called by Open Panel Accessory to jump to places
+ * created: 2012-03-13
+ */
+- (void)changeOpenLocation:(id)sender
+{   NSOpenPanel *openPanel = (NSOpenPanel*)[sender window];
+    NSString    *path = nil;
+
+    switch ([(NSCell*)[sender selectedCell] tag])
+    {
+        case 0:     // Examples
+            path = vhfLocalLibrary(APPNAME);     // ex: "/Library/Cenon"
+            break;
+        case 1:     // User Library
+            path = vhfUserLibrary(APPNAME);      // ex: "HOME/Library/Cenon"
+            break;
+        case 2:     // Documents
+            path = vhfUserDocuments(APPNAME);    // ex: HOME/Documents/Cenon
+            break;
+        default:
+            NSLog(@"App, changeOpenLocation: Unknown index");
+            return;
+    }
+    [openPanel setDirectory:path];
+}
 
 /*
  * Saves the file.  If this document has never been saved to disk,
@@ -765,13 +873,13 @@ typedef enum
     switch ([sender indexOfSelectedItem])
     {
         case 0: [savePanel setRequiredFileType:DOCUMENT_EXT]; break;
-        case 1: [savePanel setRequiredFileType:EPS_EXT]; break;
-        case 2: [savePanel setRequiredFileType:GERBER_EXT]; break;
-        case 3: [savePanel setRequiredFileType:DXF_EXT]; break;
-        case 4: [savePanel setRequiredFileType:HPGL_EXT]; break;
-        case 5: [savePanel setRequiredFileType:TIFF_EXT]; break;
-        case 6: [savePanel setRequiredFileType:FONT_EXT]; break;
-        case 7: [savePanel setRequiredFileType:DIN_EXT]; break;
+        case 1: [savePanel setRequiredFileType:EPS_EXT];      break;
+        case 2: [savePanel setRequiredFileType:GERBER_EXT];   break;
+        case 3: [savePanel setRequiredFileType:DXF_EXT];      break;
+        case 4: [savePanel setRequiredFileType:HPGL_EXT];     break;
+        case 5: [savePanel setRequiredFileType:TIFF_EXT];     break;
+        case 6: [savePanel setRequiredFileType:FONT_EXT];     break;
+        case 7: [savePanel setRequiredFileType:DIN_EXT];      break;
         default: NSLog(@"App, changeFileType: Unknown file type");
     }
 }
@@ -874,7 +982,8 @@ typedef enum
 
 /* Snapshots
  * created:  2010-01-12
- * modified: 2010-04-23
+ * modified: 2012-09-04 (prefer Defaults-file over System-Defaults, don't save to Defaults any more)
+ *           2010-04-23
  */
 - (void)takeSnapshot:sender
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
@@ -913,62 +1022,145 @@ typedef enum
             file = [savePanel filename];
         else
             return;
-        if ( [file rangeOfString:name].length )
-            [defaults setObject:snapArray forKey:@"snapShotDocuments"];
+        if ( [file rangeOfString:name].length ) // if we save to Default.plist, we also save it to Defaults
+            [defaults removeObjectForKey:@"snapShotDocuments"]; // old -> delete
+            //[defaults setObject:snapArray forKey:@"snapShotDocuments"];
 
         [snapArray writeToFile:file atomically:YES];
     }
 }
 - (void)restoreSnapshot:sender
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
-    NSArray         *snapArray = [defaults objectForKey:@"snapShotDocuments"];
-    int             i;
+    NSString        *name = @"Default.plist";
+    NSArray         *snapArray = nil;   //[defaults objectForKey:@"snapShotDocuments"];
+    int             w, i;
+    BOOL            abortOp = NO;
 
     if ( [sender isKindOfClass:[NSMenuItem class]] )
     {   NSString    *path = vhfPathWithPathComponents(vhfUserLibrary(APPNAME), @".snapshots", nil);
-        NSString    *file, *name = @"Default.plist";
+        NSString    *file = vhfPathWithPathComponents(path, name, nil);
         NSOpenPanel *openPanel = [NSOpenPanel openPanel];
 
-        file = vhfPathWithPathComponents(path, name, nil);
         [openPanel setRequiredFileType:@"plist"];
         if ( [openPanel runModalForDirectory:path file:name] )
             file = [openPanel filename];
         else
             return;
-        if ( ! (snapArray = [NSArray arrayWithContentsOfFile:file]) )
-            snapArray = [defaults objectForKey:@"snapShotDocuments"];
+        if ( ! (snapArray = [NSArray arrayWithContentsOfFile:file]) )   // load from file
+            snapArray = [defaults objectForKey:@"snapShotDocuments"];   // no file -> load from System-Defaults
+    }
+    else    // load Defaults.plist or old @"snapShotDocuments" from System-Defaults
+    {   NSString    *path = vhfPathWithPathComponents(vhfUserLibrary(APPNAME), @".snapshots", nil);
+        NSString    *file = vhfPathWithPathComponents(path, name, nil);
+
+        if ( ! (snapArray = [NSArray arrayWithContentsOfFile:file]) )   // load from file
+            snapArray = [defaults objectForKey:@"snapShotDocuments"];   // no file -> load from System-Defaults
     }
 
+    // TODO: check for unsaved windows before starting restore snapshot from menu
+    for (w = [[self windows] count]-1; w >= 0; w--)
+    {	Document    *doc = [self documentInWindow:[[self windows] objectAtIndex:w]];
+
+        if (doc && [doc dirty])
+        {
+            switch (NSRunAlertPanel(CLOSEWINDOW_STRING, UNSAVEDDOCS_STRING, REVIEW_STRING, DONTSAVE_STRING, CANCEL_STRING))
+            {
+                case NSAlertDefaultReturn:      // review unsaved
+                    w = -1;
+                    break;
+                case NSAlertAlternateReturn:    // close anyway
+                    for (i = w; i >= 0; i--)
+                        [[self documentInWindow:[[self windows] objectAtIndex:i]] setDirty:NO];
+                    break;
+                default:                        // cancel
+                    return;
+            }
+        }
+    }
+    /* close documents which are not in snapArray */
+    for ( w = [[self windows] count]-1; w >= 0; w-- )
+    {	Document    *doc = [self documentInWindow:[[self windows] objectAtIndex:w]];
+
+        if (doc)
+        {   DocWindow   *window = [doc window];
+            NSString    *fileName = [doc filename];
+            BOOL        removeDoc = YES;
+
+            for (i=0; i<[snapArray count]; i++)
+            {   NSArray *array = [snapArray objectAtIndex:i];
+
+                if ( [array isKindOfClass:[NSArray class]] && [array count] >= 1 )
+                {   NSString    *path = [array objectAtIndex:0];
+
+                    if ( [path isEqual:fileName] )
+                    {   removeDoc = NO;
+                        break;  // keep document
+                    }
+                }
+            }   // end loop: snapArray
+            if ( removeDoc )
+            {
+                if ( ! [doc dirty] )
+                    [window performClose:self];
+                else
+                    abortOp = YES;
+            }
+        }
+    }
+    if ( abortOp )    // if window is not closed, then cancel
+        return;	// cancel restauration
+
+    /* open documents in snapArray */
     for (i=0; i<[snapArray count]; i++)
     {   NSArray *array = [snapArray objectAtIndex:i];
 
-        if ( [array isKindOfClass:[NSString class]] )
+        if ( [array isKindOfClass:[NSString class]] )   // old format
             [self openFile:[snapArray objectAtIndex:i]];
         else if ([array count] >= 1)
-        {   [self openFile:[array objectAtIndex:0]];
-            if (document && [array count] >= 2) // point or rect
+        {   NSString    *path = [array objectAtIndex:0];
+            Document    *doc;
+
+#           ifdef __APPLE__ // keep things working through transition to new Library-location
+            NSFileManager   *fileManager = [NSFileManager defaultManager];
+            if ( ! [fileManager fileExistsAtPath:path] )
+            {   NSString    *oldUserLib = [vhfUserLibrary(nil) stringByDeletingLastPathComponent];
+                NSRange     range;
+
+                oldUserLib = [oldUserLib stringByAppendingPathComponent:APPNAME];
+                range = [path rangeOfString:oldUserLib];
+                if ( range.length )
+                {
+                    path = [vhfUserLibrary(APPNAME) stringByAppendingPathComponent:
+                            [path substringFromIndex:range.location+range.length]];
+                }
+            }
+#           endif
+            [self openFile:path];
+            doc = (document) ? document : [self currentDocument];   // current doc if doc was open already
+
+            if (doc && [[doc filename] isEqual:path] && [array count] >= 2) // point or rect
             {    NSArray    *components = [[array objectAtIndex:1] componentsSeparatedByString:@" "];
 
                 if ( [components count] >= 4 )       // rectangle
                 {   NSRect  rect = rectFromPropertyList([array objectAtIndex:1]);
 
-                    [[document window] setFrame:rect display:NO];
+                    [[doc window] setFrame:rect display:NO];
 #ifdef __APPLE__
                     if ( [array count] >= 3 &&  // unfoldedSize
                         [[array objectAtIndex:2] respondsToSelector:@selector(floatValue)] )
                     {   float   h = [[array objectAtIndex:2] floatValue];
-                        [[document window] setUnfoldedHeight:h];
+                        [[doc window] setUnfoldedHeight:h];
                     }
 #endif
                 }
                 else                            // point only (v 3.9.1 only)
                 {   NSPoint p = pointFromPropertyList([array objectAtIndex:1]);
 
-                    [[document window] setFrameOrigin:p];
+                    [[doc window] setFrameOrigin:p];
                 }
             }
         }
-    }
+    }   // end loop: snapArray
 }
 
 /*
@@ -990,8 +1182,12 @@ typedef enum
 
     /* set version number and date of compilation */
     {   NSDictionary    *infoDict = [[NSBundle mainBundle] infoDictionary];
-        NSString        *version = [infoDict objectForKey:@"CFBundleVersion"];
+        NSString        *version = [infoDict objectForKey:@"CFBundleShortVersionString"];
 
+        if ( ! version )
+            version = [infoDict objectForKey:@"CFBundleVersion"];   // Apple, 2nd chance
+        if ( ! version )
+            version = [infoDict objectForKey:@"NSVersion"];         // GNUstep
         if ( version )
         {   NSString    *compileDate = [self compileDate];
 
@@ -1015,11 +1211,12 @@ typedef enum
 }
 - (NSString*)version
 {   NSDictionary    *infoDict = [[NSBundle mainBundle] infoDictionary];
-    NSString        *version;
+    NSString        *version = [infoDict objectForKey:@"CFBundleShortVersionString"];
 
-    version = [infoDict objectForKey:@"CFBundleVersion"];   // Apple
     if ( ! version )
-        version = [infoDict objectForKey:@"NSVersion"];     // GNUstep
+        version = [infoDict objectForKey:@"CFBundleVersion"];   // Apple, 2nd chance
+    if ( ! version )
+        version = [infoDict objectForKey:@"NSVersion"];         // GNUstep
     if ( ! version )
         version = infoVersionNo;
     return version;
@@ -1029,9 +1226,9 @@ typedef enum
     char    date[15];
 
     if ( strlen(compileDate) == 11 )    // Apr  2 2010 -> 2010-04-02
-    {   NSArray *mArray;
-        char    mStr[4];
-        int     m;
+    {   NSArray     *mArray;
+        char        mStr[4];
+        NSUInteger  m;
 
         strncpy(mStr, compileDate, 3); mStr[3] = 0;
         mArray = [NSArray arrayWithObjects:@"Jan", @"Feb", @"Mar", @"Apr", @"May", @"Jun", @"Jul", @"Aug", @"Sep", @"Oct", @"Nov", @"Dec", nil];
@@ -1094,7 +1291,7 @@ typedef enum
  */
 - (void)showWebPage:(id)sender
 {   NSString    *site;
-    int         tag = [sender tag];
+    int         tag = [(NSMenuItem*)sender tag];
 
 	if ([sender isKindOfClass:[NSMatrix class]])
 		tag = [sender selectedTag];
@@ -1190,6 +1387,11 @@ typedef enum
     return transformPanel;
 }
 
+- (void)showVectorizer:sender
+{
+    [[Vectorizer sharedInstance] showPanel:sender];
+}
+
 - (void)showProjectSettingsPanel:sender
 {
     if (!projectSettingsPanel)
@@ -1271,6 +1473,11 @@ typedef enum
 /*
  */
 
+- (NSPanel*)toolPanel
+{
+    return toolPanel;
+}
+
 /* shows the tool panel
  */
 - (void)showToolPanel:sender
@@ -1282,13 +1489,14 @@ typedef enum
  */
 - (void)displayToolPanel:(BOOL)flag
 {
-    if (!toolPanel)
+    if ( !toolPanel )
     {	if (![NSBundle loadModelNamed:@"ToolPanel" owner:self])
             NSLog(@"Cannot load ToolPanel interface file");
         [toolPanel setFrameUsingName:@"ToolPanel"];
         [toolPanel setFrameAutosaveName:@"ToolPanel"];
+        [[NSNotificationCenter defaultCenter] postNotificationName:ToolPanelWillDisplay
+                                                            object:toolPanel userInfo:nil];
     }
-
     [toolPanel setBecomesKeyOnlyIfNeeded:YES];
     [toolPanel setFloatingPanel:YES];
     [toolPanel orderFront:self];
@@ -1312,7 +1520,7 @@ typedef enum
 
         if (sender && sender != self)	// command key pressed
             [[[self currentDocument] window] endEditingFor:nil];	// end editing of text
-        current2DTool = [[matrix selectedCell] tag];
+        current2DTool = [(NSCell*)[matrix selectedCell] tag];
         switch (current2DTool)
         {
             case TOOL2D_ROTATE:		// rotate
@@ -1403,9 +1611,13 @@ typedef enum
         if ([[self documentInWindow:[[self windows] objectAtIndex:count]] window])
             return;	// cancel termination
 
-    /* quit sub process */
-    if ([self respondsToSelector:@selector(quitXyzSubProcess)])
-        [self performSelector:@selector(quitXyzSubProcess)];
+    /* terminate sub processes */
+    for (count=[modules count]-1; count>=0; count--)
+    {  NSBundle     *module = [modules objectAtIndex:count];
+
+        if ( [[[module principalClass] instance] respondsToSelector:@selector(terminate)] )
+            [[[module principalClass] instance] terminate];
+    }
 
     [super terminate:sender];
 }
@@ -1493,11 +1705,11 @@ typedef enum
  * It returns NO if the action is not valid now,
  * otherwise it returns YES.
  */
-- (BOOL)validateMenuItem:(id <NSMenuItem>)anItem
+//- (BOOL)validateMenuItem:(id <NSMenuItem>)anItem
+- (BOOL)validateMenuItem:(NSMenuItem*)anItem
 {   SEL	action = [anItem action];
 
     if ( (action == @selector(import:) ||
-          action == @selector(importASCII:) ||
           action == @selector(revertToSaved:) ||
           action == @selector(save:) ||
           action == @selector(saveAs:)) &&
