@@ -337,22 +337,22 @@
 {   int		i, cnt=[[group list] count];
 
     for (i=0; i<cnt; i++)
-    {   id	g = [[group list] objectAtIndex:i];
+    {   VGraphic    *g = [[group list] objectAtIndex:i];
 
         if ( [g isKindOfClass:[VLine class]] )
             [self writeLine:[g pointWithNum:0] :[g pointWithNum:1] :[g width]];
         else if ( [g isKindOfClass:[VPolyLine class]] )
-            [self exportPolyLine:g];
+            [self exportPolyLine:(VPolyLine*)g];
         else if (  [g isKindOfClass:[VRectangle class]] )
-            [self exportRectangle:g];
+            [self exportRectangle:(VRectangle*)g];
         else if (  [g isKindOfClass:[VArc class]] )
-            [self exportArc:g];
+            [self exportArc:(VArc*)g];
         else if (  [g isKindOfClass:[VCurve class]] )
-            [self exportCurve:g];
+            [self exportCurve:(VCurve*)g];
         else if (  [g isKindOfClass:[VPath class]] )
-            [self exportPath:g];
+            [self exportPath:(VPath*)g];
         else if (  [g isKindOfClass:[VGroup class]] )
-            [self exportGroup:g];
+            [self exportGroup:(VGroup*)g];
         else
             NSLog(@"DXF exportGroup g type not implemented\n");
     }
@@ -382,7 +382,7 @@
     //fileDirectory = [filename stringByDeletingLastPathComponent];
     /* save */
 //    if ([fileManager isWritableFileAtPath:filename])
-    {   id		layerList = [docView layerList];
+    {   NSArray *layerList = [docView layerList];
         int		i;
 
         [self setRes:Prefs_DXFRes];
@@ -399,8 +399,8 @@
             {   int	j, cnt = [[lObj list] count];
 
                 for (j=0; j<cnt; j++)
-                {   id		g = [[lObj list] objectAtIndex:j];
-                    NSColor	*col = [g color];
+                {   VGraphic    *g = [[lObj list] objectAtIndex:j];
+                    NSColor     *col = [g color];
 
                     if ([g respondsToSelector:@selector(fillColor)] && [g width] == 0.0)
                      	col = [(VPath*)g fillColor];
@@ -417,19 +417,19 @@
                     if ( [g isKindOfClass:[VLine class]] )
                         [self writeLine:[g pointWithNum:0] :[g pointWithNum:1] :[g width]];
                     else if ( [g isKindOfClass:[VPolyLine class]] )
-                        [self exportPolyLine:g];
+                        [self exportPolyLine:(VPolyLine*)g];
                     else if (  [g isKindOfClass:[VRectangle class]] )
-                        [self exportRectangle:g];
+                        [self exportRectangle:(VRectangle*)g];
                     else if (  [g isKindOfClass:[VArc class]] )
-                        [self exportArc:g];
+                        [self exportArc:(VArc*)g];
                     else if (  [g isKindOfClass:[VCurve class]] )
-                        [self exportCurve:g];
+                        [self exportCurve:(VCurve*)g];
                     else if (  [g isKindOfClass:[VPath class]] )
-                        [self exportPath:g];
+                        [self exportPath:(VPath*)g];
                     else if (  [g isKindOfClass:[VGroup class]] )
-                        [self exportGroup:g];
+                        [self exportGroup:(VGroup*)g];
                     else if (  [g isKindOfClass:[VText class]] )
-                        [self exportText:g];
+                        [self exportText:(VText*)g];
                     else NSLog(@"DXF exportToFile g type not implemented\n");
                 }
             }
