@@ -1,11 +1,15 @@
 /* Document.m
  * Cenon document class
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2013 by vhf interservice GmbH
  * Author: Georg Fleischmann
  *
  * Created:  1996-02-09
- * Modified: 2010-11-26 (-saveTiff text work)
+ * Modified: 2013-06-29 (+newFromFile: scale to saved scaleFactor)
+ *           2013-02-05 (-save  Cenon Version saved with file)
+ *           2013-01-02 (-printDocument: remove paper margins from view, which are not ignored any more since OS 10.8)
+ *           2011-03-30 (-exportLock, -setExportLock:)
+ *           2010-11-26 (-saveTiff text work)
  *           2010-06-30 (-saveTiff added)
  *           2008-10-21 (clean-up)
  *           2008-07-24 (document units, update units)
@@ -64,8 +68,9 @@
 @interface Document(PrivateMethods)
 - initWindow;
 - (void)initializePrintInfo;
-- (BOOL)setWindowSize:(NSSize)size;
-- (BOOL)loadOutputListsFromFile:(const char*)fileName;
+//- (BOOL)setWindowSize:(NSSize)size;
+- (void)scale:(NSSize)scaleSize withCenter:(NSPoint)center;
+//- (BOOL)loadOutputListsFromFile:(const char*)fileName;
 - (BOOL)saveTiff:(NSString*)filename;
 - (BOOL)saveEPS:(NSString*)filename;
 - (BOOL)saveGerber:(NSString*)filename;
@@ -183,7 +188,7 @@ static NSMutableArray *zoneList = nil;
 {
     [super init];
 
-    baseUnit = -1;  // use unit from Preferences
+    baseUnit = UNIT_NONE;   // use unit from Preferences
 
     //[self registerForServicesMenu];
     return self;
@@ -193,7 +198,7 @@ static NSMutableArray *zoneList = nil;
 {   NSRect	frameRect;
     DocView	*view;
 
-    [window setDelegate:self];
+    [window setDelegate:(id)self];
     [scrollView setHasHorizontalScroller:YES];
     [scrollView setHasVerticalScroller:YES];
 
@@ -248,7 +253,7 @@ static NSMutableArray *zoneList = nil;
         if (isDirectory)
         {
             fileDirectory = fileName;
-            fileName = [fileDirectory stringByAppendingPathComponent:DOCUMENT_NAME];
+            fileName   = [fileDirectory stringByAppendingPathComponent:DOCUMENT_NAME];
             outputFile = [fileDirectory stringByAppendingPathComponent:OUTPUT_NAME];
         }
         else if ([[fileName lastPathComponent] isEqual:DOCUMENT_NAME])
@@ -269,6 +274,7 @@ static NSMutableArray *zoneList = nil;
             [newDocument init];
 
             plist = [NSDictionary dictionaryWithContentsOfFile:fileName];
+            /* PrintInfo */
             if ( [[plist objectForKey:@"PrintInfo"] isKindOfClass:[NSDictionary class]] )
                 newDocument->printInfo = printInfoFromPropertyList([plist objectForKey:@"PrintInfo"], [self zone]);
             else
@@ -282,6 +288,8 @@ static NSMutableArray *zoneList = nil;
             newDocument->docCopyright = [[plist objectForKey:@"Copyright"]  retain];
             newDocument->docComment   = [[plist objectForKey:@"Comment"]    retain];
 
+            newDocument->exportLock   = ([plist objectForKey:@"exportLock"]) ? YES : NO;
+
             /* Project Settings */
             if ([plist objectForKey:@"DocSettings"])
                 newDocument->docSettingsDict = [[plist objectForKey:@"DocSettings"] mutableCopy];
@@ -291,7 +299,7 @@ static NSMutableArray *zoneList = nil;
             directoryContainingDocument = fileDirectory ? fileDirectory
                                                         : [fileName stringByDeletingLastPathComponent];
 
-            [newDocument->window setDelegate:newDocument];
+            [newDocument->window setDelegate:(id)newDocument];
             [newDocument->scrollView setHasHorizontalScroller:YES];
             [newDocument->scrollView setHasVerticalScroller:YES];
 
@@ -336,12 +344,14 @@ static NSMutableArray *zoneList = nil;
 #endif
 
             /* workaround: something (NSWindow according documentation) doesn't work with size>10000.0 */
+            // #if VHF_IS_DOUBLE == 0
             fRect = rectFromPropertyList([plist objectForKey:@"ViewSize"]);
             if ( fRect.size.width > 10000.0 )
                 fRect.size.width = 10000.0;
             if ( fRect.size.height > 10000.0 )
                 fRect.size.height = 10000.0;
             view = [[DocView allocWithZone:[self zone]] initWithFrame:fRect];
+            // #endif
 
             [view setDocument:newDocument];
             newDocument->magazineIndex = [plist intForKey:@"MagazineIndex"];
@@ -394,6 +404,14 @@ static NSMutableArray *zoneList = nil;
 
             newDocument->haveSavedDocument = YES;
 
+            /* zoom */
+            {   NSMutableDictionary *plist = [[(App*)NSApp currentDocument] docSettingsDict];
+                float               s;
+
+                if ( (s = [plist floatForKey:@"scaleFactor"]) && s != 1.0 )
+                    [[newDocument scrollView] scaleTo:s];
+            }
+
             [[NSNotificationCenter defaultCenter] postNotificationName:DocumentDidOpen
                                                                 object:view];
         }
@@ -409,9 +427,9 @@ static NSMutableArray *zoneList = nil;
     NSDictionary	*plist = nil;
     NSString		*fileDirectory = nil;
     NSFileManager	*fileManager = [NSFileManager defaultManager];
-    BOOL		isDirectory = NO;
-    NSArray		*layerList;
-    int			l, lCnt;
+    BOOL            isDirectory = NO;
+    NSArray         *layerList;
+    int             l, lCnt;
 
     if ([fileManager fileExistsAtPath:fileName isDirectory:&isDirectory])
     {
@@ -465,10 +483,10 @@ static NSMutableArray *zoneList = nil;
     [(App*)NSApp setCurrentDocument:newDocument];
     [newDocument setName:nil andDirectory:nil];
 
-    win = [newDocument window];
+    win   = [newDocument window];
     sView = [newDocument scrollView];
 
-    [win setDelegate:newDocument];
+    [win setDelegate:(id)newDocument];
     [sView setHasHorizontalScroller:YES];
     [sView setHasVerticalScroller:YES];
 
@@ -487,6 +505,9 @@ static NSMutableArray *zoneList = nil;
     //newDocument->printInfo = [[NSPrintInfo allocWithZone:[newDocument zone]] init];
     [newDocument initializePrintInfo];
 
+    /* Prefs_RotateObjects */
+    /* rotate list with given Degree */
+
     /* calculate size of view */
     frameRect = [DocView boundsOfArray:list];	/* get bounds of list */
     frameRect.size.width  += frameRect.origin.x + MMToInternal(20.0);
@@ -494,12 +515,14 @@ static NSMutableArray *zoneList = nil;
     frameRect.origin.x = 0.0;
     frameRect.origin.y = 0.0;
 
-    /* workaround: something doesn't work with size>10000.0 */
+    /* workaround: something doesn't work with size > 10000.0 */
+    // #if VHF_IS_DOUBLE == 0
     fRect = frameRect;
     if ( fRect.size.width > 10000.0 )
         fRect.size.width = 10000.0;
     if ( fRect.size.height > 10000.0 )
         fRect.size.height = 10000.0;
+    // #endif
 
     /* set view */
     view = [[[DocView allocWithZone:[newDocument zone]] initWithFrame:fRect] initView];
@@ -611,24 +634,29 @@ static NSMutableArray *zoneList = nil;
 #endif
 
 - (void)scale:(float)x :(float)y withCenter:(NSPoint)center
+{
+    NSLog(@"[document scale:x :y ...] is deprecated, use [document scale:NSSize ...");
+    [self scale:NSMakeSize(x, y) withCenter:center];
+}
+- (void)scale:(NSSize)scaleSize withCenter:(NSPoint)center
 {   NSPoint	p = {0, 0};
     DocView	*view = [scrollView documentView];
     NSRect	oldFrame = [view frame], bRect;
 
     /* scale doc view and cache window */
-    [view scaleUnitSquareToSize:NSMakeSize(x, y)];
+    [view scaleUnitSquareToSize:scaleSize];
 
     [window disableFlushWindow];
 
     /* resize doc view and cache window (old size * scaleFactor) */
-    [view setFrameSize:NSMakeSize(NSWidth(oldFrame)*x, NSHeight(oldFrame)*y)];	// resize view
+    [view setFrameSize:NSMakeSize(NSWidth(oldFrame)*scaleSize.width, NSHeight(oldFrame)*scaleSize.height)];	// resize
 
     oldFrame = [window frame];
     oldFrame = [view convertRect:oldFrame fromView:nil];
     bRect = [view bounds];
     p = center;
-    if ((p.x -= oldFrame.size.width /2.0)<0.0) p.x = 0.0;
-    if ((p.y -= oldFrame.size.height/2.0)<0.0) p.y = 0.0;
+    if ((p.x -= oldFrame.size.width /2.0) < 0.0) p.x = 0.0;
+    if ((p.y -= oldFrame.size.height/2.0) < 0.0) p.y = 0.0;
     if (p.x > bRect.size.width  - oldFrame.size.width)  p.x = bRect.size.width  - oldFrame.size.width;
     if (p.y > bRect.size.height - oldFrame.size.height) p.y = bRect.size.height - oldFrame.size.height;
     [view scrollPoint:p];
@@ -689,12 +717,20 @@ static NSMutableArray *zoneList = nil;
     dirty = flag;
     [window setDocumentEdited:flag]; 
 }
-
 - (BOOL)dirty
 {
     return dirty;
 }
 
+- (void)setExportLock:(BOOL)flag
+{
+    exportLock = flag;
+}
+- (BOOL)exportLock
+{
+    return exportLock;
+}
+
 /* Services menu registrar */
 - (void)registerForServicesMenu
 {
@@ -774,11 +810,11 @@ static NSMutableArray *zoneList = nil;
 - (CenonUnit)baseUnitFlat           { return baseUnit; }
 - (CenonUnit)baseUnit
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
-    return (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+    return (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 }
 - (float)convertToUnit:(float)iValue
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
-    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+    CenonUnit       unit = (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 
     switch ( unit )
     {
@@ -791,7 +827,7 @@ static NSMutableArray *zoneList = nil;
 }
 - (float)convertFrUnit:(float)uValue
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
-    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+    CenonUnit       unit = (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 
     switch ( unit )
     {
@@ -804,7 +840,7 @@ static NSMutableArray *zoneList = nil;
 }
 - (float)convertMMToUnit:(float)mmValue
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
-    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+    CenonUnit       unit = (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 
     switch ( unit )
     {
@@ -817,7 +853,7 @@ static NSMutableArray *zoneList = nil;
 }
 - (float)convertUnitToMM:(float)uValue
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
-    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+    CenonUnit       unit = (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 
     switch ( unit )
     {
@@ -841,7 +877,7 @@ static NSMutableArray *zoneList = nil;
  * ???kb NSString - This now returns an autorelease... I tried to fix up
  * all users of this, but it may kill something!
  */
-- (NSString *)filename
+- (NSString*)filename
 {   NSString	*returnString = @"";
 
     if (!directory && !name)
@@ -850,13 +886,11 @@ static NSMutableArray *zoneList = nil;
         returnString = [directory stringByAppendingPathComponent:name];
     return returnString;
 }
-
-- (NSString *)directory
+- (NSString*)directory
 {
     return directory;
 }
-
-- (NSString *)name
+- (NSString*)name
 {
     return name;
 }
@@ -1033,7 +1067,8 @@ static NSMutableArray *zoneList = nil;
  * 3. move file name (old) file to backup (fileName.cenon -> fileName.cenon~)
  * 4. move temporary file to file name (#fileName.cenon -> fileName.cenon)
  *
- * modified: 2007-07-24 (save baseUnit and project settings)
+ * modified: 2013-07-04 (save Cenon version to creator)
+ *           2007-07-24 (save baseUnit and project settings)
  */
 - (BOOL)save
 {   BOOL                savedOk = NO;
@@ -1066,7 +1101,8 @@ static NSMutableArray *zoneList = nil;
     pool = [NSAutoreleasePool new];
     /* save */
     if ([fileManager isWritableFileAtPath:fileDirectory])
-    {   NSArchiver	*archiver;
+    {   NSArchiver      *archiver;
+        NSString        *version  = [(App*)NSApp version];
 
         NS_DURING
             [window makeFirstResponder:[self documentView]];
@@ -1083,7 +1119,8 @@ static NSMutableArray *zoneList = nil;
             [fileContents setInt:[window hasCoordDisplay] forKey:@"hasCoordBox"];
             [fileContents setInt:magazineIndex forKey:@"MagazineIndex"];
             //[fileContents setObject:propertyListFromInt([VGraphic currentGraphicIdentifier]) forKey:@"GraphicIdentifier"];
-            [fileContents setObject:@"Cenon" forKey:@"Creator"];
+            //[fileContents setObject:@"Cenon" forKey:@"Creator"];
+            [fileContents setObject:[NSString stringWithFormat:@"Cenon %@", version] forKey:@"Creator"];
             [fileContents setObject:[[NSCalendarDate date] descriptionWithCalendarFormat:@"%Y-%m-%d %H:%M"] forKey:@"CreationDate"];
             /* Project Info */
             if (docVersion)
@@ -1094,6 +1131,8 @@ static NSMutableArray *zoneList = nil;
                 [fileContents setObject:docCopyright    forKey:@"Copyright"];
             if (docComment)
                 [fileContents setObject:docComment      forKey:@"Comment"];
+            if (exportLock)
+                [fileContents setObject:@"Y"            forKey:@"exportLock"];
             /* Project Settings */
             if (baseUnit != -1)
                 [fileContents setObject:[NSString stringWithFormat:@"%d", baseUnit] forKey:@"Unit"];
@@ -1112,7 +1151,7 @@ static NSMutableArray *zoneList = nil;
             /* ASCII PropertyList... it just doesn't work on Mac OS X,
              * so we use the class from the GNUstep project so long
              */
-            if ( Prefs_OSPropertyList ) // ASCII PropertyList (most human readable + half the size of XML)
+            if ( Prefs_OSPropertyList ) // ASCII PropertyList (best human readable, half the size of XML)
             {
                 if ( [NSPropertyListSerialization propertyList:fileContents
                                               isValidForFormat:NSPropertyListOpenStepFormat] )
@@ -1124,7 +1163,7 @@ static NSMutableArray *zoneList = nil;
                                                                       format:NSPropertyListOpenStepFormat
                                                             errorDescription:&error];
                     if (error)
-                    {   NSLog(error); [error release]; }
+                        { NSLog(@"%@", error); [error release]; }
                     savedOk = [data writeToFile:filename atomically:YES];
 
                     /* Apple stuff doesn't work:
@@ -1147,14 +1186,14 @@ static NSMutableArray *zoneList = nil;
                                                                       format:NSPropertyListOpenStepFormat
                                                             errorDescription:&error];
                     if (error)
-                    {   NSLog(error); [error release]; }
+                    { NSLog(@"%@", error); [error release]; }
                     savedOk = [data writeToFile:filename atomically:YES];
                 }
             }
             else	// use native PropertyList format
 #endif
-                /* FIXME: converting dictionary to string needs a lot lot of memory !! */
-                savedOk = [fileContents writeToFile:filename atomically:YES];
+            /* FIXME: converting dictionary to string needs a lot lot of memory !! */
+            savedOk = [fileContents writeToFile:filename atomically:YES];
             haveSavedDocument = savedOk;
 
             if ( fileDirectory && savedOk )
@@ -1204,6 +1243,10 @@ static NSMutableArray *zoneList = nil;
     return YES;
 }
 
+/* save or export
+ * return: YES = saved succesfully
+ *         NO  = save failed or just exporded
+ */
 - (BOOL)saveAs:(id <NSMenuItem>)invokingMenuItem
 {   NSSavePanel *savepanel = [(App*)NSApp saveAsPanel];
 
@@ -1215,11 +1258,15 @@ static NSMutableArray *zoneList = nil;
     {   NSString *path = [savepanel filename];
 
         if ([[path pathExtension] isEqual:DOCUMENT_EXT])
-	{
-            [self setName:path];
+        {   [self setName:path];
             return [self save];
         }
-        else if ([[path pathExtension] isEqual:EPS_EXT])
+        if ( exportLock )   // this file is not supposed to be exported
+        {
+            NSRunAlertPanel(SAVE_TITLE, EXPORTLOCK_STRING, nil, nil, nil);
+            return NO;
+        }
+        if ([[path pathExtension] isEqual:EPS_EXT])
             [self saveEPS:path];
         else if ([[path pathExtension] isEqual:GERBER_EXT])
             [self saveGerber:path];
@@ -1373,7 +1420,7 @@ static NSMutableArray *zoneList = nil;
                 {   VGraphic    *gr = [llist objectAtIndex:i];
 
                     [gr moveBy:movePt];
-                    [gr drawWithPrincipal:nil];
+                    [gr drawWithPrincipal:nil/*[self documentView]*/]; // nil because our view don't answer to mustDrawPale
                     [gr moveBy:moveBackPt];
                 }
             }
@@ -1724,7 +1771,7 @@ static NSMutableArray *zoneList = nil;
     [docView setSeparationColor:sepColor];
     [sepColor release];
     /* op is allready currentOperation !!!! */
-    [op setShowPanels:NO]; // else the user get a second printpanel
+    [op setShowsPrintPanel:NO]; // else the user get a second printpanel
     [op runOperation];
     [pool release];
 
@@ -1734,7 +1781,7 @@ static NSMutableArray *zoneList = nil;
     ml = [[layerList objectAtIndex:0] list]; // set Color string
     [[ml objectAtIndex:[ml count]-1] setString:[NSString stringWithFormat:@"Magenta  "]];
     sop = [NSPrintOperation printOperationWithView:docView printInfo:pi];
-    [sop setShowPanels:NO];
+    [sop setShowsPrintPanel:NO];
     spi = [sop printInfo]; // from here we add a "_c" to the NSPrintSavePath string in printInfo dictionary
     piDict = [spi dictionary];
     if ([piDict objectForKey:NSPrintSavePath])
@@ -1753,7 +1800,7 @@ static NSMutableArray *zoneList = nil;
     ml = [[layerList objectAtIndex:0] list]; // set Color string
     [[ml objectAtIndex:[ml count]-1] setString:[NSString stringWithFormat:@"Yellow  "]];
     sop = [NSPrintOperation printOperationWithView:docView printInfo:pi];
-    [sop setShowPanels:NO];
+    [sop setShowsPrintPanel:NO];
     spi = [sop printInfo];
     piDict = [spi dictionary];
     if ([piDict objectForKey:NSPrintSavePath])
@@ -1772,7 +1819,7 @@ static NSMutableArray *zoneList = nil;
     ml = [[layerList objectAtIndex:0] list]; // set Color string
     [[ml objectAtIndex:[ml count]-1] setString:[NSString stringWithFormat:@"Black  "]];
     sop = [NSPrintOperation printOperationWithView:docView printInfo:pi];
-    [sop setShowPanels:NO];
+    [sop setShowsPrintPanel:NO];
     spi = [sop printInfo];
     piDict = [spi dictionary];
     if ([piDict objectForKey:NSPrintSavePath])
@@ -1833,15 +1880,15 @@ static NSMutableArray *zoneList = nil;
 }
 
 /*
- * modified: 2005-09-01
+ * modified: 2013-01-02 (10.8: remove paper margins by adjusting print view)
  */
 - (void)printDocument:sender
 {   NSPrintOperation    *op;
     NSPrintPanel        *pp;
     NSPrintInfo         *pi;
     DocView             *docView = [self documentView];
-    NSRect              visibleRect, oldFrame;
-    float               width, height, scale = [docView scaleFactor];
+    NSRect              visibleRect, oldFrame, pageBounds = [printInfo imageablePageBounds];
+    VFloat              width, height, scale = [docView scaleFactor];
 
     /* set scale to 1.0 */
     scale = [docView scaleFactor];
@@ -1849,15 +1896,32 @@ static NSMutableArray *zoneList = nil;
     oldFrame = [docView frame];
     width  = oldFrame.size.width  / scale;
     height = oldFrame.size.height / scale;
+#ifdef __APPLE__
+    /* Apple 10.8: the PrintInfo margins are 0, but there are paper margins PMPaperMargins()
+     * we could create a custom paper with PMPaperCreateCustom() or adjust our print view
+     */
+    if ( NSAppKitVersionNumber > NSAppKitVersionNumber10_7 )    // Workaround Max OS 10.8: make view smaller by paper margins
+    {   width  -= ([printInfo paperSize].width  - pageBounds.size.width);
+        height -= ([printInfo paperSize].height - pageBounds.size.height);
+    }
+#endif
     [window disableFlushWindow];
     [docView scaleUnitSquareToSize:NSMakeSize(1.0/scale, 1.0/scale)];
     [docView setFrameSize:NSMakeSize(width, height)];
+    /* Workaround Mac OS 10.8: we have to translate printing by the page margin to remove the margin */
+#ifdef __APPLE__
+    if ( NSAppKitVersionNumber > NSAppKitVersionNumber10_7 )
+        [docView translateOriginToPoint:NSMakePoint(-pageBounds.origin.x, -pageBounds.origin.y)];
+#endif
+
+    //[printInfo setHorizontalPagination:NSClipPagination];
+    //[printInfo setVerticalPagination:NSClipPagination];
 
     /* multi page document */
     if ([docView isMultiPage])
     {
         op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
-        [op setShowPanels:YES];
+        [op setShowsPrintPanel:YES];
         [op runOperation];
 
         // FIXME: restore enabled/disabled pages
@@ -1875,21 +1939,22 @@ static NSMutableArray *zoneList = nil;
             [printInfo setPrinter:[NSPrintInfo defaultPrinter]];	// set default printer for the first time
 
         op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
-        [op setShowPanels:YES];
+        [op setShowsPrintPanel:YES];
         [NSPrintOperation setCurrentOperation:op];
 
         pp = [NSPrintPanel printPanel];
         [pp setAccessoryView:[[(App*)NSApp printPanelAccessory] retain]];
+        //[pp addAccessoryController:[[(App*)NSApp printPanelAccessory] retain]];
         [op setPrintPanel:pp];
 
         if ([pp runModal] == NSCancelButton)
         {
-            [op setShowPanels:NO];	// else the user get a second printpanel
-            [op runOperation];	// this will end the operation !
+            [op setShowsPrintPanel:NO]; // else the user get a second printpanel
+            //[op runOperation];	// this will end the operation ! (this deletes a saved file from the prev. operation !)
         }
         else if (![[(App*)NSApp ppaRadio] selectedRow])		// composite
         {
-            [op setShowPanels:NO];	// else the user get a second printpanel
+            [op setShowsPrintPanel:NO]; // else the user get a second printpanel
             [op runOperation];
         }
         else							// color separation
@@ -1901,7 +1966,13 @@ static NSMutableArray *zoneList = nil;
         printInfo = [pi retain];	// remember printInfo values for the next time ! (2005-05-06)
 #endif
     }
+    [op cleanUpOperation];
 
+#ifdef __APPLE__
+    /* translate back */
+    if ( NSAppKitVersionNumber > NSAppKitVersionNumber10_7 )
+        [docView translateOriginToPoint:NSMakePoint(pageBounds.origin.x, pageBounds.origin.y)];
+#endif
     /* scale view back to old scale */
     [docView scaleUnitSquareToSize:NSMakeSize(scale, scale)];
     [docView setFrameSize:oldFrame.size];
@@ -1920,8 +1991,7 @@ static NSMutableArray *zoneList = nil;
  * the document is now dirty (by performing the dirty method).
  */
 - (void)changeLayout:sender
-{
-    //NSRect		frame;
+{   //NSRect		frame;
     //float		lm, rm, tm, bm;
     //NSSize		paperSize;
     NSPrintInfo		*tempPrintInfo;
@@ -1977,7 +2047,7 @@ static NSMutableArray *zoneList = nil;
         switch (NSRunAlertPanel(CLOSEWINDOW_STRING, SAVECHANGES_STRING, SAVE_STRING, DONTSAVE_STRING, CANCEL_STRING, [sender title]))
         {
             case NSAlertDefaultReturn:      // save
-                [self save];
+                [self save:nil];            // brings up a Save Panel, in case doc hasn't been saved before
                 break;
             case NSAlertAlternateReturn:    // dont save
                 break;
