@@ -1,7 +1,7 @@
 /* GerberExportSub.m
  * Sub class of Gerber export
  *
- * Copyright (C) 2002-2008 by vhf interservice GmbH
+ * Copyright (C) 2002-2012 by vhf interservice GmbH
  * Author:   Ilonka Fleischmann
  *
  * created:  2002-04-25
@@ -252,7 +252,7 @@
     /* get all bounds of polygons and */
     /* get all indexes of polygons */
     for (j=0; j<cnt; j++)
-    {   id	g = [[group list] objectAtIndex:j];
+    {   VGraphic    *g = [[group list] objectAtIndex:j];
 
         if ([g isKindOfClass:[VPath class]])
         {
@@ -265,9 +265,10 @@
     {
         /* count how many times each path is inside all others */
         for (j=0; j < polyCnt; j++)
-        {   id      list = [[[group list] objectAtIndex:polygonIs[j]] list], gr;
-            NSPoint	pt;
-            int     inside = 0;
+        {   NSArray     *list = [[[group list] objectAtIndex:polygonIs[j]] list];
+            VGraphic    *gr;
+            NSPoint     pt;
+            int         inside = 0;
 
             if (![list count])
                 continue;   // empty group
@@ -333,8 +334,8 @@
             continue;
         }
         for (j=0; j<cnt; j++)
-        {   id	g = [[group list] objectAtIndex:j];
-            float	w = [g width];
+        {   VGraphic    *g = [[group list] objectAtIndex:j];
+            float       w = [g width];
 
             if ( !(!k && ![g isKindOfClass:[VPath class]] && !valueInArray(j, insideIs, insideCnt)) &&
                  /*!(k == 1 && [g isKindOfClass:[VPath class]]) &&*/
@@ -357,19 +358,19 @@
             if ( [g isKindOfClass:[VLine class]] )
                 [self writeLine:[g pointWithNum:0] :[g pointWithNum:1]];
             else if ( [g isKindOfClass:[VPolyLine class]] )
-                [self exportPolyLine:g];
+                [self exportPolyLine:(VPolyLine*)g];
             else if (  [g isKindOfClass:[VRectangle class]] )
-                [self exportRectangle:g];
+                [self exportRectangle:(VRectangle*)g];
             else if (  [g isKindOfClass:[VArc class]] )
-                [self exportArc:g];
+                [self exportArc:(VArc*)g];
             else if (  [g isKindOfClass:[VCurve class]] )
-                [self exportCurve:g];
+                [self exportCurve:(VCurve*)g];
             else if (  [g isKindOfClass:[VPath class]] )
-                NSLog(@"GerberExportSub.m: exportToFile, exportPath\n"); /*[self exportPath:g]*/
+                NSLog(@"GerberExportSub.m: exportToFile, exportPath\n"); /*[self exportPath:(VPath*)g]*/
             else if (  [g isKindOfClass:[VGroup class]] )
-                [self exportGroup:g];
+                [self exportGroup:(VGroup*)g];
             else if (  [g isKindOfClass:[VText class]] )
-                 [self exportText:g];
+                 [self exportText:(VText*)g];
         }
     }
 /*
@@ -432,8 +433,8 @@
     //fileDirectory = [filename stringByDeletingLastPathComponent];
     /* save */
 //    if ([fileManager isWritableFileAtPath:filename])
-    {   id		layerList = [docView layerList];
-        int		i, lcnt = [layerList count];
+    {   NSArray         *layerList = [docView layerList];
+        int             i, lcnt = [layerList count];
         NSMutableArray	*gList = [NSMutableArray array];
 
 
@@ -540,8 +541,8 @@
                     continue;
                 }
                 for (j=0; j<cnt; j++)
-                {   id	g = [gList objectAtIndex:j];
-                    float	w = [g width];
+                {   VGraphic    *g = [gList objectAtIndex:j];
+                    float       w = [g width];
 
                     if ( !(!k && ![g isKindOfClass:[VPath class]] && !valueInArray(j, insideIs, insideCnt)) &&
                          /*!(k == 1 && [g isKindOfClass:[VPath class]]) &&*/
@@ -564,19 +565,19 @@
                     if ( [g isKindOfClass:[VLine class]] )
                         [self writeLine:[g pointWithNum:0] :[g pointWithNum:1]];
                     else if ( [g isKindOfClass:[VPolyLine class]] )
-                        [self exportPolyLine:g];
+                        [self exportPolyLine:(VPolyLine*)g];
                     else if (  [g isKindOfClass:[VRectangle class]] )
-                        [self exportRectangle:g];
+                        [self exportRectangle:(VRectangle*)g];
                     else if (  [g isKindOfClass:[VArc class]] )
-                        [self exportArc:g];
+                        [self exportArc:(VArc*)g];
                     else if (  [g isKindOfClass:[VCurve class]] )
-                        [self exportCurve:g];
+                        [self exportCurve:(VCurve*)g];
                     else if (  [g isKindOfClass:[VPath class]] )
-                        NSLog(@"GerberExportSub.m: exportToFile, exportPath\n");/*[self exportPath:g];*/
+                        NSLog(@"GerberExportSub.m: exportToFile, exportPath\n");/*[self exportPath:(VPath*)g];*/
                               else if (  [g isKindOfClass:[VGroup class]] )
-                              [self exportGroup:g];
+                              [self exportGroup:(VGroup*)g];
                               else if (  [g isKindOfClass:[VText class]] )
-                              [self exportText:g];
+                              [self exportText:(VText*)g];
                 }
             }
         }
