@@ -56,13 +56,13 @@
     /* beginning with the first graphic
      */
     for ( c=0; c < (int)[list count]-1; c++ )
-    {	id	cg = [list objectAtIndex:c];
-        NSColor	*cColor = [[cg color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-        NSColor	*cFillColor=nil, *oFillColor=nil, *cEndColor=nil, *oEndColor=nil;
-        NSRect	cBounds;
-        float	cw = [cg width], ow;
-        int	cf = 0, of = 0;
-        int	ic=0, icCnt = 1;
+    {	VGraphic    *cg = [list objectAtIndex:c];
+        NSColor     *cColor = [[cg color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+        NSColor     *cFillColor=nil, *oFillColor=nil, *cEndColor=nil, *oEndColor=nil;
+        NSRect      cBounds;
+        float       cw = [cg width], ow;
+        int         cf = 0, of = 0;
+        int         ic=0, icCnt = 1;
 
         if ( [cg isKindOfClass:[VText class]] || [cg isKindOfClass:[VTextPath class]] )
             continue;	// skip text	// FIXME: convert text to path
@@ -83,13 +83,13 @@
         }
 
         if ([cg isKindOfClass:[VGroup class]])
-            icCnt = [cg countRecursive]; // [[cg list] count];
+            icCnt = [(VGroup*)cg countRecursive]; // [[cg list] count];
 
         for (ic=0; ic < icCnt; ic++)
-        {   id	icg = cg;
+        {   VGraphic    *icg = cg;
 
             if ([cg isKindOfClass:[VGroup class]])
-                icg = [cg recursiveObjectAtIndex:ic]; // [[cg list] objectAtIndex:ic];
+                icg = [(VGroup*)cg recursiveObjectAtIndex:ic]; // [[cg list] objectAtIndex:ic];
 
             if ( [icg isKindOfClass:[VText class]] || [icg isKindOfClass:[VTextPath class]] )
                 continue;	// skip text	// FIXME: convert text to path
@@ -97,8 +97,8 @@
             if ([icg respondsToSelector:@selector(fillColor)])
             {
                 cf = [icg filled];
-                cFillColor = [[icg fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-                cEndColor  = [[icg endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                cFillColor = [[(VPath*)icg fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                cEndColor  = [[(VPath*)icg endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
             }
 
             cBounds = [icg bounds];
@@ -108,9 +108,9 @@
              * else: the hidden areas of the hidden graphic will be removed or the hole hiddenG
              */
             for ( o=c+1; o < (int)[list count]; o++ )
-            {   id	og = [list objectAtIndex:o];
-                NSRect	oBounds;
-                int	io=0, ioCnt = 1;
+            {   VGraphic    *og = [list objectAtIndex:o];
+                NSRect      oBounds;
+                int         io=0, ioCnt = 1;
 
                 if ( [og isKindOfClass:[VText class]] || [og isKindOfClass:[VTextPath class]] )
                     continue;	// skip text	// FIXME: convert text to path
@@ -122,13 +122,13 @@
                 } // konvertiert text in pfad ist dann aber definitiv ein Pfad !!!???
 */
                 if ([og isKindOfClass:[VGroup class]])
-                    ioCnt = [og countRecursive]; // [[og list] count];
+                    ioCnt = [(VGroup*)og countRecursive]; // [[og list] count];
 
                 for (io=0; io < ioCnt; io++)
-                {   id	iog = og;
+                {   VGraphic	*iog = og;
 
                     if ([og isKindOfClass:[VGroup class]])
-                        iog = [og recursiveObjectAtIndex:io]; // [[og list] objectAtIndex:io];
+                        iog = [(VGroup*)og recursiveObjectAtIndex:io]; // [[og list] objectAtIndex:io];
 
                     if ( [iog isKindOfClass:[VText class]] || [iog isKindOfClass:[VTextPath class]] )
                         continue;	// skip text	// FIXME: convert text to path
@@ -144,8 +144,8 @@
                         if ([iog respondsToSelector:@selector(fillColor)])
                         {
                             of = [iog filled];
-                            oFillColor = [[iog fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-                            oEndColor  = [[iog endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                            oFillColor = [[(VPath*)iog fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                            oEndColor  = [[(VPath*)iog endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
                         }
 
                         if (!(((((cw && cw == ow) || (!cw && !ow && !cf && !of)) && [cColor isEqual:oColor]) ||
@@ -158,7 +158,7 @@
                             {	/* if the hidden graphic is in the iog we remove the hiddenG */
 
                                 if ([cg isKindOfClass:[VGroup class]])
-                                {   [cg recursiveRemoveObjectAtIndex:ic]; // [[cg list] removeObjectAtIndex:ic];
+                                {   [(VGroup*)cg recursiveRemoveObjectAtIndex:ic]; // [[cg list] removeObjectAtIndex:ic];
                                     icCnt--;
                                     ic--;
                                 }
@@ -170,8 +170,8 @@
                                 break;
                             }
                             if ([cg isKindOfClass:[VGroup class]])
-                            {   [cg recursiveRemoveObjectAtIndex:ic];      // [[cg list] removeObjectAtIndex:ic];
-                                [cg recursiveInsertObject:icg atIndex:ic]; // [[cg list] insertObject:icg atIndex:ic];
+                            {   [(VGroup*)cg recursiveRemoveObjectAtIndex:ic];      // [[cg list] removeObjectAtIndex:ic];
+                                [(VGroup*)cg recursiveInsertObject:icg atIndex:ic]; // [[cg list] insertObject:icg atIndex:ic];
                             }
                             else
                             {   [list removeObjectAtIndex:c];
@@ -183,7 +183,7 @@
                 }
             }
         }
-        if ([cg isKindOfClass:[VGroup class]] && ![cg countRecursive])
+        if ([cg isKindOfClass:[VGroup class]] && ![(VGroup*)cg countRecursive])
         {
             [list removeObjectAtIndex:c];
             c--;
@@ -201,13 +201,13 @@
      */
     for ( c=0 ; c < (int)[list count] ; c++ )
     {   NSAutoreleasePool	*pool;
-	id			cg = [list objectAtIndex:c];
-        NSColor			*cColor;
-        NSColor			*cFillColor=nil, *oFillColor=nil, *cEndColor=nil, *oEndColor=nil;
-        float			cw = [cg width], ow;
-        int			cf = 0, of = 0;
-        NSRect			cBounds;
-        int			ic=0, icCnt = 1;
+        VGraphic            *cg = [list objectAtIndex:c];
+        NSColor             *cColor;
+        NSColor             *cFillColor=nil, *oFillColor=nil, *cEndColor=nil, *oEndColor=nil;
+        float               cw = [cg width], ow;
+        int                 cf = 0, of = 0;
+        NSRect              cBounds;
+        int                 ic=0, icCnt = 1;
 
         /* lines, curves, full/unfilled arcs or unfilled paths can't be united */
         if ( [cg isKindOfClass:[VLine class]] || [cg isKindOfClass:[VCurve class]] ||
@@ -229,13 +229,13 @@
         }
 
         if ([cg isKindOfClass:[VGroup class]])
-            icCnt = [cg countRecursive]; // [[cg list] count];
+            icCnt = [(VGroup*)cg countRecursive]; // [[cg list] count];
 
         for (ic=0; ic < icCnt; ic++)
         {   id	icg = cg;
 
             if ([cg isKindOfClass:[VGroup class]])
-            {   icg = [cg recursiveObjectAtIndex:ic]; // [[cg list] objectAtIndex:ic];
+            {   icg = [(VGroup*)cg recursiveObjectAtIndex:ic]; // [[cg list] objectAtIndex:ic];
                 prevCnt = icCnt;
             }
             else
@@ -279,7 +279,7 @@
                 }
 
                 for (io=0; io < ioCnt; io++)
-                {   id	iog = og;
+                {   VGraphic	*iog = og;
 
                     if ([og isKindOfClass:[VGroup class]])
                         iog = [og recursiveObjectAtIndex:io]; // [[og list] objectAtIndex:io];
@@ -311,8 +311,8 @@
                         {
                             if ([cg isKindOfClass:[VGroup class]])
                             {
-                                [cg recursiveRemoveObjectAtIndex:ic];
-                                [cg recursiveInsertObject:ng atIndex:ic];
+                                [(VGroup*)cg recursiveRemoveObjectAtIndex:ic];
+                                [(VGroup*)cg recursiveInsertObject:ng atIndex:ic];
                                 if ([og isKindOfClass:[VGroup class]])
                                 {   [og recursiveRemoveObjectAtIndex:io];
                                     io--;
@@ -321,7 +321,7 @@
                                 {   [list removeObjectAtIndex:o]; // if the graphics are united we remove the og
                                     o--;
                                 }
-                                icg = [cg recursiveObjectAtIndex:ic];
+                                icg = [(VGroup*)cg recursiveObjectAtIndex:ic];
                                 cBounds = [icg bounds];
                                 /* cg will be removed perhaps -> we must set the c colors new !!!! */
                                 setcColorsNew = YES;
@@ -670,10 +670,11 @@
 /* return YES if cg is completly inside og
  */
 - (BOOL)removePartsOf:(VGraphic**)curG hiddenBy:(id)og
-{   int			i, iCnt, nothingRemoved = 0;
-    id			cg = *curG, ng;
-    NSMutableArray	*splitListOg = nil;
-    NSPoint		*iPts;
+{   int             i, iCnt, nothingRemoved = 0;
+    VGraphic        *cg = *curG;
+    VPath           *ng;
+    NSMutableArray  *splitListOg = nil;
+    NSPoint         *iPts;
 
     /* cg can't be inside a line or curve or part of arc */
     if ( !([og isKindOfClass:[VPath class]] && [og filled]) && !([og isKindOfClass:[VArc class]] && Abs([og angle]) == 360 && [og filled]) && !([og isKindOfClass:[VRectangle class]] && [og filled]) && !([og isKindOfClass:[VPolyLine class]] && [og filled]) )
@@ -684,11 +685,11 @@
     ng = [VPath path];
     [ng setColor:[cg color]];
     if ([cg respondsToSelector:@selector(fillColor)])
-    {   [ng setFillColor:[cg fillColor]];
-        [ng setEndColor:[cg endColor]];
-        [ng setRadialCenter:[cg radialCenter]];
-        [ng setStepWidth:[cg stepWidth]];
-        [ng setGraduateAngle:[cg graduateAngle]];
+    {   [ng setFillColor:    [(VPath*)cg fillColor]];
+        [ng setEndColor:     [(VPath*)cg endColor]];
+        [ng setRadialCenter: [(VPath*)cg radialCenter]];
+        [ng setStepWidth:    [(VPath*)cg stepWidth]];
+        [ng setGraduateAngle:[(VPath*)cg graduateAngle]];
     }
     [ng setFilled:[cg filled] optimize:NO];
     [ng setWidth:[cg width]];
@@ -725,7 +726,7 @@
         if ( [[ng list] count] == 1 )
         {   NSPoint	p;
 
-            ( [cg isKindOfClass:[VPath class]] ) ? [[[cg list] objectAtIndex:0] getPoint:&p at:0.4] : [cg getPoint:&p at:0.4];
+            ( [cg isKindOfClass:[VPath class]] ) ? [[[(VPath*)cg list] objectAtIndex:0] getPoint:&p at:0.4] : [cg getPoint:&p at:0.4];
             if ( ![og isPointInside:p] )
             {   *curG = [[cg copy] autorelease];
                 return NO;
@@ -754,7 +755,7 @@
     /* if cg is'nt tiled
      * the hiddenG (cg) is in the otherG or the otherG is in the hiddenG else -> FALSE
      */
-    if ( ([cg isKindOfClass:[VPath class]] && [[ng list] count] == [[cg list] count]) ||
+    if ( ([cg isKindOfClass:[VPath class]] && [[ng list] count] == [[(VPath*)cg list] count]) ||
         ([cg isKindOfClass:[VArc class]] && [[ng list] count] == 1) ||
         ([cg isKindOfClass:[VRectangle class]] && [[ng list] count] == 1) ||
         ([cg isKindOfClass:[VPolyLine class]] && [[ng list] count] == 1) )
@@ -763,14 +764,14 @@
         if (iCnt) free(iPts);
         /* cg inside og - cg will removed
         */
-        ( [cg isKindOfClass:[VPath class]] ) ? [[[cg list] objectAtIndex:0] getPoint:&p at:0.4] : [cg getPoint:&p at:0.4];
+        ( [cg isKindOfClass:[VPath class]] ) ? [[[(VPath*)cg list] objectAtIndex:0] getPoint:&p at:0.4] : [cg getPoint:&p at:0.4];
         if ( [og isPointInside:p] )
             return YES;
         /* og inside cg - add og to ng -> curG
          */
         ( [og isKindOfClass:[VPath class]] ) ? [[[(VPath*)og list] objectAtIndex:0] getPoint:&p at:0.4] : [og getPoint:&p at:0.4];
         if ( ([cg isKindOfClass:[VPath class]] || [cg isKindOfClass:[VArc class]] ||
-              [cg isKindOfClass:[VRectangle class]] || [cg isKindOfClass:[VPolyLine class]]) && [cg isPointInside:p] )
+              [cg isKindOfClass:[VRectangle class]] || [cg isKindOfClass:[VPolyLine class]]) && [(VRectangle*)cg isPointInside:p] )
         {
             if ( [og isKindOfClass:[VPath class]] )
                 for (i=0; i<(int)[[(VPath*)og list] count]; i++)
