@@ -1,7 +1,7 @@
 /* PerformanceMap.m
  * Map of graphic objects for optimized access
  *
- * Copyright (C) 1993-2010 by vhf interservice GmbH
+ * Copyright (C) 1993-2008 by vhf interservice GmbH
  * Authors:  T+T Hennerich (1993), Georg Fleischmann (2001-), Ilonka Fleischmann (2005-)
  *
  * created:  1993 T+T Hennerich, 2001-08-17 Georg Fleischmann
@@ -33,10 +33,10 @@
 #include "VImage.h"	// VImage class
 #include "VRectangle.h"	// -isPointInside
 
-#define MIN_SIZE            30	// minimum width and height of segment
-#define MAXCAPACITY         20	// maximum number of objects in segment (default)
-#define INCREASECAPACITY    10	// amount to increase maximum number of objects in segment
-#define MINSIZECAPACITY     15000
+#define MIN_SIZE		30	// minimum width and height of segment
+#define MAXCAPACITY		20	// maximum number of objects in segment (default)
+#define INCREASECAPACITY	10	// amount to increase maximum number of objects in segment
+#define MINSIZECAPACITY		15000
 
 @implementation PerformanceMap
 
@@ -175,10 +175,10 @@
 
 /* test if object is inside bounds
  */
-- (BOOL)isObjectInside:(id)g
-{   NSRect	objectBounds = [g bounds];	// bounds including line width and vertices
-    int		i, cnt;
-    VRectangle	*gRect;
+- (BOOL)isObjectInside:(VGraphic*)g
+{   NSRect      objectBounds = [g bounds];	// bounds including line width and vertices
+    int         i, cnt;
+    VRectangle  *gRect;
 
     /* object bounds entirely inside segment */
     if ( NSContainsRect(bounds, objectBounds) )
@@ -209,7 +209,7 @@
         return YES;
     /* object if filled and point of segment is inside object */
     if ( [g filled] &&
-         [g respondsToSelector:@selector(isPointInside:)] && [g isPointInside:bounds.origin] )
+         [(VPath*)g respondsToSelector:@selector(isPointInside:)] && [(VPath*)g isPointInside:bounds.origin] )
         return YES;
     /* same test as before for images */
     if ( [g isKindOfClass:[VImage class]] && NSPointInRect(bounds.origin, [g bounds]) )
@@ -227,8 +227,8 @@
  * to achieve a better performance when drawing everything.
  */
 - (void)addObject:anObject inTryNumber:(int)try withReferenceList:(NSArray*)referenceList
-{   int 	i, count, objectIndex;
-    id 		segment;
+{   NSInteger   i, count, objectIndex;
+    id          segment;
 
     if (!segmentList && (int)[graphicList count] == capacity)			// limit of segment reached
         [self splitSegmentInTryNumber:++try withReferenceList:referenceList];	// -> request sub segments
@@ -254,7 +254,7 @@
                 {
                     for (i=0; i<count; i++)
                     {
-                        if ((int)[referenceList indexOfObject:[graphicList objectAtIndex:i]] > objectIndex)
+                        if ((NSInteger)[referenceList indexOfObject:[graphicList objectAtIndex:i]] > objectIndex)
                         {   [graphicList insertObject:anObject atIndex:i];
                             break;
                         }
