@@ -1,11 +1,15 @@
 /* VGraphic.m
  * Graphic object - root class
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2013 by vhf interservice GmbH
  * Author:   Georg Fleischmann
+ * WWW:      http://www.cenon.info
  *
  * created:  1996-01-29
- * modified: 2010-07-28 (-setLabel:, -label added, version = 4)
+ * modified: 2011-05-28 (excluded, -isExcluded, -setExcluded:, +initialize: version=5)
+ *           2011-04-06 (-drawStartAtScale: added)
+ *           2011-03-04 (-size, -setSize: default implementation to make it work for all objects)
+ *           2010-07-28 (-setLabel:, -label added, version = 4)
  *           2010-07-08 (-transform: added)
  *
  * This program is free software; you can redistribute it and/or
@@ -23,8 +27,8 @@
  * with this program; see the file LICENSE. If not, write to vhf.
  *
  * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * eMail: cenon@vhf.de
+ * http://www.vhf.biz
  */
 
 #include <AppKit/AppKit.h>
@@ -36,21 +40,19 @@
 
 @implementation VGraphic
 
-/*
- * The fastKnobFill optimization just keeps a list of black and dark gray
+/* The fastKnobFill optimization just keeps a list of black and dark gray
  * rectangles (the knobbies are made out of black and dark gray rectangles)
  * and emits them in a single NSRectFillList() which is much faster than
  * doing individual rectfills (we also save the repeated setgrays).
  */
-
-static NSRect *lgrayRectList = NULL;
-static int lgrayRectSize = 0;
-static int lgrayRectCount = 0;
-static NSRect *blackRectList = NULL;
-static int blackRectSize = 0;
-static int blackRectCount = 0;
+static NSRect *lgrayRectList  = NULL;
+static int lgrayRectSize   = 0;
+static int lgrayRectCount  = 0;
+static NSRect *blackRectList  = NULL;
+static int blackRectSize   = 0;
+static int blackRectCount  = 0;
 static NSRect *dkgrayRectList = NULL;
-static int dkgrayRectSize = 0;
+static int dkgrayRectSize  = 0;
 static int dkgrayRectCount = 0;
 
 
@@ -63,17 +65,17 @@ static int dkgrayRectCount = 0;
 {
     if ( isBlack == YES )
     {
-	if (!blackRectList)
+        if (!blackRectList)
         {   blackRectSize = 16;
-	    blackRectList = NSZoneMalloc((NSZone *)[(NSObject *)NSApp zone], (blackRectSize) * sizeof(NSRect));
-	}
+            blackRectList = NSZoneMalloc((NSZone *)[(NSObject *)NSApp zone], (blackRectSize) * sizeof(NSRect));
+        }
         else if ( blackRectCount >= blackRectSize )
         {
-	    while (blackRectCount >= blackRectSize)
-                blackRectSize <<= 1;
-	    blackRectList = NSZoneRealloc((NSZone *)[(NSObject *)NSApp zone], blackRectList, (blackRectSize) * sizeof(NSRect));
-	}
-	blackRectList[blackRectCount++] = aRect;
+            while (blackRectCount >= blackRectSize)
+                    blackRectSize <<= 1;
+            blackRectList = NSZoneRealloc((NSZone *)[(NSObject *)NSApp zone], blackRectList, (blackRectSize) * sizeof(NSRect));
+        }
+        blackRectList[blackRectCount++] = aRect;
     }
     else if ( isBlack == NO )
     {
@@ -110,20 +112,17 @@ static int dkgrayRectCount = 0;
 + (void)showFastKnobFills
 {
     if (lgrayRectCount)
-    {
-        [[NSColor whiteColor] set]; // lightGrayColor
+    {   [[NSColor whiteColor] set]; // lightGrayColor
         NSRectFillList(lgrayRectList, lgrayRectCount);
         lgrayRectCount = 0;
     }
     if (blackRectCount)
-    {
-        [[NSColor blackColor] set];
+    {   [[NSColor blackColor] set];
         NSRectFillList(blackRectList, blackRectCount);
         blackRectCount = 0;
     }
     if (dkgrayRectCount)
-    {
-        [[NSColor lightGrayColor] set];
+    {   [[NSColor lightGrayColor] set];
         NSRectFillList(dkgrayRectList, dkgrayRectCount);
         dkgrayRectCount = 0;
     }
@@ -188,7 +187,7 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
  */
 + (void)initialize
 {
-    [VGraphic setVersion:4];
+    [VGraphic setVersion:5];
     return;
 }
 
@@ -388,14 +387,16 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
 {
 }
 
-- (void)setSize:(NSSize)size
-{
-    NSLog(@"method -setSize: not implemented");
+- (void)setSize:(NSSize)newSize
+{   NSRect	bRect = [self coordBounds];
+
+    [self scale:((bRect.size.width)  ? newSize.width /bRect.size.width  : 1.0)
+               :((bRect.size.height) ? newSize.height/bRect.size.height : 1.0)
+     withCenter:bRect.origin];
 }
 - (NSSize)size
-{
-    NSLog(@"method -size not implemented");
-    return NSMakeSize(0.0, 0.0);
+{   NSRect	bRect = [self coordBounds];
+    return bRect.size;
 }
 
 - (void)setRadius:(float)r
@@ -408,14 +409,11 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     return 0.0;
 }
 
-- (BOOL)isLocked;
-{
-    return isLocked;
-}
-- (void)setLocked:(BOOL)l
-{
-    isLocked = l;
-}
+- (BOOL)isExcluded              { return isExcluded; }
+- (void)setExcluded:(BOOL)flag  { isExcluded = flag; }
+
+- (BOOL)isLocked                { return isLocked; }
+- (void)setLocked:(BOOL)l       { isLocked = l; }
 
 - (float)angle
 {
@@ -519,7 +517,7 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
 - (void)drawColorPale:(BOOL)drawPale
 {
     if ( drawPale )
-    {   float	h, s, b, a;
+    {   CGFloat h, s, b, a;
 
         [[color colorUsingColorSpaceName:NSDeviceRGBColorSpace] getHue:&h saturation:&s brightness:&b alpha:&a];
         [[NSColor colorWithCalibratedHue:h saturation:s brightness:(b<0.5) ? 0.5 : b alpha:a] set];
@@ -624,6 +622,34 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     PSgrestore();
 }
 
+- (void)drawStartAtScale:(float)scaleFactor
+{   NSPoint         p0, p;
+    float           a, scale = 1.0 / scaleFactor;
+    NSBezierPath    *bPath = [NSBezierPath bezierPath];
+
+    p = [self gradientAt:0.0];
+    a = (!p.x) ?  ((p.y>0)?90.0:270.0) : RadToDeg(atan(p.y/p.x));
+    if (p.x < 0)
+        a += 180.0;
+    p0 = [self pointAt:0.0];
+
+    PSgsave();
+    [[NSColor blackColor] set];
+    [bPath setLineWidth:0.0];
+    [bPath moveToPoint:p0];
+    p = vhfPointRotatedAroundCenter(NSMakePoint(p0.x, p0.y - 6*scale), a, p0);
+    [bPath lineToPoint:p];
+    p = vhfPointRotatedAroundCenter(NSMakePoint(p0.x+2*scale, p0.y - 6*scale), a, p0);
+    [bPath lineToPoint:p];
+    p = vhfPointRotatedAroundCenter(NSMakePoint(p0.x+2*scale, p0.y + 6*scale), a, p0);
+    [bPath lineToPoint:p];
+    p = vhfPointRotatedAroundCenter(NSMakePoint(p0.x, p0.y + 6*scale), a, p0);
+    [bPath lineToPoint:p];
+    [bPath lineToPoint:p0];
+    [bPath fill];
+    PSgrestore();
+}
+
 /* exact coordinate bounds of an object
  * in case of curve this gives a slow calculation but more precise bounds
  * may have zero size!
@@ -937,6 +963,8 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     }
     if ( version >= 4 ) // 2010-07-28
         label = [[aDecoder decodeObject] retain];
+    if ( version >= 5 ) // 2011-05-28
+        [aDecoder decodeValuesOfObjCTypes:"c", &isExcluded];
 
     return self;
 }
@@ -948,6 +976,7 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     [aCoder encodeValuesOfObjCTypes:"c", &isLocked];
     [aCoder encodeValuesOfObjCTypes:"ciif", &relief, &reliefType, &reliefDirection, &reliefFlatness];
     [aCoder encodeObject:label];
+    [aCoder encodeValuesOfObjCTypes:"c", &isExcluded];
 }
 
 /* archiving with property list
@@ -961,6 +990,7 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     if ( color && color != [NSColor blackColor] )
         [plist setObject:propertyListFromNSColor(color) forKey:@"color"];
     if (width)          [plist setObject:propertyListFromFloat(width) forKey:@"width"];
+    if (isExcluded)     [plist setObject:@"1" forKey:@"isExcluded"];
     if (isLocked)       [plist setObject:@"1" forKey:@"isLocked"];
     if (isDirectionCCW) [plist setObject:@"1" forKey:@"ccw"];
     //if (!isDirectionCCW) [plist setObject:@"1" forKey:@"cw"];
@@ -980,7 +1010,8 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     if (!(color = colorFromPropertyList([plist objectForKey:@"color"], [self zone])))
         [self setColor:[NSColor blackColor]];
     width = [plist floatForKey:@"width"];
-    isLocked = ([plist objectForKey:@"isLocked"]) ? YES : NO;
+    isExcluded = ([plist objectForKey:@"isExcluded"]) ? YES : NO;
+    isLocked   = ([plist objectForKey:@"isLocked"])   ? YES : NO;
     //isDirectionCCW = ([plist objectForKey:@"cw"]) ? NO : YES;
     isDirectionCCW = ([plist objectForKey:@"ccw"] || [plist objectForKey:@"isDirectionCCW"]) ? YES : NO;
 
