@@ -1,11 +1,14 @@
 /* VGroup.m
  * Group of graphic objects
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2010-07-08 (relocated setList:)
+ * modified: 2012-08-12 (-setSize: use coordBounds for correct scaling)
+ *           2012-07-17 (*pts=NULL, *iPts=NULL initialized)
+ *           2012-04-20 (-coordBounds zero size Bounds are not recognized)
+ *           2010-07-08 (relocated -setList:)
  *
  * the group is implemented as follows:
  * - all objects are stored in a list with absolute coordinates
@@ -129,7 +132,7 @@
 {
     return list;
 }
-- (void)setList:(NSArray*)aList
+- (void)setList:(NSMutableArray*)aList
 {
     [list release];
     list = [aList retain];
@@ -384,7 +387,7 @@
 {   int	i, cnt;
 
     for (i=0, cnt = [list count]; i<cnt; i++)
-        [[list objectAtIndex:i] setWidth:w];
+        [(VGraphic*)[list objectAtIndex:i] setWidth:w];
 
     width = w;
     [self setUniColoringNew];
@@ -518,7 +521,7 @@
     [super setSelected:flag];
 }
 
-- (void)addObject:g
+- (void)addObject:(VGraphic*)g
 {
     [list addObject:g];
     [g setSelected:NO];
@@ -526,7 +529,7 @@
 
     /* check and set the colors */
     if ( ([list count] > 1 && uniColoring == NO) ||
-         ([g isKindOfClass:[VGroup class]] && [g uniColored] == NO) )
+         ([g isKindOfClass:[VGroup class]] && [(VGroup*)g uniColored] == NO) )
     {
         if (uniColoring == YES )
         {   /* init color */
@@ -561,7 +564,7 @@
         }
     }
     else // uniColoring == YES compare g colors with our
-    {   int		fild0 = 0, fild1 = 0;
+    {   int         fild0 = 0, fild1 = 0;
         NSColor		*col0, *fcol0 = nil, *ecol0 = nil, *col1, *fcol1 = nil, *ecol1 = nil;
         float		w0 = 0, ga0 = 0, w1 = 0, ga1 = 0;
         NSPoint		rc0 = {0.5,0.5}, rc1 = {0.5,0.5};
@@ -668,7 +671,7 @@
 }*/
 
 - (void)setSize:(NSSize)newSize
-{   NSRect	bRect = [self bounds];
+{   NSRect	bRect = [self coordBounds];
     //int		i;
 
     //for (i=[list count]-1; i>=0; i--)
@@ -719,7 +722,8 @@
         coordBounds = [[list objectAtIndex:cnt] coordBounds];
         for (i=cnt-1; i>=0; i--)
         {   rect = [[list objectAtIndex:i] coordBounds];
-            coordBounds = VHFUnionRect(rect, coordBounds);
+            if ( rect.size.width > 0.0 || rect.size.height > 0.0 )    // in case we have a bogus object
+                coordBounds = VHFUnionRect(rect, coordBounds);
         }
     }
     return coordBounds;
@@ -1094,31 +1098,43 @@
     /* display intersections within path */
     for (i=0; i<(int)[list count]-1; i++)
     {	id	g = [list objectAtIndex:i];
-        NSPoint	*pts;
+        NSPoint	*pts = NULL;
         int	cnt, j;
 
         cnt = [g getIntersections:&pts with:[list objectAtIndex:i+1]];
         for (j=0; j<cnt; j++)
             CROSS_45(pts[j]);
-        free(pts);
+        if (pts)
+            free(pts);
     }
 #endif
 }
 
 /*
- * Check for a control point hit. No need to perform the hit detection in
- * the server since its a simple rectangle intersection check. Return the
- * point number hit in the pt_num argument.
+ * Check for a control point hit. Return the point number hit in the pt argument.
  * Does not set the graphic selection!!
  */
 - (BOOL)hitEdge:(NSPoint)p fuzz:(float)fuzz :(NSPoint*)pt :(float)controlsize
-{   int	i;
+{   int     i;
+    BOOL    gotHit = NO;
+    NSPoint hitP = p;
+    double  sqrDistBest = MAXFLOAT;
 
     for (i=[list count]-1; i>=0; i--)
-    {	id obj = [list objectAtIndex:i];
+    {	id  obj = [list objectAtIndex:i];
 
         if ([obj hitEdge:p fuzz:fuzz :pt :controlsize])
-            return YES;
+        {   gotHit = YES;
+            if ( SqrDistPoints(*pt, p) < sqrDistBest )
+            {   hitP = *pt;
+                sqrDistBest = SqrDistPoints(*pt, p);
+            }
+            //return YES;
+        }
+    }
+    if (gotHit)
+    {   *pt = hitP;
+        return YES;
     }
     return NO;
 }
@@ -1231,7 +1247,7 @@
 
 - (int)getIntersections:(NSPoint**)ppArray with:g
 {   int		i, j, iCnt = 0, ptsCnt = Min(100, [self numPoints]);
-    NSPoint	*pts;
+    NSPoint	*pts = NULL;
 
     *ppArray = malloc(ptsCnt * sizeof(NSPoint));
     for (i=[list count]-1; i>=0; i--)
@@ -1293,12 +1309,13 @@
                     (*ppArray)[iCnt++] = pts[j];
             }
         }
-        free(pts);
+        if (pts)
+            free(pts);
     }
 
     if (!iCnt)
     {	free(*ppArray);
-        *ppArray = 0;
+        *ppArray = NULL;
     }
 
     return iCnt;
@@ -1388,10 +1405,10 @@
 }*/
 #if 0
 - getIntersectionsAndSplittedObjects:(NSPoint**)ppArray :(int*)iCnt with:g
-{   int			i, j, lCnt = [list count], ptsCnt = Min(100, [self numPoints]);
-    NSPoint		*pts;
-    NSRect		gBounds = [g bounds];
-    NSMutableArray	*splitList = [NSMutableArray array], *spList = nil;
+{   int             i, j, lCnt = [list count], ptsCnt = Min(100, [self numPoints]);
+    NSPoint         *pts = NULL;
+    NSRect          gBounds = [g bounds];
+    NSMutableArray  *splitList = [NSMutableArray array], *spList = nil;
 
     *iCnt = 0;
     *ppArray = malloc(ptsCnt * sizeof(NSPoint));
@@ -1462,12 +1479,13 @@
                     (*ppArray)[(*iCnt)++] = pts[j];
             }
         }
-        free(pts);
+        if (pts)
+            free(pts);
     }
 
     if (!(*iCnt))
     {	free(*ppArray);
-        *ppArray = 0;
+        *ppArray = NULL;
     }
     if ( [splitList count] > [list count] )
         return splitList;
@@ -1475,11 +1493,11 @@
 }
 
 - uniteWith:(VGraphic*)ug
-{   int			i, j, iCnt, nothingRemoved = 0, cnt;
-    VPath		*ng;
-    NSMutableArray	*splitListG, *splitListUg;
-    NSPoint		*iPts;
-    NSAutoreleasePool	*pool;
+{   int                 i, j, iCnt, nothingRemoved = 0, cnt;
+    VPath               *ng;
+    NSMutableArray      *splitListG, *splitListUg;
+    NSPoint             *iPts = NULL;
+    NSAutoreleasePool   *pool;
 
     if ( ![ug isKindOfClass:[VPath class]] && ![ug isKindOfClass:[VArc class]]
         && ![ug isKindOfClass:[VRectangle class]] && ![ug isKindOfClass:[VGroup class]] )
@@ -1520,7 +1538,8 @@
         else
             [splitListUg addObject:[[ug copy] autorelease]];
     }
-    free(iPts);
+    if (iPts)
+        free(iPts);
 
     /* ug is'nt a path and not splitted now -> nothing to unite - NO */
     if ( ![ug isKindOfClass:[VPath class]] && [splitListUg count] == 1 )
@@ -1589,7 +1608,8 @@
     [aCoder encodeObject:fillColor];
     [aCoder encodeObject:endColor];
     [aCoder encodeValuesOfObjCTypes:"ff", &graduateAngle, &stepWidth];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    [aCoder encodePoint:radialCenter];          // 2012-01-08
 }
 - (id)initWithCoder:(NSCoder *)aDecoder
 {   int	version;
@@ -1606,7 +1626,8 @@
         fillColor = [[aDecoder decodeObject] retain];
         endColor = [[aDecoder decodeObject] retain];
         [aDecoder decodeValuesOfObjCTypes:"ff", &graduateAngle , &stepWidth];
-        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        radialCenter = [aDecoder decodePoint];  // 2012-01-08
     }
     else
         [self setColorNew];
