@@ -1,10 +1,11 @@
 /* VImage.m
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1998-03-22
- * modified: 2010-04-14 (-initWithRepresentations: 10.6 keep image size despite using rep size)
+ * modified: 2012-01-30 (-drawWithPrincipal: if principal is nil -> set scale = 1.0)
+ *           2010-04-14 (-initWithRepresentations: 10.6 keep image size despite using rep size)
  *           2010-02-25 (-initWithRepresentations: workaround for Mac OS 10.6 problem with image sizes < pixel size)
  *           2010-01-29 (-writeFilesToDirectory: modified type converion to work with Mac OS 10.6)
  *           2008-12-01 (-drawWithPrincipal: relief draw pale)
@@ -172,9 +173,10 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
  */
 - init
 {
+    [super init];
     identifier = currentGraphicIdentifier++;
     [self setParameter];
-    return [super init];
+    return self;
 }
 
 - (NSString*)title		{ return @"Image"; }
@@ -287,6 +289,7 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
     clipPath = nil;
     thumbnail = NO;
     imageFile = [[NSString stringWithFormat:@"Image%d.tiff", identifier] retain];
+    [self setLabel:[imageFile stringByDeletingPathExtension]];
     sourcePath = nil;
     fileType = VImageTIFF_LZW;
     compressionFactor = 0.9;
@@ -307,14 +310,23 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
 - (void)setOrigin:(NSPoint)newOrigin    { origin = newOrigin; dirty = YES; }
 - (NSPoint)origin                       { return origin; }
 
+/* this sets the file name (extension) and label (no extension)
+ */
 - (NSString*)setName:(NSString*)str
 {   NSString    *ext = [VImage fileExtensionForFileType:fileType];
 
-    if ( ! [[str pathExtension] isEqual:ext] )  // set correct path extension
-        str = [[str stringByDeletingPathExtension] stringByAppendingPathExtension:ext];
-
     [imageFile release];
-    imageFile = [[NSString stringWithFormat:@"%@", str] retain];
+    if ( ! [[str pathExtension] isEqual:ext] )  // set correct path extension
+        imageFile = [[[str stringByDeletingPathExtension] stringByAppendingPathExtension:ext] retain];
+    else
+        imageFile = [[NSString stringWithString:str] retain];
+
+    /* set label to same name but without extension */
+    if ( [str pathExtension] )
+        [self setLabel:[str stringByDeletingPathExtension]];
+    else
+        [self setLabel:str];
+
     return imageFile;
 }
 
@@ -1081,7 +1093,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
 {   NSPoint         p;
     NSSize          currentSize;
     NSRect          bounds, rect;
-    float           scale = [principal scaleFactor];
+    float           scale = (principal) ? [principal scaleFactor] : 1.0;
     NSBezierPath    *bPath;
     BOOL            releaseImage = NO;
     NSImage         *sepImage = nil; // color separated image
@@ -1090,7 +1102,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
         return;
 
     bounds.origin = origin;
-    bounds.size = size;
+    bounds.size   = size;
     bounds.size.width  *= scale;
     bounds.size.height *= scale;
 
@@ -1133,19 +1145,19 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
                 /* draw white clipPath */
                 if (clipPath)
                 {   NSColor	*oldFillColor = nil;
-                    int		oldFill = [clipPath filled];
+                    int		oldFill  = [clipPath filled];
                     float	oldWidth = [clipPath width];
 
                     [clipPath setWidth:0.0];
                     [clipPath setFilled:1];
                     if (oldFill)
-                        oldFillColor = [[clipPath fillColor] retain];
-                    [clipPath setFillColor:[NSColor whiteColor]];
+                        oldFillColor = [[(VPath*)clipPath fillColor] retain];
+                    [(VPath*)clipPath setFillColor:[NSColor whiteColor]];
                     [clipPath drawWithPrincipal:principal]; // draw the white clipPath
                     [clipPath setFilled:oldFill];
                     [clipPath setWidth:oldWidth];
                     if (oldFill)
-                    {   [clipPath setFillColor:oldFillColor];
+                    {   [(VPath*)clipPath setFillColor:oldFillColor];
                         [oldFillColor release];
                     }
                 }
@@ -1202,17 +1214,17 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
             [bPath setLineCapStyle:NSRoundLineCapStyle];
             [bPath setLineJoinStyle:NSRoundLineJoinStyle];
             if ([clipPath isMemberOfClass:[VPath class]])
-            {   int	i, cnt = [[clipPath list] count];
-                NSPoint	currentPoint = NSMakePoint(LARGENEG_COORD, LARGENEG_COORD);
+            {   int     i, cnt = [[(VPath*)clipPath list] count];
+                NSPoint currentPoint = NSMakePoint(LARGENEG_COORD, LARGENEG_COORD);
 
                 for (i=0; i<cnt; i++)
-                    currentPoint = [[[clipPath list] objectAtIndex:i] appendToBezierPath:bPath
+                    currentPoint = [[[(VPath*)clipPath list] objectAtIndex:i] appendToBezierPath:bPath
                                                                             currentPoint:currentPoint];
             }
             else
             {   NSPoint	currentPoint = NSMakePoint(LARGENEG_COORD, LARGENEG_COORD);
 
-                currentPoint = [clipPath appendToBezierPath:bPath currentPoint:currentPoint];
+                currentPoint = [(VLine*)clipPath appendToBezierPath:bPath currentPoint:currentPoint];
             }
             [bPath setWindingRule:NSEvenOddWindingRule];
             [bPath addClip];
@@ -1294,7 +1306,8 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
         {
             bounds.size = size; // important
 #ifdef __APPLE__	// Apple works
-            rect = bounds;
+            //rect = bounds;    // 2011-02-28: seems to be better equal to OpenStep
+            rect = NSIntersectionRect(bounds, [principal visibleRect]);
 #else			// workaround for OpenStep
             rect = NSIntersectionRect(bounds, [principal visibleRect]);
 #endif
@@ -1624,12 +1637,12 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
     {   BOOL    flag;
 
         if ([clipPath respondsToSelector:@selector(setFilled:optimize:)])
-            [clipPath setFilled:YES optimize:NO];
+            [(VPath*)clipPath setFilled:YES optimize:NO];
         else
             [clipPath setFilled:YES];
         flag = [clipPath hit:p fuzz:fuzz];
         if ([clipPath respondsToSelector:@selector(setFilled:optimize:)])
-            [clipPath setFilled:NO optimize:NO];
+            [(VPath*)clipPath setFilled:NO optimize:NO];
         else
             [clipPath setFilled:NO];
         return flag;
@@ -1671,16 +1684,17 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
 - (VPath*)contour:(float)w
 {   PathContour	*pathContour = [[PathContour new] autorelease];
 
+    // TODO: this would be better as one PolyLine per subpath !
     return [pathContour contourImage:self width:w];
 }
 
 /*[self saveToCMYKTiff:directory];*/
 - (BOOL)saveToCMYKTiff:(NSString*)directory
-{   int			i;
-    NSArray		*imageReps;
-    NSBitmapImageRep	*bmImageRep = nil;
-    NSFileManager	*fileManager = [NSFileManager defaultManager];
-    NSData		*imageData;
+{   int                 i;
+    NSArray             *imageReps;
+    NSBitmapImageRep    *bmImageRep = nil;
+    NSFileManager       *fileManager = [NSFileManager defaultManager];
+    NSData              *imageData;
 
     if (!image)	// load image
     {	NSString	*backupFilename = [directory stringByAppendingString:@"~"];
@@ -2021,8 +2035,11 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
 - (void)encodeWithCoder:(NSCoder *)aCoder
 {
     [super encodeWithCoder:aCoder];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &origin, &size];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &originalSize];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &origin, &size];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &originalSize];
+    [aCoder encodePoint:origin];            // 2012-01-08
+    [aCoder encodeSize:size];
+    [aCoder encodeSize:originalSize];
     [aCoder encodeValuesOfObjCTypes:"i", &identifier];
     if (!imageFile)
         imageFile = [[NSString stringWithFormat:@"Image%d.tiff", identifier] retain];
@@ -2044,8 +2061,11 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
         [aDecoder decodeValuesOfObjCTypes:"{ff}", &originalSize];
     }
     else
-    {   [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &origin, &size];
-        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &originalSize];
+    {   //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &origin, &size];
+        //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &originalSize];
+        origin       = [aDecoder decodePoint];  // 2012-01-08
+        size         = [aDecoder decodeSize];
+        originalSize = [aDecoder decodeSize];
     }
     [aDecoder decodeValuesOfObjCTypes:"i", &identifier]; // from copied object
     if (identifier >= currentGraphicIdentifier)
@@ -2116,6 +2136,8 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
     if (identifier >= currentGraphicIdentifier)
         currentGraphicIdentifier = identifier+1;
     imageFile = [[plist objectForKey:@"imageFile"] retain];
+    if ( !label )   // no label yet, we make one
+        label = [[imageFile stringByDeletingPathExtension] retain];
 
     /* thumbnail */
     thumbnail = ([plist objectForKey:@"thumbnail"] ? YES : NO);
