@@ -1,11 +1,12 @@
 /* VLine.m
  * 2-D Line object
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2010-02-18 (exit with right mouse click added)
+ * modified: 2011-07-11 (-sqrDistanceGraphic:::, -sqrDistanceGraphic:; some new graphics recognized)
+ *           2010-02-18 (exit with right mouse click added)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -76,7 +77,7 @@
  * modified: 
  */
 - copy
-{   id		line = [[VLine allocWithZone:[self zone]] init];
+{   VLine   *line = [[VLine allocWithZone:[self zone]] init];
 
     [line setWidth:width];
     [line setSelected:isSelected];
@@ -136,9 +137,9 @@
     [[(App*)NSApp inspectorPanel] loadGraphic:self];	// set the values of the inspector to self
 
     /* get start location, convert window to view coordinates */
-    start = [view convertPoint:[event locationInWindow] fromView:nil];
-    hitPoint = start;
-    hitEdge = [view hitEdge:&hitPoint spare:self];	// snap to point
+    start     = [view convertPoint:[event locationInWindow] fromView:nil];
+    hitPoint  = start;
+    hitEdge   = [view hitEdge:&hitPoint spare:self];	// snap to point
     gridPoint = [view grid:start];			// set on grid
     if ( hitEdge &&
          ((gridPoint.x == start.x && gridPoint.y == start.y)  ||
@@ -959,7 +960,7 @@
     [list addObject:line];
 
     for (i=[list count]-1; i>=0; i--)
-    {	id	g = [list objectAtIndex:i];
+    {	VGraphic    *g = [list objectAtIndex:i];
 
         [g setWidth:0.0];
         [g setColor:color];
@@ -1024,17 +1025,17 @@
 }
 
 - getListOfObjectsSplittedFrom:(NSPoint*)pArray :(int)iCnt
-{   NSMutableArray	*splitList = nil;
-    int			i, cnt = 0;
-    NSPoint		*ps = malloc((iCnt) * sizeof(NSPoint)), bufp;
-    NSRect		bounds;
-    float		dx, dy;
-    NSAutoreleasePool	*pool;
+{   NSMutableArray      *splitList = nil;
+    int                 i, cnt = 0;
+    NSPoint             *ps = malloc((iCnt) * sizeof(NSPoint)), bufp;
+    NSRect              bounds;
+    float               dx, dy;
+    NSAutoreleasePool   *pool;
 
     bounds = [self coordBounds];
     /* only important for horicontal and vertical lines! */
-    bounds.origin.x -= 15.0*TOLERANCE;
-    bounds.origin.y -= 15.0*TOLERANCE;
+    bounds.origin.x    -= 15.0*TOLERANCE;
+    bounds.origin.y    -= 15.0*TOLERANCE;
     bounds.size.width  += 30.0*TOLERANCE;
     bounds.size.height += 30.0*TOLERANCE;
     dx = p1.x - p0.x;
@@ -1133,6 +1134,7 @@
         iCnt = [g getIntersections:ppArray with:self];
     else
     {   NSLog(@"VLine, intersection with unknown class!");
+        *ppArray = NULL;
         return 0;
     }
 
@@ -1140,7 +1142,7 @@
         sortPointArray(*ppArray, iCnt, p0);
     else
     {	free(*ppArray);
-        *ppArray = 0;
+        *ppArray = NULL;
     }
 
     return iCnt;
@@ -1154,9 +1156,9 @@
         [g getVertices:&pv0 :&pv1];
         return [self sqrDistanceLine:pv0 :pv1 :pg1 :pg2];
     }
-    else if ([g isKindOfClass:[VArc class]])
-        return [g sqrDistanceLine:p0 :p1 :pg2 :pg1];
-    else if ([g isKindOfClass:[VRectangle class]])
+    else if ( [g isKindOfClass:[VPath     class]] || [g isKindOfClass:[VGroup     class]] ||
+              [g isKindOfClass:[VPolyLine class]] || [g isKindOfClass:[VRectangle class]] ||
+              [g isKindOfClass:[VCurve    class]] || [g isKindOfClass:[VArc       class]] )
         return [g sqrDistanceGraphic:self :pg2 :pg1];
     else
     {   NSLog(@"VLine, distance (with two nearest points) with unknown class!");
@@ -1167,8 +1169,9 @@
 
 - (float)sqrDistanceGraphic:g
 {
-    if ([g isKindOfClass:[VPath class]] || [g isKindOfClass:[VGroup class]] ||
-        [g isKindOfClass:[VPolyLine class]] || [g isKindOfClass:[VRectangle class]])
+    if ( [g isKindOfClass:[VPath     class]] || [g isKindOfClass:[VGroup     class]] ||
+         [g isKindOfClass:[VPolyLine class]] || [g isKindOfClass:[VRectangle class]] ||
+         [g isKindOfClass:[VCurve    class]] || [g isKindOfClass:[VArc       class]] )
         return [g sqrDistanceGraphic:self];
     else if ([g isKindOfClass:[VLine class]])
     {	NSPoint	pv0, pv1;
@@ -1176,8 +1179,6 @@
         [g getVertices:&pv0 :&pv1];
         return [self sqrDistanceLine:pv0 :pv1];
     }
-    else if ([g isKindOfClass:[VCurve class]] || [g isKindOfClass:[VArc class]])
-        return [g sqrDistanceLine:p0 :p1];
     else
     {   NSLog(@"VLine, distance with unknown class!");
         return -1.0;
@@ -1253,32 +1254,36 @@
     [g getPoint:0 :&s2];
     [g getPoint:3 :&e2];
     if ( (Diff(p0.x, s2.x) <= TOLERANCE && Diff(p0.y, s2.y) <= TOLERANCE &&
-         Diff(p1.x, e2.x) <= TOLERANCE && Diff(p1.y, e2.y) <= TOLERANCE) ||
-        (Diff(p0.x, e2.x) <= TOLERANCE && Diff(p0.y, e2.y) <= TOLERANCE &&
-         Diff(p1.x, s2.x) <= TOLERANCE && Diff(p1.y, s2.y) <= TOLERANCE) )
+          Diff(p1.x, e2.x) <= TOLERANCE && Diff(p1.y, e2.y) <= TOLERANCE) ||
+         (Diff(p0.x, e2.x) <= TOLERANCE && Diff(p0.y, e2.y) <= TOLERANCE &&
+          Diff(p1.x, s2.x) <= TOLERANCE && Diff(p1.y, s2.y) <= TOLERANCE) )
         return YES;
     return NO;
 }
 
+- (void)encodeWithCoder:(NSCoder *)aCoder
+{
+    [super encodeWithCoder:aCoder];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &p0, &p1];
+    [aCoder encodePoint:p0];    // 2012-01-08
+    [aCoder encodePoint:p1];
+}
 - (id)initWithCoder:(NSCoder *)aDecoder
 {   int	version;
 
     [super initWithCoder:aDecoder];
-    version = [aDecoder versionForClassName:@"Line"];
+    version = [aDecoder versionForClassName:@"VLine"];
     if ( version < 1 )
         [aDecoder decodeValuesOfObjCTypes:"{ff}{ff}", &p0, &p1];
     else
-        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &p0, &p1];
+    {   //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &p0, &p1];
+        p0 = [aDecoder decodePoint];    // 2012-01-08
+        p1 = [aDecoder decodePoint];
+    }
 
     [self setParameter];
-
     return self;
 }
-- (void)encodeWithCoder:(NSCoder *)aCoder
-{
-    [super encodeWithCoder:aCoder];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &p0, &p1];
-}
 
 /* archiving with property list
  */
@@ -1294,6 +1299,7 @@
     [super initFromPropertyList:plist inDirectory:directory];
     p0 = pointFromPropertyList([plist objectForKey:@"p0"]);
     p1 = pointFromPropertyList([plist objectForKey:@"p1"]);
+    [self setParameter];
     return self;
 }
 
