@@ -1,11 +1,11 @@
 /* VLine3D.h
  * 3-D line object
  *
- * Copyright (C) 1996-2002 by vhf interservice GmbH
+ * Copyright (C) 1996-2011 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-19
- * modified: 2002-07-09
+ * modified: 2011-04-04 (-line3DWithPoints: added)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -40,8 +40,10 @@
 
 /* class methods */
 + (VLine3D*)line3D;
++ (VLine3D*)line3DWithPoints:(V3Point)p0 :(V3Point)p1;
 
 /* line methods */
+- (void)setVertices3D:(V3Point)pv0 :(V3Point)pv1;
 - (void)setZLevel:(float)zv0 :(float)zv1;
 - (void)getZLevel:(float*)zv0 :(float*)zv1;
 
