@@ -1,11 +1,13 @@
 /* VRectangle.m
  * 2-D Rectangle object
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2013 by vhf interservice GmbH
  * Author:   Georg Fleischmann
+ * WWW:      http://www.cenon.info
  *
  * created:  1996-09-17
- * modified: 2010-07-17 (-transform added, -scale: scales width correctly with negative scales (flip))
+ * modified: 2013-08-13 (-hit:fuzz: rotate p back before let path do the job)
+ *           2010-07-17 (-transform added, -scale: scales width correctly with negative scales (flip))
  *           2010-03-03 (-copy setDirectionCCW, -pathRepresentation edges build with r, not radius)
  *           2006-02-08
  *
@@ -24,8 +26,8 @@
  * with this program; see the file LICENSE. If not, write to vhf.
  *
  * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * eMail: info@vhf.biz
+ * http://www.vhf.biz
  */
 
 #include <AppKit/AppKit.h>
@@ -84,10 +86,10 @@
 /* deep copy
  *
  * created:  2001-02-15
- * modified: 
+ * modified: 2012-01-06
  */
 - copy
-{   id		rectangle = [[VRectangle allocWithZone:[self zone]] init];
+{   VRectangle  *rectangle = [[VRectangle allocWithZone:[self zone]] init];
 
     [rectangle setSelected:isSelected];
     [rectangle setLocked:NO];
@@ -643,7 +645,7 @@
                 col = [self separationColor:fillColor]; // get individual separation color
 
             if ( [principal mustDrawPale] )
-            {   float	h, s, b, a;
+            {   CGFloat h, s, b, a;
 
                 [[col colorUsingColorSpaceName:NSDeviceRGBColorSpace] getHue:&h saturation:&s brightness:&b alpha:&a];
                 [[NSColor colorWithCalibratedHue:h saturation:s brightness:(b<0.5) ? 0.5 : b alpha:a] set];
@@ -658,11 +660,14 @@
             [NSBezierPath fillRect:NSMakeRect(origin.x, origin.y, size.x, size.y)];
         }
         if ( width > 0.0 || !filled || Diff(size.x, 0) < 500*TOLERANCE || Diff(size.y, 0) < 500*TOLERANCE )
-        {   NSColor	*oldColor=nil;
+        {   NSColor	*oldColor = nil;
             float	defaultWidth = [NSBezierPath defaultLineWidth];
 
+            if ( filled && (Diff(size.x, 0) < 500*TOLERANCE || Diff(size.y, 0) < 500*TOLERANCE) )
+                defaultWidth = 0.1 / [principal scaleFactor];   // make visible what actually is invisible
+
             if (!VHFIsDrawingToScreen() && [principal separationColor])
-            {   NSColor	*sepColor = [self separationColor:color]; // get individual separation color
+            {   NSColor	*sepColor = [self separationColor:color];   // get individual separation color
 
                 oldColor = [color retain];
                 [self setColor:sepColor];
@@ -1100,8 +1105,11 @@
         if ( !Prefs_SelectByBorder && filled && !r )
             return YES;
         if ( !Prefs_SelectByBorder && filled && r )
+        {
+            if ( rotAngle )
+                vhfRotatePointAroundCenter(&p, origin, rotAngle); // rotate back
             return [[self pathRepresentation] hit:p fuzz:fuzz];
-
+        }
         for ( i=(ccw)?0:4; (ccw)?i<4:i; (ccw)?i++:i-- )
         {
             j = (ccw) ? i+1 : i-1;
@@ -2250,7 +2258,9 @@
 - (void)encodeWithCoder:(NSCoder *)aCoder
 {
     [super encodeWithCoder:aCoder];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &origin, &size];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &origin, &size];
+    [aCoder encodePoint:origin];            // 2012-01-08
+    [aCoder encodePoint:size];  // size is actually a point !
     [aCoder encodeValuesOfObjCTypes:"f", &radius];
     [aCoder encodeValuesOfObjCTypes:"ic", &filled, &isDirectionCCW]; // 2002-07-07
     [aCoder encodeValuesOfObjCTypes:"f", &rotAngle];
@@ -2258,7 +2268,8 @@
     [aCoder encodeObject:fillColor];
     [aCoder encodeObject:endColor];
     [aCoder encodeValuesOfObjCTypes:"ff", &graduateAngle, &stepWidth];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    [aCoder encodePoint:radialCenter];      // 2012-01-08
 }
 - (id)initWithCoder:(NSCoder *)aDecoder
 {   int	version;
@@ -2268,7 +2279,10 @@
     if ( version < 3 )
         [aDecoder decodeValuesOfObjCTypes:"{ff}{ff}", &origin, &size];
     else
-        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &origin, &size];
+    {   //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &origin, &size];
+        origin = [aDecoder decodePoint];    // 2012-01-08
+        size   = [aDecoder decodePoint];
+    }
     [aDecoder decodeValuesOfObjCTypes:"f", &radius];
     if ( version<1 )
         [aDecoder decodeValuesOfObjCTypes:"c", &filled];
@@ -2296,7 +2310,8 @@
     {   fillColor = [[aDecoder decodeObject] retain];
         endColor = [[aDecoder decodeObject] retain];
         [aDecoder decodeValuesOfObjCTypes:"ff", &graduateAngle , &stepWidth];
-        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        radialCenter = [aDecoder decodePoint];  // 2012-01-08
     }
     [self setParameter];
     graduateDirty = YES;
@@ -2310,34 +2325,34 @@
 - (id)propertyList
 {   NSMutableDictionary	*plist = [super propertyList];
 
-    [plist setObject:propertyListFromNSPoint(origin) forKey:@"origin"];
-    [plist setObject:propertyListFromNSPoint(size) forKey:@"size"];
+    [plist setObject:propertyListFromNSPoint(origin)            forKey:@"origin"];
+    [plist setObject:propertyListFromNSPoint(size)              forKey:@"size"];
     if (radius)
-        [plist setObject:propertyListFromFloat(radius) forKey:@"radius"];
+        [plist setObject:propertyListFromFloat(radius)          forKey:@"radius"];
     if (rotAngle)
-        [plist setObject:propertyListFromFloat(rotAngle) forKey:@"rotAngle"];
+        [plist setObject:propertyListFromFloat(rotAngle)        forKey:@"rotAngle"];
     if (filled)
-        [plist setObject:propertyListFromInt(filled) forKey:@"filled"];
+        [plist setObject:propertyListFromInt(filled)            forKey:@"filled"];
     if (fillColor != [NSColor blackColor])
-        [plist setObject:propertyListFromNSColor(fillColor) forKey:@"fillColor"];
+        [plist setObject:propertyListFromNSColor(fillColor)     forKey:@"fillColor"];
     if (endColor != [NSColor blackColor])
-        [plist setObject:propertyListFromNSColor(endColor) forKey:@"endColor"];
+        [plist setObject:propertyListFromNSColor(endColor)      forKey:@"endColor"];
     if (graduateAngle)
-        [plist setObject:propertyListFromFloat(graduateAngle) forKey:@"graduateAngle"];
+        [plist setObject:propertyListFromFloat(graduateAngle)   forKey:@"graduateAngle"];
     if (stepWidth != 7)
-        [plist setObject:propertyListFromFloat(stepWidth) forKey:@"stepWidth"];
+        [plist setObject:propertyListFromFloat(stepWidth)       forKey:@"stepWidth"];
     if (!(radialCenter.x == 0.5 && radialCenter.y == 0.5))
-        [plist setObject:propertyListFromNSPoint(radialCenter) forKey:@"radialCenter"];
+        [plist setObject:propertyListFromNSPoint(radialCenter)  forKey:@"radialCenter"];
     return plist;
 }
 - (id)initFromPropertyList:(id)plist inDirectory:(NSString *)directory
 {
     [super initFromPropertyList:plist inDirectory:directory];
     origin = pointFromPropertyList([plist objectForKey:@"origin"]);
-    size = pointFromPropertyList([plist objectForKey:@"size"]);
-    radius = [plist floatForKey:@"radius"];
+    size   = pointFromPropertyList([plist objectForKey:@"size"]);
+    radius   = [plist floatForKey:@"radius"];
     rotAngle = [plist floatForKey:@"rotAngle"];
-    filled = [plist intForKey:@"filled"];
+    filled   = [plist intForKey:@"filled"];
     if (!filled && [plist objectForKey:@"filled"])
         filled = 1;
     if (!(fillColor = colorFromPropertyList([plist objectForKey:@"fillColor"], [self zone])))
