@@ -1,11 +1,13 @@
 /* VText.m
  * 2-D Text object
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-02-15
- * modified: 2010-07-29 (-drawWithPrincipal:...) move -setTextContainerInset: before -setFrame to make mirror on Apple visible
+ * modified: 2012-02-28 (-drawWithPrincipal: set frame twice to make mirrored text visible)
+ *           2012-01-04 (-textDidEndEditing: if locked, we don't adjust the size of the text box)
+ *           2010-07-29 (-drawWithPrincipal:...) move -setTextContainerInset: before -setFrame to make mirror on Apple visible
  *           2010-07-28 (+valueForKey:inArray:) new
  *           2009-12-29 (-drawWithPrincipal:): for GNUstep renew drawText after rotAngle
  *           2009-02-12 (-textDidEndEditing: add correction to height to make text appear safely on Apple)
@@ -98,7 +100,7 @@ static void initClassVars()
 + (void)initialize
 {   //NSString	*path;
 
-    [VText setVersion:7];
+    [VText setVersion:8];
 
     /* load font list of font which we should not fill */
     /*
@@ -125,7 +127,7 @@ static void initClassVars()
  * created: 2010-07-28
  */
 + (NSString*)valueForKey:(NSString*)key inArray:(NSArray*)array
-{   int         i, cnt = [array count];
+{   int i, cnt = [array count];
 
     for ( i=0; i<cnt; i++ )
     {   VText   *g = [array objectAtIndex:i];
@@ -847,6 +849,8 @@ static void initClassVars()
     dirty = YES;
 }
 
+/* reduce kerning to fit text in width of text box
+ */
 - (void)kernToFitHorizontal
 {   NSLayoutManager	*lm = [drawText layoutManager];
     NSRect		rect;
@@ -867,10 +871,11 @@ static void initClassVars()
     {
         [self setKerning:f+0.05];
         [self updateDrawText];
+        // NOTE: on Apple this method returns heights that are less than our lineHeight
         rect = [lm boundingRectForGlyphRange:NSMakeRange(0, [lm numberOfGlyphs])
                              inTextContainer:[[lm textContainers] objectAtIndex:0]];
-        l = rect.size.height / [self lineHeight];
-        if ( l <= lines )
+        l = (rect.size.height+0.5) / Min([[self font] pointSize], [self lineHeight]);
+        if ( l <= lines )   // no line break any more -> kerning is small enough or too small
         {
             if (!i || i>10)
             {   [self setKerning:f];
@@ -878,7 +883,7 @@ static void initClassVars()
             }
             min = f;
         }
-        else
+        else                // kerning is too big
             max = f;
     }
     [drawText setFrame:bounds];
@@ -965,23 +970,25 @@ static void initClassVars()
 }
 
 /* created:  1995-10-21
- * modified: 2002-12-04
+ * modified: 2012-02-27 (make sure angle is within 0 and 360 deg)
  * purpose:  rotate the graphic around cp
  */
 - (void)setAngle:(float)angle withCenter:(NSPoint)cp
 {
+    if (angle <= -360.0) angle += 360.0; // 2012-02-27
+    if (angle >=  360.0) angle -= 360.0;
+
     if (filled)
     {   graduateAngle -= angle;
-        if (graduateAngle < 0.0)
-            graduateAngle += 360.0;
-        if (graduateAngle > 360.0)
-            graduateAngle -= 360.0;
+        if (graduateAngle <    0.0) graduateAngle += 360.0;
+        if (graduateAngle >= 360.0) graduateAngle -= 360.0;
         //vhfRotatePointAroundCenter(&radialCenter, NSMakePoint(0.5, 0.5), -angle);
     }
     vhfRotatePointAroundCenter(&bounds.origin, cp, -angle);
     rotAngle -= angle;
-    if (rotAngle < 0.0)
-        rotAngle += 360;
+    //if (rotAngle <    0.0) rotAngle += 360.0;
+    //if (rotAngle >= 360.0) rotAngle -= 360.0;   // 2012-02-27
+    rotAngle = vhfModulo(rotAngle, 360.0);
     dirty = YES;
 }
 
@@ -1028,8 +1035,8 @@ static void initClassVars()
         [[drawText textStorage] beginEditing];
         /* check color */
         if ((!VHFIsDrawingToScreen() && [principal separationColor]) || [principal mustDrawPale])
-        {   NSMutableAttributedString	*mas = [attributedString mutableCopyWithZone:[self zone]];
-            int		i, cnt = [mas length];
+        {   NSMutableAttributedString   *mas = [attributedString mutableCopyWithZone:[self zone]];
+            int                         i, cnt = [mas length];
 
             for (i=0; i<cnt; i++)
             {   NSRange	range;
@@ -1040,7 +1047,7 @@ static void initClassVars()
                 if (!colorAtt)
                     colorAtt = fillColor;
                 if ([principal mustDrawPale])
-                {   float	h, s, b, a;
+                {   CGFloat h, s, b, a;
 
                     [[colorAtt colorUsingColorSpaceName:NSDeviceRGBColorSpace] getHue:&h
                                                                            saturation:&s
@@ -1059,12 +1066,12 @@ static void initClassVars()
         }
 #if !defined(GNUSTEP_BASE_VERSION) && !defined(__APPLE__)	// OpenStep 4.2
         else if (VHFIsDrawingToScreen() && [[color colorSpaceName] isEqualToString:NSDeviceCMYKColorSpace])
-        {   NSMutableAttributedString	*mas = [attributedString mutableCopyWithZone:[self zone]];
-            int		i, cnt = [mas length];
+        {   NSMutableAttributedString   *mas = [attributedString mutableCopyWithZone:[self zone]];
+            int                         i, cnt = [mas length];
 
             for (i=0; i<cnt; i++)
-            {   NSRange			range;
-                NSColor			*colorAtt;
+            {   NSRange range;
+                NSColor *colorAtt;
 
                 colorAtt = [mas attribute:@"NSColor" atIndex:i longestEffectiveRange:&range
                                   inRange:NSMakeRange(0, cnt)];
@@ -1085,33 +1092,42 @@ static void initClassVars()
         [[drawText textStorage] endEditing];
         [drawText setTextContainerInset:NSMakeSize(0.0, 0.0)];  // 2010-07-29: moved before setFrame: for Apple mirror
         [drawText setFrame:bounds];
+        [drawText setFrame:bounds]; // 2012-02-28: well, without this extra invitation Apple doesn't draw mirrored text !
         //[drawText writeRTFDToFile:@"/tmp/text.rtf" atomically:NO];	// debugging
         //[drawText setFrame:bounds];
 
         if ( centerVertical )	// vertical center: (bounds.size.height-textHeight)/2
         {   NSRect	rect = [lm boundingRectForGlyphRange:NSMakeRange(0, [lm numberOfGlyphs])
-                                             inTextContainer:[drawText textContainer]];
+                                         inTextContainer:[drawText textContainer]];
             [drawText setTextContainerInset:NSMakeSize(0.0, Max(0.0, (bounds.size.height-rect.size.height-1.0)/2.0))];
         }
 
         [drawText setFrameRotation:rotAngle];
-        // FIXME: text with aspectRatio != 1.0 mirrors outside text box
         [drawText scaleUnitSquareToSize:NSMakeSize(1.0, aspectRatio)];
         //displayModeFilled = filled;
         [window setAutodisplay:NO]; // don't let addSubview: cause redisplay (not wanted for resize)
-        [[NSView focusView] addSubview:drawText];
+        /* focusView is either cache or DocView
+         *   DocView        -> editView (FlippedView)
+         *                  -> drawText
+         *   cache (NSView) -> drawText
+         */
+        [[NSView focusView] addSubview:drawText];   // cache (NSView) / DocView -> drawText
         [drawText lockFocus];
         [drawText drawRect:[drawText bounds]];
         [drawText unlockFocus];
         [drawText removeFromSuperview];
         [window setAutodisplay:saveAutoDisplay];
+        //[[NSView focusView] setNeedsDisplay:NO];
         [drawText scaleUnitSquareToSize:NSMakeSize(1.0, 1.0/aspectRatio)];
-#ifdef GNUSTEP_BASE_VERSION // 2009-12-29 gui 0.17.1: GNUstpe workaround, rotation screws up everything
+#   ifdef GNUSTEP_BASE_VERSION  // 2009-12-29 gui 0.17.1: GNUstep workaround, rotation screws up everything
         if ( rotAngle != 0.0 )
         {   [drawText release]; drawText = nil;
             initClassVars();
         }
-#endif
+//#   else
+//        [drawText release]; drawText = nil;
+//        initClassVars();
+#   endif
         //[drawText setFrameRotation:0.0];
     	/*if (DrawStatus == Resizing)
     	{   [[NSColor lightGrayColor] set];
@@ -1119,17 +1135,17 @@ static void initClassVars()
     	}*/
     }
 
-#ifdef __APPLE__    // Workaround: display Single Line Fonts
+#   ifdef __APPLE__    // Workaround: display Single Line Fonts
     if (antialias)
         VHFSetAntialiasing(antialias);
-#endif
+#   endif
 }
 
 /* extract number from data, increment this number and add it to data. Then write string to drawText
  */
 - (void)setSerialTextFor:(NSTextView*)drawText withOffset:(int)o setData:(BOOL)setData
-{   int		i, lennew, lentot;
-    NSString	*string, *prefix, *valueStr;
+{   int         i, lennew, lentot;
+    NSString    *string, *prefix, *valueStr;
 
     if (attributedString && o)
     {	NSRange		range;
@@ -1548,7 +1564,8 @@ static void initClassVars()
  * back to nil.
  * For further explanation of the two "ack!" lines, see edit:in: above.
  *
- * modified: 2009-02-12
+ * modified: 2012-01-04 (if isLocked, we don't adjust the size of the text box)
+ *           2009-02-12
  */
 - (void)textDidEndEditing:(NSNotification *)notification
 {   NSTextView          *fe = [notification object];
@@ -1584,12 +1601,14 @@ static void initClassVars()
 
             oldBounds = [self extendedBoundsWithScale:[graphicView scaleFactor]];
             [fe setVerticallyResizable:YES];
-            [fe sizeToFit];
-            bounds = [fe frame];
-            bounds.size = [fe bounds].size;
-            bounds.size.width  += correction;
-            bounds.size.height += correction;   // origin = upper/left here!
-            bounds = [editView convertRect:bounds toView:graphicView];  // ack!
+            if ( ! isLocked )
+            {   [fe sizeToFit];
+                bounds = [fe frame];
+                bounds.size = [fe bounds].size;
+                bounds.size.width  += correction;
+                bounds.size.height += correction;   // origin = upper/left here!
+                bounds = [editView convertRect:bounds toView:graphicView];  // ack!
+            }
             //bounds = [graphicView centerScanRect:bounds];	/* center pixels */
             //redrawRect = NSUnionRect(bounds, redrawRect);
 
@@ -1705,7 +1724,7 @@ static void initClassVars()
             [pathG setAngle:-rotAngle withCenter:bRect.origin];
             /* FIXME: stroked fonts shouldn't be filled! */
             [pathG setFilled:YES optimize:NO];
-            if (group)
+            if ( group && [pathG count] )
                 [group addObject:pathG];
         }
 
@@ -1715,9 +1734,9 @@ static void initClassVars()
     /*
      * we draw the text inside editView, then writing it to postscript-data for interpretation
      */
-    {	PSImportSub	*psImport;
-        NSData		*psData;
-        NSMutableArray	*list;
+    {	PSImportSub     *psImport;
+        NSData          *psData;
+        NSMutableArray  *list;
 
         if (!editView)
         {
@@ -1796,19 +1815,21 @@ static void initClassVars()
     [aCoder encodeValuesOfObjCTypes:"@", &attributedString];
     [aCoder encodeValuesOfObjCTypes:"@", &font];
     [aCoder encodeValuesOfObjCTypes:"ff", &rotAngle, &aspectRatio];
-    [aCoder encodeValuesOfObjCTypes:"{NSRect=ffff}", &bounds];
+    //[aCoder encodeValuesOfObjCTypes:"{NSRect=ffff}", &bounds];    // doesn't work with 64 bit !
+    [aCoder encodeRect:bounds];                 // 2012-01-08
     [aCoder encodeValuesOfObjCTypes:"ccc", &isSerialNumber, &fitHorizontal, &centerVertical];
     [aCoder encodeObject:fillColor];
     [aCoder encodeObject:endColor];
     [aCoder encodeValuesOfObjCTypes:"ff", &graduateAngle, &stepWidth];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    [aCoder encodePoint:radialCenter];          // 2012-01-08
 }
 - (id)initWithCoder:(NSCoder *)aDecoder
 {   int	version;
 
     [super initWithCoder:aDecoder];
     version = [aDecoder versionForClassName:@"VText"];
-    if ( version<4 )	// 13.02.00
+    if ( version < 4 )	// 13.02.00
     {   float	length;
         [aDecoder decodeValuesOfObjCTypes:"i", &length];
     }
@@ -1825,7 +1846,8 @@ static void initClassVars()
     if ( version < 5 )	// 04.05.00
         [aDecoder decodeValuesOfObjCTypes:"{ffff}", &bounds];
     else
-        [aDecoder decodeValuesOfObjCTypes:"{NSRect=ffff}", &bounds];
+        //[aDecoder decodeValuesOfObjCTypes:"{NSRect=ffff}", &bounds];
+        bounds = [aDecoder decodeRect];         // 2012-01-08
     if ( version < 2 )
         [aDecoder decodeValuesOfObjCTypes:"c", &isSerialNumber];
     else if ( version < 3)
@@ -1834,9 +1856,10 @@ static void initClassVars()
         [aDecoder decodeValuesOfObjCTypes:"ccc", &isSerialNumber, &fitHorizontal, &centerVertical];
     if ( version >= 6)
     {   fillColor = [[aDecoder decodeObject] retain];
-        endColor = [[aDecoder decodeObject] retain];
+        endColor  = [[aDecoder decodeObject] retain];
         [aDecoder decodeValuesOfObjCTypes:"ff", &graduateAngle , &stepWidth];
-        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        radialCenter = [aDecoder decodePoint];  // 2012-01-08
     }
     if ( version < 1 )
     {   float	f;
@@ -1854,24 +1877,24 @@ static void initClassVars()
 - (id)propertyList
 {   NSMutableDictionary	*plist = [super propertyList];
 
-    [plist setObject:propertyListFromFloat(rotAngle) forKey:@"rotAngle"];
-    [plist setObject:propertyListFromFloat(aspectRatio) forKey:@"aspectRatio"];
+    [plist setObject:propertyListFromFloat(rotAngle)            forKey:@"rotAngle"];
+    [plist setObject:propertyListFromFloat(aspectRatio)         forKey:@"aspectRatio"];
     [plist setObject:[self richTextData] forKey:@"data"];
-    [plist setObject:propertyListFromNSRect(bounds) forKey:@"bounds"];
-    if (isSerialNumber) [plist setObject:@"YES" forKey:@"isSerialNumber"];
-    if (fitHorizontal) [plist setObject:@"YES" forKey:@"fitHorizontal"];
-    if (centerVertical) [plist setObject:@"YES" forKey:@"centerVertical"];
+    [plist setObject:propertyListFromNSRect(bounds)             forKey:@"bounds"];
+    if (isSerialNumber) [plist setObject:@"YES"                 forKey:@"isSerialNumber"];
+    if (fitHorizontal)  [plist setObject:@"YES"                 forKey:@"fitHorizontal"];
+    if (centerVertical) [plist setObject:@"YES"                 forKey:@"centerVertical"];
 
     if (fillColor != [NSColor blackColor])
-        [plist setObject:propertyListFromNSColor(fillColor) forKey:@"fillColor"];
+        [plist setObject:propertyListFromNSColor(fillColor)     forKey:@"fillColor"];
     if (endColor != [NSColor blackColor])
-        [plist setObject:propertyListFromNSColor(endColor) forKey:@"endColor"];
+        [plist setObject:propertyListFromNSColor(endColor)      forKey:@"endColor"];
     if (graduateAngle)
-        [plist setObject:propertyListFromFloat(graduateAngle) forKey:@"graduateAngle"];
+        [plist setObject:propertyListFromFloat(graduateAngle)   forKey:@"graduateAngle"];
     if (stepWidth != 7)
-        [plist setObject:propertyListFromFloat(stepWidth) forKey:@"stepWidth"];
+        [plist setObject:propertyListFromFloat(stepWidth)       forKey:@"stepWidth"];
     if (radialCenter.x != 0.5 && radialCenter.y != 0.5)
-        [plist setObject:propertyListFromNSPoint(radialCenter) forKey:@"radialCenter"];
+        [plist setObject:propertyListFromNSPoint(radialCenter)  forKey:@"radialCenter"];
 
     return plist;
 }
@@ -1879,12 +1902,12 @@ static void initClassVars()
 {
     [self setParameter];
     [super initFromPropertyList:plist inDirectory:directory];
-    rotAngle = [plist floatForKey:@"rotAngle"];
-    aspectRatio = [plist floatForKey:@"aspectRatio"];
+    rotAngle       = [plist floatForKey:@"rotAngle"];
+    aspectRatio    = [plist floatForKey:@"aspectRatio"];
     [self setRichTextData:[plist objectForKey:@"data"]];    // this also loads the text colors for paragraphs !
     bounds = rectFromPropertyList([plist objectForKey:@"bounds"]);
     isSerialNumber = ([plist objectForKey:@"isSerialNumber"] ? YES : NO);
-    fitHorizontal = ([plist objectForKey:@"fitHorizontal"] ? YES : NO);
+    fitHorizontal  = ([plist objectForKey:@"fitHorizontal"]  ? YES : NO);
     centerVertical = ([plist objectForKey:@"centerVertical"] ? YES : NO);
     [self font];
 
