@@ -1,11 +1,12 @@
 /*
  * IPBasicLevel.h
  *
- * Copyright (C) 1996-2008 by vhf interservice GmbH
+ * Copyright (C) 1996-2011 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * Created:  1996-04-23
- * Modified: 2008-03-17
+ * Modified: 2011-05-30 (labelField, -setLabel: added)
+ *           2011-05-28 (excludeSwitch, -setExcluded: added)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -34,6 +35,9 @@
 @interface IPBasicLevel:NSPanel
 {
     id		window;         // the inspector panel
+
+    id      labelField;
+    id		excludeSwitch;
     id		lockSwitch;
 }
 
@@ -46,7 +50,9 @@
 - (NSString*)name;
 - (void)update:sender;
 
-- (void)setLock:sender;
+- (void)setLabel:sender;    // set label
+- (void)setExcluded:sender; // exclude from processing
+- (void)setLock:sender;     // lock position for mouse handling
 
 /* delegate methods
  */
