@@ -1,11 +1,13 @@
 /* LayerObject.m
  * Object managing a single layer and its attributes
  *
- * Copyright (C) 1996-2008 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * Created:  1996-03-07
- * Modified: 2008-11-12 (layerDict, clean-up)
+ * Modified: 2012-01-25 (-setInvisible:, -invisible)
+ *           2012-01-24 (-stepsForDip:)
+ *           2008-11-12 (layerDict, clean-up)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -84,6 +86,7 @@
     settleBefore    = NO;
     inlay           = NO;
     levelingX       = levelingY = 2;
+    invisible       = NO;
 
     return self;
 }
@@ -210,9 +213,7 @@
 /* layers which can't be edited */
 - (BOOL)isPassive
 {
-    if (LAYER_PASSIVE)
-        return YES;
-    return NO;
+    return (LAYER_PASSIVE) ? YES : NO;
 }
 /* all layers which should have a side correction icon */
 - (BOOL)fastSideSelection
@@ -223,17 +224,20 @@
     return NO;
 }
 
-- (void)setString:(NSString *)aString	{ [string release]; string = [aString retain]; }
-- (NSString *)string			{ return string; }
+- (void)setString:(NSString *)aString   { [string release]; string = [aString retain]; }
+- (NSString *)string                    { return string; }
 
-- (void)setTag:(int)newTag		{ tag = newTag; }
-- (int)tag				{ return tag; }
+- (void)setTag:(int)newTag              { tag = newTag; }
+- (int)tag                              { return tag; }
 
-- (void)setColor:(NSColor *)aColor	{ color=aColor; }
-- (NSColor *)color			{ return color; }
+- (void)setColor:(NSColor *)aColor      { color=aColor; }
+- (NSColor *)color                      { return color; }
 
-- (void)setState:(int)flag		{ state = flag; }
-- (int)state				{ return state; }
+- (void)setState:(int)flag              { state = flag; }
+- (int)state                            { return state; }
+
+- (void)setInvisible:(int)flag          { invisible = flag; }
+- (int)invisible                        { return invisible; }
 
 - (void)setEditable:(BOOL)flag
 {
@@ -241,9 +245,9 @@
     if (editable)
         state = YES;
 }
-- (BOOL)editable			{ return ( !state ) ? NO : editable; }
+- (BOOL)editable                        { return ( !state ) ? NO : editable; }
 
-- (int)type				{ return type; }
+- (int)type                             { return type; }
 - (void)setType:(int)newType
 {
     type = newType;
@@ -536,16 +540,17 @@
         calcSteps = [self stepsForDip:dippingDepth];
     return calcSteps;
 }
-- (OutputSteps)stepsForDip:(float)dip
-{   OutputSteps	dipSteps;
-    float	h, prev = 0.0;
-    int		i;
+- (OutputSteps)stepsForDip:(float)dip   // [mm]
+{   OutputSteps dipSteps;
+    float       h, prev = 0.0;
+    int         i;
 
     dipSteps.count = 0;
 
     if (!stepwise || dip == 0.0)
     {   dipSteps.count = 1;
-        dipSteps.step[0] = -dippingDepth;
+        //dipSteps.step[0] = -dippingDepth;
+        dipSteps.step[0] = -dip;    // 2012-01-24
         return dipSteps;
     }
 
@@ -553,7 +558,8 @@
     {   dipSteps.step[0] = prev = -step[0];
         dipSteps.count ++;
     }
-    h = dippingDepth - step[0] - step[2];
+    //h = dippingDepth - step[0] - step[2];
+    h = dip - step[0] - step[2];    // 2012-01-24
     if (h > TOLERANCE)
         for (i=0; i<numSteps; i++)
         {   dipSteps.step[dipSteps.count] = prev = prev - h/numSteps;
@@ -704,6 +710,8 @@
     if (settleBefore) [plist setObject:@"YES"                       forKey:@"settleBefore"];
     [plist setObject:propertyListFromFloat(settle)                  forKey:@"settle"];
 
+    if (invisible) [plist setObject:@"YES"                          forKey:@"invisible"];
+
     return plist;
 }
 - (id)initFromPropertyList:(id)plist inDirectory:(NSString *)directory
@@ -752,6 +760,8 @@
     settleBefore    = ([plist objectForKey:@"settleBefore"] ? YES : NO);
     settle          = [plist floatForKey:@"settle"];
 
+    invisible       = ([plist objectForKey:@"invisible"] ? YES : NO);
+
     return self;
 }
 
