@@ -1,11 +1,11 @@
 /* TPScale.h
  * Transform panel for scaling objects
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2011 by vhf interservice GmbH
  * Author: Georg Fleischmann
  *
  * Created:  1996-04-22
- * Modified: 2002-07-07
+ * Modified: 2011-03-03
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -31,11 +31,13 @@
 
 @interface TPScale:TPBasicLevel
 {
+    id  scalePopup;         // relative, scale to size
     id	scaleXField;
     id	scaleYField;
-    id	uniformScaleSwitch;
+    id	uniformScaleSwitch; // on = keep aspekt ratio
 }
 
+- (void)setHowToScale:sender;
 - (void)setUniformScale:sender;
 - (void)setScale:sender;
 
