@@ -1,11 +1,11 @@
 /* Type1Font.m
  * Type 1 Font object used for the type 1 import
  *
- * Copyright (C) 1996-2006 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2000-11-26
- * modified: 2006-10-21
+ * modified: 2012-02-07 (-writeToFile: 2x use -writeToFile:...encoding:error:)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -138,14 +138,14 @@
     if ( [pDict objectForKey:@"blueScale"] )
         privateDict.blueScale = [[pDict objectForKey:@"blueScale"] floatValue];
     privateDict.blueShift = [[pDict objectForKey:@"blueShift"] longValue];
-    if ( [[pDict objectForKey:@"blueValues"] count] == 6 )
+    if ( [(NSArray*)[pDict objectForKey:@"blueValues"] count] == 6 )
     {   int	i;
 
         for (i=0; i<6; i++)
             privateDict.blueValues[i] = [[[pDict objectForKey:@"blueValues"] objectAtIndex:i] floatValue];
     }
 
-    if ( [[pDict objectForKey:@"minFeature"] count] == 2 )
+    if ( [(NSArray*)[pDict objectForKey:@"minFeature"] count] == 2 )
     {   privateDict.minFeature[0] = [[[pDict objectForKey:@"minFeature"] objectAtIndex:0] floatValue];
         privateDict.minFeature[1] = [[[pDict objectForKey:@"minFeature"] objectAtIndex:1] floatValue];
     }
@@ -860,12 +860,12 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 /* fileName = "PATH/FILENAME.font" (font directory)
  */
 - (BOOL)writeToFile:(NSString*)filename
-{   BOOL		savedOk = NO;
+{   BOOL            savedOk = NO;
     NSString		*afmfilename = nil;
     NSString		*backupFilename = [filename stringByAppendingString:@"~"];
     NSString		*fileDirectory = nil;
-    NSFileManager	*fileManager = [NSFileManager defaultManager];
-    BOOL		isDirectory = YES;
+    NSFileManager   *fileManager = [NSFileManager defaultManager];
+    BOOL            isDirectory = YES;
 
 //encoding = [self standardEncoding];
 //encodingCnt = [self standardEncodingCnt];
@@ -978,7 +978,7 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 	//for (i=0; i<subrsCnt; i++)
 	for (i=0; i<privateDict.subrsCnt; i++)
 	{   NSString	*str = [NSString stringWithCString:(char*)privateDict.subrs[i].proc
-                                                    length:privateDict.subrs[i].length];
+                                                length:privateDict.subrs[i].length];
 
             eexecStr = [eexecStr stringByAppendingFormat:@"dup %d %d RD ", i, privateDict.subrs[i].length];
             eexecStr = [eexecStr stringByAppendingString:str];
@@ -1003,8 +1003,8 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
         }
 #endif
 	for (i=0; i<charStringCnt; i++)	/* georg */
-	{   NSString	*str=[NSString stringWithCString:(char*)charStrings[i].code
-                                                  length:charStrings[i].length];
+	{   NSString    *str=[NSString stringWithCString:(char*)charStrings[i].code
+                                              length:charStrings[i].length];
 
             eexecStr = [eexecStr stringByAppendingFormat:@"/%s %d RD ", charStrings[i].name, charStrings[i].length];
             eexecStr = [eexecStr stringByAppendingString:str];
@@ -1014,61 +1014,65 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 	eexecStr = [eexecStr stringByAppendingFormat:@"readonly put\n"];
 	eexecStr = [eexecStr stringByAppendingFormat:@"dup/FontName get exch definefont pop\n"];
 	eexecStr = [eexecStr stringByAppendingFormat:@"mark currentfile closefile\n"];
-#if 0
-{ NSString	*file = [NSString stringWithFormat:@"/Net/nesquick/Users/ilonka/Tempo/Test/testSubrs"];
-[eexecStr writeToFile:file atomically:YES];
-}
-#endif
-	// NXGetMemoryBuffer(eexecStream, &buf, &len, &maxlen);
-    {   char	*eExecCStr, *buffer;
+#   if 0
+        { NSString	*file = [NSString stringWithFormat:@"/Net/nesquick/Users/ilonka/Tempo/Test/testSubrs"];
+            [eexecStr writeToFile:file atomically:YES];
+        }
+#   endif
+        // NXGetMemoryBuffer(eexecStream, &buf, &len, &maxlen);
+        {   char	*eExecCStr, *buffer;
 
-        len = [eexecStr length]; // +1 -> 0
-        buffer = malloc(len+1);
-        [eexecStr getCString:buffer];
-        eExecCStr = (char*)encryptEexec((unsigned char*)buffer, len);
-        fontStr = [fontStr stringByAppendingString:[NSString stringWithCString:eExecCStr length:len*2]];
-        free(eExecCStr);
-        free(buffer);
-    }
-	// NXWrite(stream, encryptEexec(buf, len), len*2);
+            len = [eexecStr length]; // +1 -> 0
+            buffer = malloc(len+1);
+            [eexecStr getCString:buffer];
+            eExecCStr = (char*)encryptEexec((unsigned char*)buffer, len);
+            fontStr = [fontStr stringByAppendingString:[NSString stringWithCString:eExecCStr length:len*2]];
+            free(eExecCStr);
+            free(buffer);
+        }
+        // NXWrite(stream, encryptEexec(buf, len), len*2);
 
-	fontStr = [fontStr stringByAppendingFormat:@"\n"];
-	for (i=0;i<512;i++)
-		fontStr = [fontStr stringByAppendingFormat:@"0"];
-	fontStr = [fontStr stringByAppendingFormat:@"\ncleartomark"];
+        fontStr = [fontStr stringByAppendingFormat:@"\n"];
+        for (i=0;i<512;i++)
+            fontStr = [fontStr stringByAppendingFormat:@"0"];
+        fontStr = [fontStr stringByAppendingFormat:@"\ncleartomark"];
 
-        savedOk = [fontStr writeToFile:filename atomically:YES];
+        //savedOk = [fontStr writeToFile:filename atomically:YES];
+        savedOk = [fontStr writeToFile:filename atomically:YES
+                              encoding:NSUTF8StringEncoding error:NULL];    // >= 10.5
 
         afmStr = [NSString stringWithFormat:@"StartFontMetrics 2.0\n"];
-	afmStr = [afmStr stringByAppendingFormat:@"Comment %s\n", COPYRIGHT];
-	afmStr = [afmStr stringByAppendingFormat:@"FontName %@\n", fontName];
-	afmStr = [afmStr stringByAppendingFormat:@"EncodingScheme AdobeStandardEncoding\n"];
-	afmStr = [afmStr stringByAppendingFormat:@"FullName %@\n", fontInfo.fullName];
-	afmStr = [afmStr stringByAppendingFormat:@"FamilyName %@\n", fontInfo.familyName];
-	afmStr = [afmStr stringByAppendingFormat:@"Weight %@\n", fontInfo.weight];
-	afmStr = [afmStr stringByAppendingFormat:@"ItalicAngle 0.0\n"];
-	afmStr = [afmStr stringByAppendingFormat:@"IsFixedPitch %s\n", (fontInfo.isFixedPitch) ? "true" : "false"];
-	afmStr = [afmStr stringByAppendingFormat:@"UnderlinePosition %d\n", fontInfo.underlinePosition];
-	afmStr = [afmStr stringByAppendingFormat:@"UnderlineThickness %d\n", fontInfo.underlineThickness];
-	afmStr = [afmStr stringByAppendingFormat:@"Version %@\n", fontInfo.version];
-	afmStr = [afmStr stringByAppendingFormat:@"Notice %s\n", COPYRIGHT];
-	afmStr = [afmStr stringByAppendingFormat:@"FontBBox %.0f %.0f %.0f %.0f\n", fontBBox[0], fontBBox[1], fontBBox[2], fontBBox[3]];
-	afmStr = [afmStr stringByAppendingFormat:@"CapHeight %.0f\n", capHeight];
-	afmStr = [afmStr stringByAppendingFormat:@"XHeight %.0f\n", xHeight];
-	afmStr = [afmStr stringByAppendingFormat:@"Descender %.0f\n", descender];
-	afmStr = [afmStr stringByAppendingFormat:@"Ascender %.0f\n", capHeight];
+        afmStr = [afmStr stringByAppendingFormat:@"Comment %s\n", COPYRIGHT];
+        afmStr = [afmStr stringByAppendingFormat:@"FontName %@\n", fontName];
+        afmStr = [afmStr stringByAppendingFormat:@"EncodingScheme AdobeStandardEncoding\n"];
+        afmStr = [afmStr stringByAppendingFormat:@"FullName %@\n", fontInfo.fullName];
+        afmStr = [afmStr stringByAppendingFormat:@"FamilyName %@\n", fontInfo.familyName];
+        afmStr = [afmStr stringByAppendingFormat:@"Weight %@\n", fontInfo.weight];
+        afmStr = [afmStr stringByAppendingFormat:@"ItalicAngle 0.0\n"];
+        afmStr = [afmStr stringByAppendingFormat:@"IsFixedPitch %s\n", (fontInfo.isFixedPitch) ? "true" : "false"];
+        afmStr = [afmStr stringByAppendingFormat:@"UnderlinePosition %d\n", fontInfo.underlinePosition];
+        afmStr = [afmStr stringByAppendingFormat:@"UnderlineThickness %d\n", fontInfo.underlineThickness];
+        afmStr = [afmStr stringByAppendingFormat:@"Version %@\n", fontInfo.version];
+        afmStr = [afmStr stringByAppendingFormat:@"Notice %s\n", COPYRIGHT];
+        afmStr = [afmStr stringByAppendingFormat:@"FontBBox %.0f %.0f %.0f %.0f\n", fontBBox[0], fontBBox[1], fontBBox[2], fontBBox[3]];
+        afmStr = [afmStr stringByAppendingFormat:@"CapHeight %.0f\n", capHeight];
+        afmStr = [afmStr stringByAppendingFormat:@"XHeight %.0f\n", xHeight];
+        afmStr = [afmStr stringByAppendingFormat:@"Descender %.0f\n", descender];
+        afmStr = [afmStr stringByAppendingFormat:@"Ascender %.0f\n", capHeight];
 
-	afmStr = [afmStr stringByAppendingFormat:@"StartCharMetrics %d\n", metricsCnt];
-	/* metrics */
-	for (i=0; i<metricsCnt; i++)
+        afmStr = [afmStr stringByAppendingFormat:@"StartCharMetrics %d\n", metricsCnt];
+        /* metrics */
+        for (i=0; i<metricsCnt; i++)
         {
             afmStr = [afmStr stringByAppendingFormat:@"C %d ; WX %.0f ; N %@ ; B %.0f %.0f %.0f %.0f ;\n", metrics[i].index, metrics[i].width, metrics[i].name, metrics[i].bbox[0], metrics[i].bbox[1], metrics[i].bbox[2], metrics[i].bbox[3]];
         }
-	afmStr = [afmStr stringByAppendingFormat:@"EndCharMetrics\n"];
+        afmStr = [afmStr stringByAppendingFormat:@"EndCharMetrics\n"];
         // composites
-	afmStr = [afmStr stringByAppendingFormat:@"EndFontMetrics\n"];
+        afmStr = [afmStr stringByAppendingFormat:@"EndFontMetrics\n"];
 
-        savedOk = [afmStr writeToFile:afmfilename atomically:YES];
+        //savedOk = [afmStr writeToFile:afmfilename atomically:YES];
+        savedOk = [afmStr writeToFile:afmfilename atomically:YES
+                             encoding:NSUTF8StringEncoding error:NULL]; // >= 10.5
     }
     else
         NSRunAlertPanel(SAVE_TITLE, DIR_NOT_WRITABLE, nil, nil, nil);
