@@ -242,12 +242,12 @@ static __inline__ char* gotoNextData(char *c)
             {	switch (token->index)
                 {
                     case T_COMMENT:
-                        cp += strcspn(cp, NEWLINE);		/* goto end of line */
+                        cp += strcspn(cp, NEWLINE);		// goto end of line
                         break;
                     case T_VERSION:
                         cp = gotoNextData(cp);
                         cString = getString(cp);
-                        [fiDict setObject:[NSString stringWithCString:cString] forKey:@"version"];
+                        [fiDict setObject:[NSString stringWithUTF8String:cString] forKey:@"version"];
                         if (!(cp = strstr(cp, "def")))
                         {   printf("Type1Import: No 'def' for /version found");
                             return 0;
@@ -267,7 +267,7 @@ static __inline__ char* gotoNextData(char *c)
                     case T_NOTICE:
                         cp = gotoNextData(cp);
                         cString = getString(cp);
-                        [fiDict setObject:[NSString stringWithCString:cString] forKey:@"notice"];
+                        [fiDict setObject:[NSString stringWithUTF8String:cString] forKey:@"notice"];
                         if (!(cp = strstr(cp, "def")))
                         {   printf("Type1Import: No 'def' for /Notice found");
                             return 0;
@@ -277,7 +277,7 @@ static __inline__ char* gotoNextData(char *c)
                     case T_COPYRIGHT:
                         cp = gotoNextData(cp);
                         cString = getString(cp);
-                        [fiDict setObject:[NSString stringWithCString:cString] forKey:@"copyright"];
+                        [fiDict setObject:[NSString stringWithUTF8String:cString] forKey:@"copyright"];
                         if (!(cp = strstr(cp, "def")))
                         {   printf("Type1Import: No 'def' for /Copyright found");
                             return 0;
@@ -287,7 +287,7 @@ static __inline__ char* gotoNextData(char *c)
                     case T_FULLNAME:
                         cp += strlen(token->string);
                         cString = getString(cp);
-                        [fiDict setObject:[NSString stringWithCString:cString] forKey:@"fullName"];
+                        [fiDict setObject:[NSString stringWithUTF8String:cString] forKey:@"fullName"];
                         if (!(cp = strstr(cp, "def")))
                         { printf("Type1Import: No 'def' for /FullName found"); return 0; }
                         cp += strlen("def");
@@ -295,7 +295,7 @@ static __inline__ char* gotoNextData(char *c)
                     case T_FAMILYNAME:
                         cp += strlen(token->string);
                         cString = getString(cp);
-                        [fiDict setObject:[NSString stringWithCString:cString] forKey:@"familyName"];
+                        [fiDict setObject:[NSString stringWithUTF8String:cString] forKey:@"familyName"];
                         if (!(cp = strstr(cp, "def")))
                         { printf("Type1Import: No 'def' for /FamilyName found"); return 0; }
                         cp += strlen("def");
@@ -303,7 +303,7 @@ static __inline__ char* gotoNextData(char *c)
                     case T_WEIGHT:
                         cp = gotoNextData(cp);
                         cString = getString(cp);
-                        [fiDict setObject:[NSString stringWithCString:cString] forKey:@"weight"];
+                        [fiDict setObject:[NSString stringWithUTF8String:cString] forKey:@"weight"];
                         if (!(cp = strstr(cp, "def")))
                         {   printf("Type1Import: No 'def' for /Weight found");
                             return 0;
@@ -617,7 +617,7 @@ encodingCnt = [fontObject standardEncodingCnt];
                     case T_SOURCE:
                         cp += strlen(token->string);
                         cString = getString(cp);
-                        [pDict setObject:[NSString stringWithCString:cString] forKey:@"source"];
+                        [pDict setObject:[NSString stringWithUTF8String:cString] forKey:@"source"];
                         if (!(cp = strstr(cp, "def")))
                         {   printf("Type1Import, PrivateDict, /Source: unexpected end of file");
                             return 0;
@@ -627,7 +627,7 @@ encodingCnt = [fontObject standardEncodingCnt];
                     case T_OTHERSUBRS:
                         cp += strlen(token->string);
                         cString = getOtherSubrs(cp);
-                        [pDict setObject:[NSString stringWithCString:cString] forKey:@"otherSubrs"];
+                        [pDict setObject:[NSString stringWithUTF8String:cString] forKey:@"otherSubrs"];
                         cp += strlen(cString);
                         break;
                     case T_SUBRS:
@@ -804,7 +804,7 @@ encodingCnt = [fontObject standardEncodingCnt];
                     case T_FONTNAME:
                         cp = gotoNextData(cp);
                         cString = getName(cp)+1;
-                        [fontObject setFontName:[NSString stringWithCString:cString]];
+                        [fontObject setFontName:[NSString stringWithUTF8String:cString]];
                         if (!(cp = strstr(cp, "def")))
                         {   printf("Type1Import: No 'def' for FontName found");
                             return 0;
@@ -915,10 +915,10 @@ encodingCnt = [fontObject standardEncodingCnt];
     switch (code)
     {
         case ERR_STACKUNDERFLOW:
-            printf("%s, %s: %%%%[ Error:stackunderflow; OffendingCommand:%s ]%%%%\n", [time cString], [type1Str cString], name);
+            printf("%s, %s: %%%%[ Error:stackunderflow; OffendingCommand:%s ]%%%%\n", [time UTF8String], [type1Str UTF8String], name);
             break;
         default:
-            printf("%s, %s: %%%%[ Error:unknown Error Code, %s ]%%%%\n", [time cString], [type1Str cString], name);
+            printf("%s, %s: %%%%[ Error:unknown Error Code, %s ]%%%%\n", [time UTF8String], [type1Str UTF8String], name);
     }
 
     return self;
