@@ -1,11 +1,12 @@
 /* UpdateController.m
  * Checking for Updates...
  *
- * Copyright 2010 by vhf interservice GmbH
+ * Copyright 2010-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2010-05-27
- * modified: 2010-07-17
+ * modified: 2012-02-07 (-connectionDidFinishLoading: use -writeToFile:...encoding:error:)
+ *           2011-03-30 (-checkForUpdates: test for Prefs_DisableAutoUpdate)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -32,6 +33,8 @@
 #include "locations.h"
 #include "messages.h"
 #include "CenonModuleMethods.h"
+#include "PreferencesMacros.h"  // Prefs_DisableAutoUpdate
+#include <VHFShared/VHFStringAdditions.h>       // -writeToFile:...
 
 static UpdateController *sharedInstance = nil;  // default object
 static NSString         *checkMarkStr = nil;    // the checkmark
@@ -168,7 +171,8 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         [installedDict setObject:[NSArray arrayWithObjects:version, (date) ? date : @"",
                                   (serial) ? serial : @"", (netId) ? netId : @"", nil]
                           forKey:name];
-        printf("installedModules: %s %s %s %s %s\n", [name UTF8String], [version UTF8String],
+        if ( !isAutoCheck )
+            printf("Installed Modules: %s = %s (%s) %s %s\n", [name UTF8String], [version UTF8String],
                (date) ? [date UTF8String] : "", (serial) ? [serial UTF8String] : "",
                (netId) ? [netId UTF8String] : "");
     }
@@ -178,16 +182,29 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 /* tries to download update.plist from vhf server
  */
 - (void)checkForUpdates:sender
-{   NSDictionary    *installedModules = [self installedModules];
+{   NSDictionary    *installedModules;
     NSArray         *keys;
     NSURLRequest    *connectionRequest;
     NSMutableString *urlStr = [NSMutableString string];
     NSString        *appVersion = [(App*)NSApp version];        // 3.9.2
     NSString        *appDate    = [(App*)NSApp compileDate];    // 2010-06-28
+    NSUserDefaults  *userDefaults = [NSUserDefaults standardUserDefaults];
+    NSString        *skipVersion     = [userDefaults objectForKey:@"skipUpdateVersion"];
+    NSDate          *lastUpdateCheck = [userDefaults objectForKey:@"lastUpdateCheck"];
     int             i;
 
+    isAutoCheck = ([sender isKindOfClass:[NSMenuItem class]]) ? NO : YES;
+    if ( isAutoCheck && Prefs_DisableAutoUpdate )
+        return; // automatic check is disabled in preferences
+    if ( isAutoCheck && lastUpdateCheck &&
+         [[NSDate date] timeIntervalSinceDate:lastUpdateCheck] < 3600*24*7 )
+        return; // not yet one week -> no check
+
+    installedModules = [self installedModules];
     [urlStr appendString:@"http://www.cenon.info/cgi-bin/updateCenon"];
     [urlStr appendFormat:@"?n=%@&v=%@&d=%@", APPNAME, appVersion, appDate];
+    if ( [skipVersion length] ) // version the user wants to skip
+        [urlStr appendFormat:@"&sk=%@", skipVersion];
     //[urlStr appenFormat:@"&p="];    // TODO: pass plattform to update script
     for (keys = [installedModules allKeys], i=0; i < [keys count]; i++)
     {   int         n = i+1;
@@ -216,6 +233,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         [progressIndicator setIndeterminate:YES];
         [progressIndicator startAnimation:nil];
         [progressPanel makeKeyAndOrderFront:self];
+        [userDefaults setObject:[NSDate date] forKey:@"lastUpdateCheck"];   // date of this check
     }
 #if 0   // this downloads a plist file directly (what, if it ends up in a proxy ?)
     NSURL           *url = [NSURL URLWithString:@"http://www.cenon.info/update/update.plist"];
@@ -237,7 +255,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
 /* check for available updates, if available display panel
  * created:  2010-05-27
- * modified: 2010-09-22
+ * modified: 2011-02-22 (don't offer update of uninstalled versions)
  * FIXME: this will not work with GNUstep, we need to access the infoDict correctly or add Apple info
  */
 - (void)checkForUpdateAndDisplayPanel
@@ -266,7 +284,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     updateDict = [[NSDictionary dictionaryWithContentsOfFile:updateFile] retain];
     updateKeys = [updateDict allKeys];
     newVersion = [[updateDict objectForKey:appName] objectForKey:@"v"];
-    if ( newVersion && skipVersion && [newVersion isEqual:skipVersion] )
+    if ( isAutoCheck && newVersion && skipVersion && [newVersion isEqual:skipVersion] )
         return;
 
     /* Installed versions of App, Modules, Fonts, etc. */
@@ -275,7 +293,9 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     //infoVersion = [[(App*)NSApp infoVersionNo] stringValue];    // ex: 3.9.1 pre 1 (2010-02-13)
     [installedDict setObject:[NSArray arrayWithObjects:appVersion, appDate, nil] forKey:appName];
     [checkOrder addObject:appName];
-    printf("checkForUpdates: %s %s %s\n", [appName UTF8String], [appVersion UTF8String], [appDate UTF8String]);
+    if ( !isAutoCheck )
+        printf("Installed App:     %s = %s (%s)\n", [appName UTF8String], [appVersion UTF8String], [appDate UTF8String]);
+    // FIXME: this is basically duplicating -installedModules and should be united
     for ( i=0; i<[modules count]; i++ ) // modules
     {   NSBundle        *bundle = [modules objectAtIndex:i]; // our loaded modules
         NSDictionary    *infoDict = [bundle infoDictionary];
@@ -287,11 +307,12 @@ static NSString         *checkMarkStr = nil;    // the checkmark
             date = [[[bundle principalClass] instance] compileDate];
         [installedDict setObject:[NSArray arrayWithObjects:version, date, nil] forKey:name];
         [checkOrder addObject:name];
-        printf("checkForUpdates: %s %s %s\n", [name UTF8String], [version UTF8String], (date) ? [date UTF8String] : "");
+        //if ( !isAutoCheck )
+        //    printf("Installed Modules: %s = %s (%s)\n", [name UTF8String], [version UTF8String], (date) ? [date UTF8String] : "");
     }
     if ( [updateDict objectForKey:@"order"] )
         checkOrder = [updateDict objectForKey:@"order"];
-    /* Check for install paths of items given in update.plist and add their versions etc. */
+    /* Check for install paths of additional items given in update.plist and add their versions etc. */
     for ( i=0; i<[updateKeys count]; i++ )
     {   NSString        *key = [updateKeys objectAtIndex:i], *path, *pathU = nil;
         NSDictionary    *uDict = [updateDict objectForKey:key];
@@ -320,7 +341,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     [dataDict setObject:[NSMutableArray array] forKey:@"url"];          // download link
 
     /* Compare available updates with installed versions */
-    for ( i=0; i<[checkOrder count]; i++ )  // Cenon, Astro, CAM, ...
+    for ( i=0; i<[checkOrder count]; i++ )  // CAM, AstroFractal, Astro, Cenon
     {   NSString        *key = [checkOrder objectAtIndex:i];
         NSDictionary    *uDict  = [updateDict objectForKey:key];    // v, s, l, i, n, r, d
         NSArray         *iArray = [installedDict objectForKey:key];
@@ -335,7 +356,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         /* is item eclipsed in rel "r" (ex: Cenon+Module.pkg eclipses Cenon.pkg alone) ? */
         {   BOOL    eclipsed = NO;
 
-            for ( j=0; j<[instKeys count]; j++ )    // all installed stuff
+            for ( j=0; j<[instKeys count]; j++ )    // all installed stuff can eclipse
             {   NSString        *instKey = [instKeys objectAtIndex:j];
                 NSDictionary    *uDictI = [updateDict objectForKey:instKey];
                 NSArray         *relArray;
@@ -344,7 +365,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
                 if ( ![uDictI isKindOfClass:[NSDictionary class]] )
                     continue;
                 relArray = [uDictI objectForKey:@"r"];
-                for (k=0; k<[relArray count]; k++ ) // and it's relationships
+                for (k=0; k<[relArray count]; k++ ) // it's relationships: "-" will eclipse
                 {   NSString    *relKey = [relArray objectAtIndex:k];
 
                     if ( ! [relKey hasPrefix:@"-"] )
@@ -363,11 +384,17 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         /* compare installed versions with available updates */
         if ( iArray )   // installed item
         {   NSString    *v = (mVersion) ? mVersion : version;   // we compile module version
-            if ( [[iArray objectAtIndex:0] compare:(id)v] == NSOrderedAscending )   // version > installed
+
+            if ( [v appearanceCountOfCharacter:'.'] <= 1 )  // workaround for single dot version numbers "1.11" -> "1.1.1"
+                v = @"0.0.0"; // this is old anyway
+            if ( [[iArray objectAtIndex:0] compare:(id)v options:NSNumericSearch] == NSOrderedAscending )   // installed < version
                 updateAvailable = YES;
-            else
+            // TODO: if version is the same, compare date
+            else    // installed >= version
                 continue;
         }
+        else    // 2011-02-22: not installed -> we don't offer item for installation
+            continue;
 
         /* check, wether to install module instead of Package (ex: CenonCAM + Astro-Module) */
         url = ([uDict objectForKey:@"d"]) ? [uDict objectForKey:@"d"] : @"";
@@ -380,7 +407,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
                 if ( ![uDictI isKindOfClass:[NSDictionary class]] )
                     continue;
                 /* if something is installed, then we installed a Cenon-Package already */
-                if ( [[dataDict objectForKey:@"updateKey"] count] )
+                if ( [(NSArray*)[dataDict objectForKey:@"updateKey"] count] )
                 {   installModule = YES;
                     url = [uDict objectForKey:@"md"];
                     version = mVersion;
@@ -406,14 +433,20 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
             price = nil;
             if ( priceDict )
-            {   NSString    *v = ([uDict objectForKey:@"mv"]) ? [uDict objectForKey:@"mv"] : version;
+            {   //NSString    *v = ([uDict objectForKey:@"mv"]) ? [uDict objectForKey:@"mv"] : version;
+                NSString    *v = ([iArray count]) ? [iArray objectAtIndex:0] : appVersion;  // version of installed module
                 NSRange     range = [v rangeOfString:@"." options:NSBackwardsSearch];
 
                 if ( range.length )
                 {   NSString    *vMajor = [v substringToIndex:range.location]; // "3.9", "1.1"
-                    price = [priceDict objectForKey:vMajor];
-                    if ( [price isEqual:@"0.0"] )
-                        price = NSLocalizedString(@"free", NULL);
+
+                    if ( iArray )   // installed
+                    {   price = [priceDict objectForKey:vMajor];
+                        if ( [price isEqual:@"0.0"] )
+                            price = NSLocalizedString(@"free", NULL);
+                    }
+                    else            // not installed (2011-02-22)
+                        price = [priceDict objectForKey:@"0"];
                 }
             }
             if ( !price )
@@ -421,14 +454,14 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         }
         [[dataDict objectForKey:@"price"]     addObject:price];
         if ( ! installModule && [uDict objectForKey:@"mv"] )    // version
-            version = [version stringByAppendingFormat:@" / %@", [uDict objectForKey:@"mv"]];
+            version = [version stringByAppendingFormat:@" (%@)", [uDict objectForKey:@"mv"]];
         [[dataDict objectForKey:@"version"]   addObject:version];
     }
 
 
     /* Add additional items for installed items (Fonts, Ephemeris, ...)
      */
-    for ( i=0; i<[[dataDict objectForKey:@"updateKey"] count]; i++ )
+    for ( i=0; i<[(NSArray*)[dataDict objectForKey:@"updateKey"] count]; i++ )
     {   NSString        *key = [[dataDict objectForKey:@"updateKey"] objectAtIndex:i];
         NSDictionary    *uDict  = [updateDict objectForKey:key];    // v, s, l, i, n, r
         NSArray         *relArray = [uDict objectForKey:@"r"];
@@ -448,12 +481,12 @@ static NSString         *checkMarkStr = nil;    // the checkmark
             /* check available version against installed stuff */
             iArray = [installedDict objectForKey:key];
             if ( [instKeys containsObject:key] &&
-                 ([iArray count] && [[iArray objectAtIndex:0] compare:(id)version] != NSOrderedAscending) )
+                 ([iArray count] && [[iArray objectAtIndex:0] compare:(id)version options:NSNumericSearch] != NSOrderedAscending) )
                 continue;   // installed and up to date
             /* compare installed versions with available updates */
             if ( iArray )   // installed item
             {
-                if ( [[iArray objectAtIndex:0] compare:(id)version] == NSOrderedAscending ) // version > installed
+                if ( [[iArray objectAtIndex:0] compare:(id)version options:NSNumericSearch] == NSOrderedAscending ) // version > installed
                     updateAvailable = YES;
             }
             [[dataDict objectForKey:@"install"]   addObject:@"0"];
@@ -520,7 +553,8 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         [[NSFileManager defaultManager] removeFileAtPath:updateFile handler:nil];   // <= 10.5
         //[[NSFileManager defaultManager] removeItemAtPath:updateFile error:NULL];  // >= 10.6
 
-    [panel makeKeyAndOrderFront:self];
+    if ( updateAvailable || !isAutoCheck )  // only display panel if update is available or manual
+        [panel makeKeyAndOrderFront:self];
 }
 
 /* Action methods
@@ -552,10 +586,11 @@ static NSString         *checkMarkStr = nil;    // the checkmark
                 if ( ![array count] && NSAppKitVersionNumber >= NSAppKitVersionNumber10_4)
                     array = NSSearchPathForDirectoriesInDomains(NSDesktopDirectory, NSUserDomainMask, YES);
 #endif
-                if ( array && [array count] )   // $HOME/Downloads/PACKAGE
+                if ( array && [array count] )   // 1. $HOME/Downloads/PACKAGE
                     path = [[array objectAtIndex:0] stringByAppendingPathComponent:fileName];
-                if ( !path )                    // /tmp/PACKAGE
-                    //path = vhfPathWithPathComponents(NSTemporaryDirectory(), fileName, nil);   // this is a nonsense folder: "/var/folders/8L/arg/-Tmp-"
+                if ( !path )                    // 2. /var/folders/8L/.../-Tmp-/PACKAGE
+                    path = vhfPathWithPathComponents(NSTemporaryDirectory(), fileName, nil);   // this is a nonsense folder: "/var/folders/8L/arg/-Tmp-"
+                if ( !path )                    // 3. /tmp/PACKAGE
                     path = vhfPathWithPathComponents(NSOpenStepRootDirectory(), @"tmp", fileName, nil);
                 fileCnt ++;
                 [downloadFiles addObject:[NSArray arrayWithObjects:url, path, nil]];    // url, path
@@ -646,7 +681,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         NSString        *url = [[updateDict objectForKey:[keyArray objectAtIndex:rowIx]] objectForKey:@"d"];
 
         if ( url && // checkmark only for downloadable items
-             (![[colArray objectAtIndex:rowIx] length] || ![[colArray objectAtIndex:rowIx] intValue]) )
+             (![(NSString*)[colArray objectAtIndex:rowIx] length] || ![[colArray objectAtIndex:rowIx] intValue]) )
             boolStr = @"1";
         [colArray replaceObjectAtIndex:rowIx withObject:boolStr];
     }
@@ -680,7 +715,9 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     {   NSString    *updateFile;
 
         updateFile = vhfPathWithPathComponents(vhfUserLibrary(APPNAME), @".update.plist", nil);
-        [string writeToFile:updateFile atomically:NO];
+        //[string writeToFile:updateFile atomically:NO];
+        [string writeToFile:updateFile atomically:NO
+                   encoding:NSUTF8StringEncoding error:NULL];   // >= 10.5
         [self checkForUpdateAndDisplayPanel];
     }
     // TODO: show "up to date" info
@@ -711,7 +748,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     sizeTotal = [response expectedContentLength];
     sizeDownl = 0;
 }
-- (void)download:(NSURLDownload *)download didReceiveDataOfLength:(unsigned)length
+- (void)download:(NSURLDownload *)download didReceiveDataOfLength:(NSUInteger)length
 {  
     sizeDownl += length;
     if (sizeTotal > 0)
@@ -785,7 +822,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
             for ( i=0, path=nil; i<[downloadFiles count]; i++)
             {   path = [[downloadFiles objectAtIndex:i] objectAtIndex:1];
-                if ( [fileManager fileExistsAtPath:path] )  // delete TAR-file
+                if ( [[path pathExtension] isEqual:@"tar"] && [fileManager fileExistsAtPath:path] ) // del TAR-file
                     [fileManager removeFileAtPath:path handler:nil];
                 // TODO: the following must not happen before the files are really installed !
                 /*path = [path stringByDeletingPathExtension];
@@ -951,7 +988,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 {
     [dataDict release];
     dataDict = [newDataDict retain];
-    rowCount = [[dataDict objectForKey:@"name"] count];
+    rowCount = [(NSArray*)[dataDict objectForKey:@"name"] count];
 }
 - (NSDictionary*)dataDict
 {
