@@ -1,11 +1,11 @@
 /* DXFExport.m
  * DXF export object
  *
- * Copyright (C) 2002-2003 by vhf interservice GmbH
+ * Copyright (C) 2002-2012 by vhf interservice GmbH
  * Author:   Ilonka Fleischmann
  *
  * created:  2002-04-25
- * modified: 2003-06-26
+ * modified: 2012-02-07 (-saveToFile: use writeToFile:...encoding:error:)
  *
  * This file is part of the vhf Export Library.
  *
@@ -36,7 +36,7 @@
 #include "DXFExport.h"
 #include "../VHFImport/dxfColorTable.h"
 #include "../VHFShared/types.h"
-#include "../VHFShared/VHFStringAdditions.h" // stringByReplacing
+#include "../VHFShared/VHFStringAdditions.h" // stringByReplacing:
 
 #define RES 25.389
 
@@ -116,12 +116,12 @@ int convertNSColorToDXFColor(NSColor *color)
 - (void)addLayer:(NSString*)name :(NSColor*)color :(int)attribut
 {   NSString		*nam = [name uppercaseString];
     NSCharacterSet	*set = [NSCharacterSet characterSetWithCharactersInString:@"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-_$"], *invSet;
-    NSRange		range;
+    NSRange         range;
 
     invSet = [set invertedSet];
 
     /* A-Z 0-9 - _ $ max 31 */
-    nam = [NSString stringWithCString:[nam lossyCString]];	// remove all umlauts
+    nam = [NSString stringWithUTF8String:[nam lossyCString]];	// remove all umlauts
     range = [nam rangeOfCharacterFromSet:invSet];
     while (range.length)
     {   NSMutableString	*mutString = [NSMutableString string];
@@ -584,8 +584,8 @@ int convertNSColorToDXFColor(NSColor *color)
 }
 
 - (BOOL)saveToFile:(NSString*)filename
-{   BOOL	savedOk = NO;
-    NSString	*dxfStr = @"";
+{   BOOL        savedOk = NO;
+    NSString    *dxfStr = @"";
 
     /* FormatStatement mm */
     dxfStr = [dxfStr stringByAppendingFormat:@"0\n"];
@@ -625,7 +625,9 @@ int convertNSColorToDXFColor(NSColor *color)
     dxfStr = [dxfStr stringByAppendingFormat:@"0\n"]; // end of programm
     dxfStr = [dxfStr stringByAppendingFormat:@"EOF\n"]; // end of programm
 
-    savedOk = [dxfStr writeToFile:filename atomically:YES];
+    //savedOk = [dxfStr writeToFile:filename atomically:YES];
+    savedOk = [dxfStr writeToFile:filename atomically:YES
+                         encoding:NSUTF8StringEncoding error:NULL]; // >= 10.5
 
     return savedOk;
 }
