@@ -1,11 +1,12 @@
 /* DINImport.m
  * DIN import object (Drill data import)
  *
- * Copyright (C) 1996-2006 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Ilonka Fleischmann
  *
  * created:  1996-05-03
- * modified: 2006-11-04
+ * modified: 2012-06-18 (leading zero support added (LZ), clean-up)
+ *           2006-11-04
  *
  * This file is part of the vhf Import Library.
  *
@@ -79,9 +80,10 @@
     fileFormat = 0;
 
     res = 1000.0;
+    tz = YES;       // trailing zeros
 
-    state.inch = 1; // inch !
-    state.pa = 1;
+    state.inch = 1; // inch
+    state.pa   = 1;
 
     state.tool = 0;
     state.point.x = state.point.y = LARGENEG_COORD;
@@ -101,44 +103,45 @@
             ops.start = @"%%3000";
             res = 25400.0;
             ops.prgend = @"$";
-            ops.coordC = @"D"; // diameter of tool
+            ops.coordC = @"D";  // diameter of tool
             break;
         case DIN_SM1000:
             ops.start = @"%%1000";
             res = 25400.0;
             ops.prgend = @"$";
-            ops.coordC = @""; // diameter of tool
+            ops.coordC = @"";   // diameter of tool
             break;
         default:
             ops.start = @"%";
             res = 10000.0;
             ops.prgend = @"M30";
-            ops.coordC = @"C"; // diameter of tool
+            ops.coordC = @"C";  // diameter of tool
     }
     resMM = 25400.0;
-    ops.offset = @"M50";
-    ops.mm = @"M71";
-    ops.reset = @"";
-    ops.selectTool = @"T";
-    ops.coordX = @"X";
-    ops.coordY = @"Y";
-    ops.coordR = @"R"; // radius of circle
-    ops.plotAbs = @"G90";
-    ops.plotRel = @"G91";
-    ops.termi = @"\n";
-    //ops.polyBegin = @"G41"; // left
+    tz    = YES;                // trailing zeros
+    ops.offset      = @"M50";
+    ops.mm          = @"M71";
+    ops.reset       = @"";
+    ops.selectTool  = @"T";
+    ops.coordX      = @"X";
+    ops.coordY      = @"Y";
+    ops.coordR      = @"R";     // radius of circle
+    ops.plotAbs     = @"G90";
+    ops.plotRel     = @"G91";
+    ops.termi       = @"\n";
+    //ops.polyBegin = @"G41";   // left
     //ops.polyBegin2 = @"G42";  // right
     //ops.polyEnd = @"G40";
-    ops.comment = @"/";
-    ops.line = @"G1"; // G01
-    ops.circleCW = @"G2"; // G02
-    ops.circleCCW = @"G3"; // G03
-    ops.drill = @"G05";
+    ops.comment     = @"/";
+    ops.line        = @"G1";    // G01
+    ops.circleCW    = @"G2";    // G02
+    ops.circleCCW   = @"G3";    // G03
+    ops.drill       = @"G05";
 }
 
 - (BOOL)checkFileFormat:(NSString*)dinStr
-{   NSScanner		*scanner = [NSScanner scannerWithString:dinStr];
-    int			location;
+{   NSScanner   *scanner = [NSScanner scannerWithString:dinStr];
+    int         location;
 
     location = [scanner scanLocation];
     [scanner scanUpToString:@"%%1000" intoString:NULL];
@@ -270,8 +273,8 @@
 }
 
 - (void)loadSM1000Apertures:(NSScanner*)scanner
-{   NSString	*parameter = @"$";
-    int		i, value, cnt = 1;
+{   NSString    *parameter = @"$";
+    int         i, value, cnt = 1;
 
     // $80$50$20$99$15$500 6 times $ at end
 
@@ -299,9 +302,9 @@
 }
 
 - (void)loadSM3000Apertures:(NSScanner*)scanner
-{   NSString	*parameter = @"$", *parmDia = @"D", *codeStr;
-    int		i, value, location = [scanner scanLocation];
-    float	dia;
+{   NSString    *parameter = @"$", *parmDia = @"D", *codeStr;
+    int         i, value, location = [scanner scanLocation];
+    float       dia;
 
     // $
     // T1D80S50F20R99N15A500T2D50...
@@ -355,29 +358,35 @@
 }
 
 - (void)loadExcellonApertures:(NSScanner*)scanner
-{   NSString	*parmDia = @"C", *codeStr;
-    int		value, location = [scanner scanLocation], inchLoc=0, mmLoc=0;
-    float	dia;
-    NSCharacterSet	*stopSet = [NSCharacterSet characterSetWithCharactersInString:@"TC"];
+{   NSString        *parmDia = @"C", *codeStr;
+    int             value, location = [scanner scanLocation], inchLoc=0, mmLoc=0;
+    float           dia;
+    NSCharacterSet  *stopSet = [NSCharacterSet characterSetWithCharactersInString:@"TC"];
 
     // get state.inch, state.pa and correct perhaps res
 
-    [scanner scanUpToString:@"G91" intoString:NULL]; // relative
+    [scanner scanUpToString:@"LZ" intoString:NULL];     // leading zeros
+    if ( [scanner scanString:@"LZ" intoString:NULL] )
+        tz = NO;    // leading zeros
+    // TODO: there is also "TZ" = trailing zeros, and "FMAT,2"
+
+    [scanner setScanLocation:location];
+    [scanner scanUpToString:@"G91" intoString:NULL];    // relative
     if ( [scanner scanString:@"G91" intoString:NULL] )
         state.pa = 0;
 
     [scanner setScanLocation:location];
-    [scanner scanUpToString:@"INCH" intoString:NULL]; // inch
+    [scanner scanUpToString:@"INCH" intoString:NULL];   // inch
     if ( [scanner scanString:@"INCH" intoString:NULL] )
         inchLoc = [scanner scanLocation];
     [scanner setScanLocation:location];
-    [scanner scanUpToString:@"M72" intoString:NULL]; // inch
+    [scanner scanUpToString:@"M72" intoString:NULL];    // inch
     if ( [scanner scanString:@"M72" intoString:NULL] )
     {   if ( !inchLoc || inchLoc > (int)[scanner scanLocation] )
             inchLoc = [scanner scanLocation];
     }
     [scanner setScanLocation:location];
-    [scanner scanUpToString:@"M71" intoString:NULL]; // mm
+    [scanner scanUpToString:@"M71" intoString:NULL];    // mm
     if ( [scanner scanString:@"M71" intoString:NULL] )
         mmLoc = [scanner scanLocation];
     [scanner setScanLocation:location];
@@ -387,7 +396,7 @@
             mmLoc = [scanner scanLocation];
     }
     [scanner setScanLocation:location];
-    [scanner scanUpToString:ops.mm intoString:NULL]; // mm (from device file)
+    [scanner scanUpToString:ops.mm intoString:NULL];    // mm (from device file)
     if ( [scanner scanString:ops.mm intoString:NULL] )
     {   if ( !mmLoc || mmLoc > (int)[scanner scanLocation] )
             mmLoc = [scanner scanLocation];
@@ -478,7 +487,7 @@
     [self loadApertures:dinStr];
 
     if ( ![tools count] )
-        NSLog(@"No apertures loaded !");
+        NSLog(@"No tools loaded !");
 
     /* interpret data
      */
@@ -491,9 +500,9 @@
 /* private methods
  */
 - (BOOL)interpret:(NSString*)dataP
-{   int		startLocation;
-    NSRect	bounds;
-    NSScanner	*scanner = [NSScanner scannerWithString:dataP];
+{   int         startLocation;
+    NSRect      bounds;
+    NSScanner   *scanner = [NSScanner scannerWithString:dataP];
 
     digitsSet = [NSCharacterSet characterSetWithCharactersInString:DIGITS];
     invDigitsSet = [digitsSet invertedSet];
@@ -526,8 +535,8 @@
         if ( ![self getGraphicFromData:scanner :list] )
             break;
 
-    bounds.origin = ll;
-    bounds.size.width = ur.x - ll.x;
+    bounds.origin      = ll;
+    bounds.size.width  = ur.x - ll.x;
     bounds.size.height = ur.y - ll.y;
 
     [self setBounds:bounds];
@@ -537,8 +546,9 @@
 /* we need cp on a number !
  */
 - (BOOL)getGraphicFromData:(NSScanner*)scanner :cList
-{   int		location;
-    float	value;
+{   int         location;
+    float       value;
+    NSString    *str = nil;
 
     location = [scanner scanLocation];
 
@@ -596,8 +606,12 @@
     }*/
     else if ( [scanner scanString:ops.coordX intoString:NULL] )
     {
-        if ( [scanner scanFloat:&value] )
-        {  // state.point.x = state.x;
+        if ( [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        //if ( [scanner scanFloat:&value] )
+        {   //state.point.x = state.x;
+            value = [str floatValue];
+            if ( ! tz ) // leading zeros
+                value *= pow(10, 7 - [str length]);
             state.x = DeviceResToInternal(value, res);
             state.draw = 1;
         }
@@ -608,8 +622,12 @@
     }
     else if ( [scanner scanString:ops.coordY intoString:NULL] )
     {
-        if ( [scanner scanFloat:&value] )
-        {//   state.point.y = state.y;
+        if ( [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        //if ( [scanner scanFloat:&value] )
+        {   //state.point.y = state.y;
+            value = [str floatValue];
+            if ( ! tz ) // leading zeros
+                value *= pow(10, 7 - [str length]);
             state.y = DeviceResToInternal(value, res);
             state.draw = 1;
         }
@@ -620,8 +638,12 @@
     }
     else if ( [scanner scanString:ops.coordR intoString:NULL] )
     {
-        if ( [scanner scanFloat:&value] )
-        {   state.a = DeviceResToInternal(value, res);
+        if ( [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        //if ( [scanner scanFloat:&value] )
+        {   value = [str floatValue];
+            if ( ! tz ) // leading zeros
+                value *= pow(10, 7 - [str length]);
+            state.a = DeviceResToInternal(value, res);
             state.draw = 1;
         }
         else
@@ -886,8 +908,8 @@
 }
 
 - (void)setMark:cList
-{   NSPoint		center, pll, pur;
-    NSDictionary	*tool; // = [tools objectAtIndex:state.tool];
+{   NSPoint         center, pll, pur;
+    NSDictionary    *tool; // = [tools objectAtIndex:state.tool];
 
     tool = ([tools count]) ? [tools objectAtIndex:state.tool] : nil;
 
