@@ -1,12 +1,20 @@
 /* DXFImport.m
  * DXF import object
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2013 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-01
- * modified: 2010-10-31 (buildArc(): dx, dy calced with double)
- *           2010-09-11 (-interpret: try UTF8 and Latin encodings to make GNUstep happy)
+ * modified: 2013-09-13 (getInsert(), ...: place objects to layer of insert, not the layer of the blocks)
+ *           2013-04-16 (getTableFromData(): return dictionary)
+ *           2012-09-13 (Diff(l, 0.0) < 0.0000001, else division with 0)
+ *           2012-02-04 (-scanGroup: transform coordinates from OCS to WCS, if necessary)
+ *           2012-02-03 (InternalToDeviceRes(), DeviceResToInternal(): float -> double)
+ *           2011-12-11 (-getGraphicFromData: TODO for SPLINEs added)
+ *           2011-04-04 (-getGraphicFromData: 3DFACE TODO-log added)
+ *           2011-02-27 (-init: initialize group.handle, state.color, blockScanner, rable, vivibleList)
+ *           2010-10-31 (buildArc(): dx, dy calced with double)
+ *           2010-09-11 (-importDXF: try UTF8 and Latin encodings to make GNUstep happy)
  *           2010-06-11 (buildArc(): 360.0 - acos(cn), was 180.0 - )
  *           2010-04-03 (-getArc: negative 360 deg arcs work now, buildArc(): dx == 0 case)
  *           2007-01-30 (close PolyLine: check for start == end point)
@@ -50,8 +58,8 @@
 //#define	SqrDistPoints(p1, p2)	(((p1).x-(p2).x)*((p1).x-(p2).x)+((p1).y-(p2).y)*((p1).y-(p2).y))
 
 /* r in points/inch */
-#define	InternalToDeviceRes(a, r)	((float)(a) * (float)(r) / 72.0)
-#define	DeviceResToInternal(a, r)	((float)(a) * 72.0 / (float)(r))
+#define	InternalToDeviceRes(a, r)	((double)(a) * (double)(r) / 72.0)
+#define	DeviceResToInternal(a, r)	((double)(a) * 72.0 / (double)(r))
 
 static NSCharacterSet	*newLineSet, *skipSet;
 
@@ -60,8 +68,8 @@ static BOOL gotoNextId(NSScanner *scanner, NSString *dataId);
 static BOOL gotoSection(NSScanner *scanner, NSString *name);
 static BOOL gotoTable(NSScanner *scanner, NSString *name);
 static BOOL gotoGroup( NSScanner *scanner, NSString *name);
-static NSArray *getTableFromData(NSString *data);
-static NSColor *getColorFromTable(NSString *str, NSArray *table);
+static NSDictionary *getTableFromData(NSString *data);
+static NSColor *getColorFromLayer(NSString *str, NSArray *table);
 static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *angle);
 
 @interface DXFImport(PrivateMethods)
@@ -71,6 +79,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 - (BOOL)getGraphicFromData:(NSScanner*)scanner :cList;
 - (BOOL)scanGroup:(NSScanner*)scanner;
 - (BOOL)getLine:(NSScanner*)scanner :(NSPoint*)p0 :(NSPoint*)p1 :(NSColor**)color;
+- (BOOL)getLine3D:(NSScanner*)scanner :(V3Point*)p0 :(V3Point*)p1 :(NSColor**)color;
 - (BOOL)getLWPolyline:(NSScanner*)scanner addToList:(NSMutableArray*)list;
 - (BOOL)getPolyline:(NSScanner*)scanner;
 - (BOOL)getVertex:(NSScanner*)scanner :(int*)kind :(NSPoint*)p0 :(NSPoint*)p1 :(float*)angle;
@@ -78,6 +87,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 - (BOOL)getCircle:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle;
 - (BOOL)getArc:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle;
 - (BOOL)getText :(NSScanner*)scanner mtext:(BOOL)mext :(NSString**)string :(float*)angle :(NSPoint*)origin :(float*)size :(float*)ar :(int*)alignment;
+- (BOOL)get3DFace:(NSScanner*)scanner points:(V3Point*)pts color:(NSColor**)color;
 - (BOOL)getInsert:(NSScanner*)scanner :cList;
 - (void)updateBounds:(NSPoint)p;
 
@@ -100,8 +110,8 @@ static BOOL gotoNextLine(NSScanner *scanner)
  * scanner have to be on an id!
  */
 static BOOL gotoNextId(NSScanner *scanner, NSString* dataId)
-{   NSString	*string;
-    int		location = [scanner scanLocation];
+{   NSString    *string;
+    int         location = [scanner scanLocation];
 
     while ( ![scanner isAtEnd] )
     {
@@ -168,11 +178,11 @@ static BOOL gotoSection(NSScanner *scanner, NSString *name)
             if ( [scanner scanString:IDNAME intoString:NULL] )		/* name code */
             {
                 if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                    NSLog(@"GotoSection: Newline expected at location: %d", [scanner scanLocation]);
+                    NSLog(@"GotoSection: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
                 if ( [scanner scanString:name intoString:NULL] )		/* check name */
                 {
                     if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                        NSLog(@"GotoSection: Newline expected at location: %d", [scanner scanLocation]);
+                        NSLog(@"GotoSection: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
                     return YES;
                 }
             }
@@ -181,7 +191,7 @@ static BOOL gotoSection(NSScanner *scanner, NSString *name)
     return NO;
 }
 
-/* created:   12.05.93
+/* created:   1993-05-12
  * modified:  08.12.97
  *
  * purpose:   set cp to the beginning (on the name) of a named table
@@ -199,11 +209,11 @@ static BOOL gotoTable(NSScanner *scanner, NSString *name)
             if ( [scanner scanCharactersFromSet:newLineSet intoString:NULL] && [scanner scanString:@"2" intoString:NULL] )	/* newline, name code */
             {
                 if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                    NSLog(@"GotoTable: Newline expected at location: %d", [scanner scanLocation]);
+                    NSLog(@"GotoTable: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
                 if ( [scanner scanString:name intoString:NULL] )		/* check name */
                 {
                     if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                        NSLog(@"GotoTable: Newline expected at location: %d", [scanner scanLocation]);
+                        NSLog(@"GotoTable: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
                     return YES;
                 }
             }
@@ -216,14 +226,16 @@ static BOOL gotoTable(NSScanner *scanner, NSString *name)
  * modified:  2001-02-23
  *
  * purpose:   get tables from data
- * 	      at the moment we only read the layer table...
+ *            at the moment we only read the layer table...
  * parameter: data (dxf file)
  */
-static NSArray *getTableFromData(NSString *data)
-{   NSScanner		*scanner = [NSScanner scannerWithString:data];
-    NSString		*string;
-    NSMutableArray	*table = [NSMutableArray array];
-    NSMutableDictionary	*dict = nil;
+static NSDictionary *getTableFromData(NSString *data)
+{   NSScanner           *scanner = [NSScanner scannerWithString:data];
+    NSString            *string;
+    NSMutableDictionary *table  = [NSMutableDictionary dictionary];
+    NSMutableArray      *layers = [NSMutableArray array];           // layers array
+    NSMutableDictionary *styles = [NSMutableDictionary dictionary]; // styles dict
+    NSMutableDictionary *dict = nil;
 
     [scanner setCharactersToBeSkipped:skipSet];
 
@@ -234,11 +246,12 @@ static NSArray *getTableFromData(NSString *data)
 
     /* read the layer table
      */
+    [table setObject:layers forKey:@"layer"];
     if (!gotoTable(scanner, NAMLAYER))
     {   NSLog(@"Can't get table 'LAYER' from dxf data");
         return nil;
     }
-    /* 2
+    /* 0
      * LAYER
      * we are here
      */
@@ -247,7 +260,7 @@ static NSArray *getTableFromData(NSString *data)
 
         /* scan id */
         if ( ![scanner scanInt:&ident] )
-        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
             return NO;
         }
         gotoNextLine(scanner);	// goto value line
@@ -264,7 +277,7 @@ static NSArray *getTableFromData(NSString *data)
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
                     break;
                 [dict setObject:string forKey:@"name"];
-                [table addObject:dict];
+                [layers addObject:dict];
                 break;
             case ID_COLOR:
                 [scanner scanUpToCharactersFromSet:newLineSet intoString:&string];
@@ -281,6 +294,64 @@ static NSArray *getTableFromData(NSString *data)
             break;
     }
 
+    /* read the style table
+     * tables -> style -> style1, style2, ...
+     */
+    [table setObject:styles forKey:@"style"];
+    [scanner setScanLocation:0];
+    if ( !gotoSection(scanner, NAMTABLES) )
+    {   NSLog(@"Can't get section 'TABLE' from dxf data");
+        return nil;
+    }
+    if (!gotoTable(scanner, NAMSTYLE))
+    {   NSLog(@"Can't get table 'STYLE' from dxf data");
+        return nil;
+    }
+    /* 0
+     * STYLE
+     * we are here
+     */
+    while ( ![scanner isAtEnd] )
+    {	int	ident;
+
+        /* scan id */
+        if ( ![scanner scanInt:&ident] )
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
+            return NO;
+        }
+        gotoNextLine(scanner);	// goto value line
+        switch ( ident )
+        {
+            case ID_GROUP:
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
+                    string = @"";
+                if (![string isEqual:GRPSTYLE]) // ENDTAB
+                    break;
+                dict = [NSMutableDictionary dictionary];
+                break;
+            case ID_NAME:
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
+                    break;
+                [dict setObject:string forKey:@"name"];
+                [styles setObject:dict forKey:string];
+                //[styles addObject:dict];
+                break;
+            case ID_DESCRIPT:   // font file name ("ariblk.TTF")
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
+                    break;
+                [dict setObject:string forKey:@"fileName"];
+                break;
+            case ID_EXTSTRING:  // font name ("Arial Black")
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
+                    break;
+                [dict setObject:string forKey:@"fontName"];
+                break;
+        }
+        gotoNextLine(scanner);	// goto id line
+        if (ident == ID_GROUP && ![string isEqual:@"STYLE"])
+            break;
+    }
+
     return table;
 }
 
@@ -310,7 +381,7 @@ static NSArray *getVisibleListFromData(NSString *data)
 
         /* scan id */
         if ( ![scanner scanInt:&ident] )
-        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
             return nil;
         }
         gotoNextLine(scanner);	// goto value line
@@ -362,7 +433,7 @@ static BOOL gotoGroup( NSScanner *scanner, NSString *name )
         if ( [scanner scanString:name intoString:NULL] )
         {
             if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                NSLog(@"GotoGroup: Newline expected at location: %d", [scanner scanLocation]);
+                NSLog(@"GotoGroup: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
             [scanner setCaseSensitive:caseSensitive];
             return YES;
         }
@@ -380,21 +451,21 @@ static BOOL gotoGroup( NSScanner *scanner, NSString *name )
  *            table ()
  * return:    color
  */
-static NSColor *getColorFromTable(NSString *str, NSArray *table)
+static NSColor *getColorFromLayer(NSString *str, NSArray *layers)
 {   int			i;
-    DXFColor		col;
+    DXFColor    col;
     NSColor		*color;
     //NSDictionary	*dict;
 
-    /*if (dict = [table objectForKey:str])
+    /*if (dict = [layers objectForKey:str])
     {   int	colIx = [dict intForKey:@"colorNum"];
 
         col = colorTable[(colIx > 0) ? colIx-1 : 0];
         color = [NSColor colorWithCalibratedRed:col.r green:col.g blue:col.b alpha:1.0];
         return color;
     }*/
-    for (i=[table count]-1; i>=0; i--)
-    {   NSDictionary	*dict = [table objectAtIndex:i];
+    for (i=[layers count]-1; i>=0; i--)
+    {   NSDictionary	*dict = [layers objectAtIndex:i];
 
         if ( [[dict objectForKey:@"name"] isEqual:str] )
         {   int	colIx = [[dict objectForKey:@"colorNum"] intValue];
@@ -416,6 +487,13 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     group.text = nil;
     group.name = nil;
     group.layer = nil;
+    group.handle = nil;
+
+    state.color = nil;
+
+    blockScanner = nil;
+    table = nil;
+    visibleList = nil;
 
     return self;
 }
@@ -483,7 +561,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     {   int	i;
 
         if (![scanner scanInt:&i])	// get id
-            NSLog(@"getHeader: integer expected at location: %d", [scanner scanLocation]);
+            NSLog(@"getHeader: integer expected at location: %u", (unsigned)[scanner scanLocation]);
 
         gotoNextLine(scanner);		// goto value
         switch (i)
@@ -521,9 +599,9 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 - (BOOL)interpret:(NSString*)data
-{   NSMutableArray	*cList;		// current list
-    NSRect		bounds;
-    NSScanner		*scanner = [NSScanner scannerWithString:data];
+{   NSMutableArray  *cList;		// current list
+    NSRect          bounds;
+    NSScanner       *scanner = [NSScanner scannerWithString:data];
 
     /* init bounds */
     ll.x = ll.y = LARGE_COORD;
@@ -535,7 +613,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
         return NO;
     }
 
-    list = [self allocateList:table];
+    list = [self allocateList:[table objectForKey:@"layer"]];
     cList = [NSMutableArray array];
 
     while ( ![scanner isAtEnd] )
@@ -560,7 +638,8 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 /* we need scanner on the id line of a group
- * modified: 2005-09-26 (close polyline with arc)
+ * modified: 2013-09-13 (toLayer:((state.layer) ? state.layer : group.layer))
+ *           2005-09-26 (close polyline with arc)
  */
 - (BOOL)getGraphicFromData:(NSScanner*)scanner :(id)cList
 {   NSPoint     p0, p1;
@@ -573,7 +652,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     [scanner scanUpToCharactersFromSet:newLineSet intoString:&name];
     /* goto id line */
     if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-        NSLog(@"getGraphicFromData:: Newline expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getGraphicFromData:: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
 
     switch ( state.mode )
     {
@@ -582,7 +661,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
             if ( [name isEqual:GRPLINE] )
             {
                 if ([self getLine:scanner :&p0 :&p1 :&state.color])
-                    [self addLine:p0 :p1 toLayer:group.layer];
+                    [self addLine:p0 :p1 toLayer:((state.layer) ? state.layer : group.layer)];
             }
             /* lwpolyline */
             else if ( [name isEqual:GRPLWPOLYLINE] )
@@ -606,7 +685,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                         [self updateBounds:ps[i]];
                     }
                     if ([cList count])
-                    {   [self addFillList:cList toLayer:group.layer];
+                    {   [self addFillList:cList toLayer:((state.layer) ? state.layer : group.layer)];
                         [cList removeAllObjects];
                     }
                 }
@@ -617,7 +696,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                 float	angle;
 
                 if ([self getCircle:scanner :&ctr :&start :&angle])
-                    [self addArc:ctr :start :angle toLayer:group.layer];
+                    [self addArc:ctr :start :angle toLayer:((state.layer) ? state.layer : group.layer)];
             }
             /* arc */
             else if ( [name isEqual:GRPARC] )
@@ -625,24 +704,24 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                 float	angle;
 
                 if ([self getArc:scanner :&ctr :&start :&angle])
-                    [self addArc:ctr :start :angle toLayer:group.layer];
+                    [self addArc:ctr :start :angle toLayer:((state.layer) ? state.layer : group.layer)];
             }
             /* text, mtext */
             else if ( [name isEqual:GRPTEXT] || [name isEqual:GRPMTEXT] )
-            {	NSPoint		origin;
-                float		angle, size, ar;
-                int		alignment;
-                NSString	*string = nil;
+            {	NSPoint     origin;
+                float       angle, size, ar;
+                int         alignment;
+                NSString    *string = nil, *fontName = @"Courier";
 
                 if ([self getText:scanner mtext:[name isEqual:GRPMTEXT]
-                                 :&string :&angle :&origin :&size :&ar :&alignment])
+                                 :&string :&angle :&origin :&size :&ar :&alignment :&fontName])
                 {
                     if ([self respondsToSelector:@selector(addText:::::at:toLayer:)])	// old
-                        [self addText:string :@"Courier" :angle :size :ar at:origin
-                              toLayer:group.layer];
+                        [self addText:string :fontName :angle :size :ar at:origin
+                              toLayer:((state.layer) ? state.layer : group.layer)];
                     else
-                        [self addText:string :@"Courier" :angle :size :ar :alignment at:origin
-                              toLayer:group.layer];
+                        [self addText:string :fontName :angle :size :ar :alignment at:origin
+                              toLayer:((state.layer) ? state.layer : group.layer)];
                 }
                 [string autorelease];
             }
@@ -651,12 +730,34 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
             {
                 [self getInsert:scanner :cList];
             }
+            /* SPLINE */
+            else if ( [name isEqual:GRPSPLINE] )
+            {
+                // TODO: SPLINEs (see example Stebner_SPLINEs*.dxf)
+                //if ([self getSpline:scanner :pts :&state.color])  // we should return/convert to bezier curve(s)
+                //    [self addCurve:pts toLayer:((state.layer) ? state.layer : group.layer)];
+                printf("TODO, DXF-Import: SPLINEs not implemented\n");
+            }
+            /* 3D LINE */
+            else if ( [name isEqual:GRP3DLINE] )
+            {   V3Point p0, p1;
+
+                if ([self getLine3D:scanner :&p0 :&p1 :&state.color])
+                    [self addLine3D:p0 :p1 toLayer:((state.layer) ? state.layer : group.layer)];
+            }
+            /* 3D FACE */
+            else if ( [name isEqual:GRP3DFACE] )
+            {   V3Point pts[4];
+
+                if ([self get3DFace:scanner points:pts color:&state.color] )
+                    [self add3DFace:pts toLayer:((state.layer) ? state.layer : group.layer)];
+            }
             break;
         case MODE_VERTEX:
             /* vertex */
             if ( [name isEqual:GRPVERTEX] )
-            {	int	kind;
-                float	a;
+            {	int     kind;
+                float   a;
 
                 [self getVertex:scanner :&kind :&p0 :&p1 :&a];
                 //printf("kind=%d p0={%.0f %.0f} p1={%.0f %.0f} a=%.0f\n", kind, p0.x, p0.y, p1.x, p1.y, a);
@@ -684,7 +785,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                     }
                 }
                 if ([cList count])
-                {   [self addStrokeList:cList toLayer:group.layer];
+                {   [self addStrokeList:cList toLayer:((state.layer) ? state.layer : group.layer)];
                     [cList removeAllObjects];
                 }
                 state.mode = MODE_NORMAL;
@@ -747,6 +848,37 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     return YES;
 }
 
+/* created:   2011-04-04
+ * modified:  
+ *
+ * purpose:   get 3D line
+ * parameter: scanner (the dxf data)
+ *            g (the destination for the element)
+ *            state
+ *            parms
+ */
+- (BOOL)getLine3D:(NSScanner*)scanner :(V3Point*)p0 :(V3Point*)p1 :(NSColor**)color
+{
+    [self scanGroup:scanner];
+    if ( [visibleList count] && ![visibleList containsObject:group.handle])
+        return NO;
+
+    *color = [self currentColor];
+    p0->x = DeviceResToInternal(group.x0, res) + state.offset.x;
+    p0->y = DeviceResToInternal(group.y0, res) + state.offset.y;
+    p0->z = DeviceResToInternal(group.z0, res) /*+ state.offset.z*/;
+    p1->x = DeviceResToInternal(group.x1, res) + state.offset.x;
+    p1->y = DeviceResToInternal(group.y1, res) + state.offset.y;
+    p1->z = DeviceResToInternal(group.z1, res) /*+ state.offset.z*/;
+
+    state.width = 0.0;
+
+    [self updateBounds:NSMakePoint(p0->x, p0->y)];
+    [self updateBounds:NSMakePoint(p1->x, p1->y)];
+
+    return YES;
+}
+
 /* created:   2001-02-22
  * modified:  2006-08-10 (check for close with & operator)
  *
@@ -777,7 +909,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 
         /* scan id */
         if ( ![scanner scanInt:&i] )
-        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
             return NO;
         }
         gotoNextLine(scanner);	// goto value line
@@ -863,7 +995,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                 if ( ![visibleList count] || [visibleList containsObject:group.handle])
                 {
                     if ([cList count])
-                    {   [self addStrokeList:cList toLayer:group.layer];
+                    {   [self addStrokeList:cList toLayer:((state.layer) ? state.layer : group.layer)];
                         [cList removeAllObjects];
                     }
                 }
@@ -1019,7 +1151,8 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 /* Note: see page 323 of DXF book
- * modified: 2010-10-31 (dx, dy calced with double)
+ * modified: 2012-09-13 (Diff(l, 0.0) < 0.0000001, else division with 0)
+ *           2010-10-31 (dx, dy calced with double)
  *           2010-06-11 (360 - acos(cn))
  *           2010-04-03
  */
@@ -1030,6 +1163,12 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 
     ang = 4.0 * RadToDeg(atan( a ));
     l = sqrt(SqrDistPoints(beg, end));  // length of arc from point to point
+    if ( Diff(l, 0.0) < 0.0000001 )
+    {
+        *ctr = beg;     // null length arc else division with 0
+        *angle = 0.0;
+        return;
+    }
     h = (a*l)/2.0;                      // height of arc
     mx = (begx+endx)/2.0; my = (begy+endy)/2.0; // mit point between start and end
     dx = mx - begx;                     // 
@@ -1114,9 +1253,9 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
  * modified:  2004-08-04
  *
  * purpose:   get circle
- * parameter: scanner	the dxf data
- *            ctr	center
- *            start	
+ * parameter: scanner   the dxf data
+ *            ctr       center
+ *            start
  *            angle	360 degree
  */
 - (BOOL)getCircle:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle
@@ -1193,7 +1332,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 }
 
 /* created:   1993-06-11
- * modified:  2002-01-09
+ * modified:  2013-04-16 (fontName added)
  *
  * purpose:   get text (TEXT, MTEXT)
  *            read a sv-polygon for each character
@@ -1205,16 +1344,18 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
  *            size
  *            ar	aspect ratio
  *            alignment	1 = top/left, 2= top/center, 3=top/right, 4=middle/left, ... 9 = bottom/right
+ *            fontName
  */
 - (BOOL)getText:(NSScanner*)scanner mtext:(BOOL)mtext
                :(NSString**)string :(float*)angle :(NSPoint*)origin :(float*)size :(float*)ar :(int*)alignment
+               :(NSString**)fontName
 {   int		adjust = 0;
     NSPoint	adjustPoint,	// a point to adjust the text
-		p;
+            p;
     float	slopeAngle = 0.0;
     float	textHeight, textScale;
 
-    *ar = 1.0;	/* aspect ratio */
+    *ar = 1.0;	// aspect ratio
 
     [self scanGroup:scanner];
     if ( [visibleList count] && ![visibleList containsObject:group.handle])
@@ -1234,6 +1375,21 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     adjust = group.adjust;	// get adjust position from file
     adjustPoint.x = DeviceResToInternal(group.x1, res);
     adjustPoint.y = DeviceResToInternal(group.y1, res);
+    if ( ! group.style )
+        group.style = @"Standard";  // FIXME: this should be case insensitive
+    if ( [table objectForKey:@"style"] )
+    {   NSDictionary    *styles = [table objectForKey:@"style"];
+        NSDictionary    *style = [styles objectForKey:group.style];
+
+        if ( [style objectForKey:@"fontName"] )
+            *fontName = [style objectForKey:@"fontName"];
+        else if ( [style objectForKey:@"fileName"] )
+        {   NSString    *fn = [style objectForKey:@"fileName"];
+
+            NSLog(@"TODO: what are we going to assign to the fonbt file name: \"%@\" ? Using Default.", fn);
+            *fontName = nil;
+        }
+    }
 
     if (!mtext)
     {
@@ -1276,26 +1432,76 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     return YES;
 }
 
-/* created:   30.09.94
- * modified:  30.09.94 10.03.97
+/* created:   2011-04-04
+ * modified:  2011-04-04
+ *
+ * purpose:   get insert
+ * parameter: scanner (the dxf data)
+ *            layer
+ * 3DFACE    Four points defining the corners of the face: (10, 20, 30),
+ * (11, 21, 31), (12, 22, 32), and (13, 23, 33).  70 (invisible
+ * edge flags -optional 0).  If only three points were entered
+ * (forming a triangular face), the third and fourth points will
+ * be the same.  The meanings of the bit-coded "invisible edge
+ * flags" are shown in the following table.
+ *
+ * Flag bit value           Meaning
+ * 1        First edge is invisible
+ * 2        Second edge is invisible
+ * 4        Third edge is invisible
+ * 8        Fourth edge is invisible
+ */
+- (BOOL)get3DFace:(NSScanner*)scanner points:(V3Point*)pts color:(NSColor**)color
+{
+    [self scanGroup:scanner];
+    if ( [visibleList count] && ![visibleList containsObject:group.handle])
+        return NO;
+
+    *color = [self currentColor];
+    pts[0].x = DeviceResToInternal(group.x0, res) + state.offset.x;
+    pts[0].y = DeviceResToInternal(group.y0, res) + state.offset.y;
+    pts[0].z = DeviceResToInternal(group.z0, res) /*+ state.offset.z*/;
+    pts[1].x = DeviceResToInternal(group.x1, res) + state.offset.x;
+    pts[1].y = DeviceResToInternal(group.y1, res) + state.offset.y;
+    pts[1].z = DeviceResToInternal(group.z1, res) /*+ state.offset.z*/;
+    pts[2].x = DeviceResToInternal(group.x2, res) + state.offset.x;
+    pts[2].y = DeviceResToInternal(group.y2, res) + state.offset.y;
+    pts[2].z = DeviceResToInternal(group.z2, res) /*+ state.offset.z*/;
+    pts[3].x = DeviceResToInternal(group.x3, res) + state.offset.x;
+    pts[3].y = DeviceResToInternal(group.y3, res) + state.offset.y;
+    pts[3].z = DeviceResToInternal(group.z3, res) /*+ state.offset.z*/;
+
+    state.width = 0.0;
+
+    [self updateBounds:NSMakePoint(pts[0].x, pts[0].y)];
+    [self updateBounds:NSMakePoint(pts[1].x, pts[1].y)];
+    [self updateBounds:NSMakePoint(pts[2].x, pts[2].y)];
+    [self updateBounds:NSMakePoint(pts[3].x, pts[3].y)];
+
+    return YES;
+}
+
+/* created:   1994-09-30
+ * modified:  2013-09-13 (Fix: place objects to layer of insert, not the layer of the blocks)
  *
  * purpose:   get insert
  * parameter: scanner (the dxf data)
  *            cList
  */
-- (BOOL)getInsert:(NSScanner*)scanner :cList
-{   NSPoint		point;				/* position of the text */
-    NSString		*name, *string;
-    float		angle;
-    NSScanner		*myBlockScanner = nil;
-    int			location;
-    NSAutoreleasePool	*pool = [[NSAutoreleasePool alloc] init];
+- (BOOL)getInsert:(NSScanner*)scanner :(id)cList
+{   NSPoint             point;				/* position of the text */
+    NSString            *name, *string;
+    float               angle;
+    NSScanner           *myBlockScanner = nil;
+    int                 location;
+    NSAutoreleasePool   *pool = [[NSAutoreleasePool alloc] init];
 
     [self scanGroup:scanner];
     if ( [visibleList count] && ![visibleList containsObject:group.handle])
         return YES;
 
     state.color = [self currentColor];
+    state.layer = [group.layer retain]; // everthing goes to this layer
     name = group.name;
     angle = group.begAngle;
     point.x = DeviceResToInternal(group.x0, res);
@@ -1364,12 +1570,14 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
      */
     state.offset = point;
     state.rotAngle = angle;
+    // FIXME: we have to put the blocks on the layer group.layer (not the layer in the blocjk !)
     while ( ![myBlockScanner isAtEnd] )
         [self getGraphicFromData:myBlockScanner :cList];
     [scanner setScanLocation:location];
     state.offset = NSMakePoint(0.0, 0.0);
     state.rotAngle = 0.0;
 
+    [state.layer release]; state.layer = nil;
     [pool release];
 
     return YES;
@@ -1378,9 +1586,9 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 /*
  */
 - (NSColor*)currentColor
-{   int		num = group.color;
-    DXFColor	col;
-    NSColor	*color;
+{   int         num = group.color;
+    DXFColor    col;
+    NSColor     *color;
 
     if ( num )
     {
@@ -1392,40 +1600,91 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     }
 
     /* search the relating color from the color table */
-    color = getColorFromTable(group.layer, table);
+    color = getColorFromLayer(group.layer, [table objectForKey:@"layer"]);
     return color;
 }
 
+/* arbitrary axis algorithm - transform OCS (Object) to WCS (World)
+ * build the cross product
+ *
+ * W        our coordinate             [WCS]
+ * AZ (N)   the extrusion direction    [WCS]
+ * return   the transformed coordinate [OCS]
+ * created: 2012-02-03
+ */
+static V3Point crossProduct(const V3Point a, const V3Point b)
+{   V3Point result;
+
+    result.x = a.y * b.z - a.z * b.y;
+    result.y = a.z * b.x - a.x * b.z;
+    result.z = a.x * b.y - a.y * b.x;
+    return result;
+}
+static V3Point scaleToUnit(V3Point p)
+{   double  len = sqrt(p.x*p.x + p.y*p.y + p.z*p.z);
+
+    if (len != 0)
+    {   p.x /= len;
+        p.y /= len;
+        p.z /= len;
+    }
+    return p;
+}
+static V3Point transformOCS(V3Point pOCS, V3Point AZ)
+{   V3Point AX, AY, p;
+
+    if ( Abs(AZ.x) < 1.0/64.0 && Abs(AZ.y) < 1.0/64.0 )
+        AX = crossProduct(V3MakePoint(0.0, 1.0, 0.0), AZ);   // transform using unit vector Y
+    else
+        AX = crossProduct(V3MakePoint(0.0, 0.0, 1.0), AZ);   // transform using unit vector Z
+    AX = scaleToUnit( AX );  // scale vector to unit (length = 1.0)
+    AY = crossProduct(AZ, AX);
+    AY = scaleToUnit( AY );
+
+    /* these are direction cosines, we have to rotate each axis around this */
+    {   double x = pOCS.x, y = pOCS.y, z = pOCS.z;
+
+        p.x = x * AX.x + y * AY.x + z * AZ.x;
+        p.y = x * AX.y + y * AY.y + z * AZ.y;
+        p.z = x * AX.z + y * AY.z + z * AZ.z;
+    }
+    return p;
+}
+
 /* get group entries
  * scanner starts at id line of first entry
  * scanner scans until it reaches the next group
  * created:  1997-03-19
- * modified: 2009-02-06 (read in extrusion direction)
+ * modified: 2013-04-16 (style added)
+ *           2011-04-04 (group.z0 - z3 added)
+ *           2009-02-06 (read in extrusion direction)
  */
 - (BOOL)scanGroup:(NSScanner*)scanner
-{
-    [group.text release]; group.text = nil;
-    [group.name release]; group.name = nil;
-    [group.layer release]; group.layer = nil;
+{   BOOL    exitLoop = NO;
+
+    [group.text release];   group.text = nil;
+    [group.style release];  group.style = nil;
+    [group.name release];   group.name = nil;
+    [group.layer release];  group.layer = nil;
     [group.handle release]; group.handle = nil;
     group.color = 0;
     group.width = -1.0;
     group.endWidth = -1.0;
     group.begAngle = group.endAngle = 0.0;
-    group.x0 = group.y0 = 0.0;
-    group.x1 = group.y1 = 0.0;
-    group.x2 = group.y2 = 0.0;
-    group.x3 = group.y3 = 0.0;
+    group.x0 = group.y0 = group.z0 = 0.0;
+    group.x1 = group.y1 = group.z1 = 0.0;
+    group.x2 = group.y2 = group.z2 = 0.0;
+    group.x3 = group.y3 = group.z3 = 0.0;
     group.a = 0.0;
     group.flags = group.genFlags = 0;
-    group.extX = group.extY = 0.0; group.extZ = -1.0;
+    group.extX = group.extY = 0.0; group.extZ = 1.0;
 
     while ( ![scanner isAtEnd] )
-    {	int	i, location = [scanner scanLocation];
+    {	int i, location = [scanner scanLocation];
 
         /* scan id */
         if ( ![scanner scanInt:&i] )
-        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
             return NO;
         }
         gotoNextLine(scanner);	// goto value line
@@ -1456,6 +1715,12 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
                 if (![scanner scanInt:&group.lineType])
                     group.lineType = 0.0;
                 break;
+            case ID_STYLE:
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&group.style])
+                    group.style = nil;
+                else
+                    [group.style retain];
+                break;
             case ID_LAYER:
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&group.layer])
                     group.layer = nil;
@@ -1470,6 +1735,10 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
                 if (![scanner scanFloat:&group.y0])
                     group.y0 = 0.0;
                 break;
+            case ID_Z0:
+                if (![scanner scanFloat:&group.z0])
+                    group.z0 = 0.0;
+                break;
             case ID_X1:
                 if (![scanner scanFloat:&group.x1])
                     group.x1 = 0.0;
@@ -1478,6 +1747,10 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
                 if (![scanner scanFloat:&group.y1])
                     group.y1 = 0.0;
                 break;
+            case ID_Z1:
+                if (![scanner scanFloat:&group.z1])
+                    group.z1 = 0.0;
+                break;
             case ID_X2:
                 if (![scanner scanFloat:&group.x2])
                     group.x2 = 0.0;
@@ -1486,6 +1759,10 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
                 if (![scanner scanFloat:&group.y2])
                     group.y2 = 0.0;
                 break;
+            case ID_Z2:
+                if (![scanner scanFloat:&group.z2])
+                    group.z2 = 0.0;
+                break;
             case ID_X3:
                 if (![scanner scanFloat:&group.x3])
                     group.x3 = 0.0;
@@ -1494,6 +1771,10 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
                 if (![scanner scanFloat:&group.y3])
                     group.y3 = 0.0;
                 break;
+            case ID_Z3:
+                if (![scanner scanFloat:&group.z3])
+                    group.z3 = 0.0;
+                break;
             case ID_WIDTH:	/* begWidth, radius, sumLen, height */
                 if (![scanner scanFloat:&group.width])
                     group.width = -1.0;
@@ -1548,7 +1829,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
                 if (![scanner scanFloat:&group.extY])
                     group.extY = 0.0;
                 if ( group.extX != 0.0 || group.extY != 0.0 )   // FIXME: extrusion direction
-                    NSLog(@"DXF-Import: extrusion direction not supported. File-Location = %d", [scanner scanLocation]);
+                    NSLog(@"DXF-Import: extrusion direction not supported. File-Location = %u", (unsigned)[scanner scanLocation]);
                 break;
             case ID_EXT_Z:
                 if (![scanner scanFloat:&group.extZ])
@@ -1556,12 +1837,30 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
                 break;
             case ID_GROUP:
                 [scanner setScanLocation:location];
-                return YES;
+                exitLoop = YES;
+                break;
             default:
                 break;
         }
+        if ( exitLoop )
+            break;
         gotoNextLine(scanner);	// goto id line
     }
+
+    /* transform coordinates from OCS to WCS (only if extrusion direction tells us so) */
+    if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )
+    {   V3Point N = V3MakePoint(group.extX, group.extY, group.extZ), p;
+
+        p = transformOCS(V3MakePoint(group.x0, group.y0, group.z0), N);
+        group.x0 = p.x; group.y0 = p.y; group.z0 = p.z;
+        p = transformOCS(V3MakePoint(group.x1, group.y1, group.z1), N);
+        group.x1 = p.x; group.y1 = p.y; group.z1 = p.z;
+        p = transformOCS(V3MakePoint(group.x2, group.y2, group.z2), N);
+        group.x2 = p.x; group.y2 = p.y; group.z2 = p.z;
+        p = transformOCS(V3MakePoint(group.x3, group.y3, group.z3), N);
+        group.x3 = p.x; group.y3 = p.y; group.z3 = p.z;
+    }
+
     return YES;
 }
 
@@ -1588,10 +1887,18 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 {
     NSLog(@"line: %f %f %f %f", beg.x, beg.y, end.x, end.y); 
 }
+- (void)addLine3D:(V3Point)beg :(V3Point)end toList:(NSMutableArray*)aList
+{
+    NSLog(@"line 3D: %f %f %f %f %f %f", beg.x, beg.y, beg.z, end.x, end.y, end.z);
+}
 - (void)addLine:(NSPoint)beg :(NSPoint)end toLayer:(NSString*)layerName
 {
     NSLog(@"line: %@ %f %f %f %f", layerName, beg.x, beg.y, end.x, end.y); 
 }
+- (void)addLine3D:(V3Point)beg :(V3Point)end toLayer:(NSString*)layerName
+{
+    NSLog(@"line 3D: %@ %f %f %f %f %f %F", layerName, beg.x, beg.y, beg.z, end.x, end.y, end.z); 
+}
 
 - (void)addArc:(NSPoint)center :(NSPoint)start :(float)angle toList:(NSMutableArray*)aList
 {
@@ -1610,6 +1917,11 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 {
     NSLog(@"curve: %@ %f %f %f %f %f %f %f %f", layerName, p0.x, p0.y, p1.x, p1.y, p2.x, p2.y, p3.x, p3.y);
 }
+- (void)add3DFace:(V3Point*)pts toLayer:(NSString*)layerName
+{
+    NSLog(@"3DFace: %@ %f %f %f, %f %f %f, %f %f %f, %f %f %f", layerName, pts[0].x, pts[0].y, pts[0].z,
+          pts[1].x, pts[1].y, pts[1].z, pts[2].x, pts[2].y, pts[2].z, pts[3].x, pts[3].y, pts[3].z);
+}
 
 /* allocate a text object and add it to layer
  * parameter: text	the text string
