@@ -1,11 +1,12 @@
 /* HPGLImport.m
  * HPGL import object
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-03
- * modified: 2008-06-15 (-getGraphicFromData: imply PD if we get an arc)
+ * modified: 2012-01-05 (-getLabelSize: scanDouble and make it work with all sizes of NSPoint)
+ *           2008-06-15 (-getGraphicFromData: imply PD if we get an arc)
  *
  * This file is part of the vhf Import Library.
  *
@@ -448,29 +449,32 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
 }
 
 /*
- * created:   18.04.94
- * modified:  18.04.94 04.05.96 09.03.97
+ * created:   1994-04-18
+ * modified:  2012-01-05 (scanDouble and make it work with all sizes of NSPoint)
  * purpose:   get label size
  *            the size comes in cm, we store it in internal units
  * parameter: cp (at or behind command)
  */
 - (BOOL)getLabelSize:(NSScanner*)scanner
-{
+{   double  d;
+
     [scanner scanString:ops.labelSize intoString:NULL];
 
-    if ( ![scanner scanFloat:&state.labelSize.width] )
-        state.labelSize.width = state.labelSize.height = 0.0;
-    else if ( ![scanner scanFloat:&state.labelSize.height] )
-    {   NSLog(@"%@ (height) expected at location:%d", ops.labelSize, [scanner scanLocation]);
-        state.labelSize.height = 0.0;
+    state.labelSize = NSZeroSize;
+    if ( [scanner scanDouble:&d] )
+    {   state.labelSize.width = d;
+        if ( [scanner scanDouble:&d] )
+            state.labelSize.height = d;
+        else
+            NSLog(@"%@ (height) expected at location:%d", ops.labelSize, [scanner scanLocation]);
     }
 
-    if ( state.labelSize.width == 0.0 )	/* default size */
+    if ( state.labelSize.width == 0.0 )	// default size
         state.labelSize.width = 11.5 * PT;
     else
         state.labelSize.width *= MM*10.0;
 
-    if (state.labelSize.height == 0.0)	/* default size */
+    if (state.labelSize.height == 0.0)	// default size
         state.labelSize.height = 11.5 * PT;
     else
         state.labelSize.height *= MM*10.0;
