@@ -1,11 +1,12 @@
 /* PSImport.m
  * PostScript import object
  *
- * Copyright (C) 1996-2009 by vhf interservice GmbH
+ * Copyright (C) 1996-2011 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-25
- * modified: 2009-02-02 (-importPS: for GhostScript: -dNOSAFER flag added
+ * modified: 2011-09-03 (-importPS: for GhostScript: try several string loadings to get encoding right)
+ *           2009-02-02 (-importPS: for GhostScript: -dNOSAFER flag added
  *           2009-01-31 (-importPS: for GhostScript+DPS: remove Corel Draw garbage before %!PS-ADOBE...)
  *           2008-11-29 (init state.color in GhostScript version of importPS)
  *
@@ -42,6 +43,7 @@
 #include "../VHFShared/vhf2DFunctions.h"
 #include "../VHFShared/types.h"
 #include "../VHFShared/VHFScannerAdditions.h"
+#include "../VHFShared/VHFStringAdditions.h"    // -writeToFile:... for 10.4
 
 /* USEDPS: 1 = use Display PostScript, 0 = use gs (GhostScript) */
 #if defined(GNUSTEP_BASE_VERSION)
@@ -71,6 +73,20 @@
 
 @implementation PSImport
 
+/* return a temporary path
+ * created: 2012-02-07
+ */
+- (NSString*)tmpPath
+{   NSString    *tmpPath = PS_TMPPATH;
+    //NSString    *app = [[NSProcessInfo processInfo] processName];
+
+#   if defined(__APPLE__) || defined(GNUSTEP_BASE_VERSION)
+    tmpPath = NSTemporaryDirectory();
+#   endif
+    //tmpPath = [tmpPath stringByAppendingPathComponent:APPNAME];
+    return tmpPath;
+}
+
 /* created:   1996-05-03
  * modified:  
  * parameter: flag
@@ -108,10 +124,10 @@
  * purpose:   import pdf data
  */
 - importPDF:(NSData*)pdfData
-{   NSString	*app = [[NSProcessInfo processInfo] processName];
-    NSString	*pdfFile = vhfPathWithPathComponents(PS_TMPPATH,
+{   NSString    *app = [[NSProcessInfo processInfo] processName];
+    NSString    *pdfFile = vhfPathWithPathComponents([self tmpPath],
                            [app stringByAppendingString:@"_pdfImport.pdf"], nil);
-    id		result;
+    id          result;
 
     [pdfData writeToFile:pdfFile atomically:NO];
     result = [self importPDFFromFile:pdfFile];
@@ -120,14 +136,14 @@
     return result;
 }
 - importPDFFromFile:(NSString*)pdfFile
-{   NSString	*app = [[NSProcessInfo processInfo] processName], *commandLine;
-    NSString	*psFile = vhfPathWithPathComponents(PS_TMPPATH,
+{   NSString    *app = [[NSProcessInfo processInfo] processName], *commandLine;
+    NSString    *psFile = vhfPathWithPathComponents([self tmpPath],
                           [app stringByAppendingString:@"_pdfImport.eps"], nil);
-    NSData	*psData;
+    NSData      *psData;
 
     /* convert pdf to eps */
     commandLine = [NSString stringWithFormat:@"%@ -q -dNOPAUSE -dBATCH -dSAFER -sDEVICE=epswrite -sOutputFile=%@ -c save pop -f '%@'", [self gsPath], psFile, pdfFile];
-    system([commandLine cString]);
+    system([commandLine UTF8String]);
 
     /* import ps */
     psData = [NSData dataWithContentsOfFile:psFile];
@@ -152,17 +168,17 @@
  * We can't use NSEPSImageRep because the context is write protected
  */
 - importPS:(NSData*)psData
-{   NSAutoreleasePool	*pool = [NSAutoreleasePool new];
-    NSDPSContext	*curContext = [NSDPSContext currentContext];
-    NSDPSServerContext	*psContext = nil;
-    NSString		*app = [[NSProcessInfo processInfo] processName];
-    NSString		*path = vhfPathWithPathComponents(PS_TMPPATH,
+{   NSAutoreleasePool   *pool = [NSAutoreleasePool new];
+    NSDPSContext        *curContext = [NSDPSContext currentContext];
+    NSDPSServerContext  *psContext = nil;
+    NSString            *app = [[NSProcessInfo processInfo] processName];
+    NSString            *path = vhfPathWithPathComponents([self tmpPath],
                                 [app stringByAppendingString:@"_psImport"], nil);
-    NSString		*data;
-    NSUserDefaults	*userDefaults = [NSUserDefaults standardUserDefaults];
-    NSString		*hostName = [userDefaults stringForKey:@"NSHost"];
-    NSString		*serverName = [userDefaults stringForKey:@"NSPSName"];
-    NSTimeInterval	timeout = [userDefaults floatForKey:@"NSNetTimeout"];
+    NSString            *data;
+    NSUserDefaults      *userDefaults = [NSUserDefaults standardUserDefaults];
+    NSString            *hostName = [userDefaults stringForKey:@"NSHost"];
+    NSString            *serverName = [userDefaults stringForKey:@"NSPSName"];
+    NSTimeInterval      timeout = [userDefaults floatForKey:@"NSNetTimeout"];
 
     if ( !psData )
         return nil;
@@ -236,11 +252,12 @@
 
 static NSString	*prolog = nil;
 - importPS:(NSData*)psData
-{   NSString        *app = [[NSProcessInfo processInfo] processName], *commandLine, *path;
-    NSString        *inFile = vhfPathWithPathComponents(PS_TMPPATH,
-                              [app stringByAppendingString:@"_psImport.in"], nil);
-    NSString        *outFile = vhfPathWithPathComponents(PS_TMPPATH, @"psImport.out", nil); // see psImport.prolog
-    NSMutableString *data = [NSMutableString string];
+{   NSString            *app = [[NSProcessInfo processInfo] processName], *commandLine, *path;
+    NSString            *inFile  = vhfPathWithPathComponents([self tmpPath], [app stringByAppendingString:@"_psImport.in"],  nil);
+    //NSString            *outFile = vhfPathWithPathComponents(PS_TMPPATH, @"_psImport.out", nil);    // see psImport.prolog !
+    NSString            *outFile = vhfPathWithPathComponents([self tmpPath], [app stringByAppendingString:@"_psImport.out"], nil); // see psImport.prolog
+    NSMutableString     *data = [NSMutableString string];
+    NSStringEncoding    enc;
 
     if ( !psData )
         return nil;
@@ -275,7 +292,21 @@ static NSString	*prolog = nil;
         path = [[NSBundle bundleForClass:[PSImport class]] resourcePath];
         path = [path stringByAppendingPathComponent:@"psImport.prolog"];
         prolog = [[NSString stringWithContentsOfFile:path] retain];
-        if (!prolog)
+        if (prolog)
+        {   NSRange range = [prolog rangeOfString:@"/tmp/psImport.out"];
+
+            /* set file name */
+            if ( [prolog respondsToSelector:@selector(stringByReplacingCharactersInRange:withString:)] )    // SDK >= 10.5
+                prolog = [[[prolog autorelease] stringByReplacingCharactersInRange:range withString:outFile] retain];
+            else
+            {   NSMutableString *newProlog = [prolog mutableCopy];
+
+                [prolog release];
+                [newProlog replaceCharactersInRange:range withString:outFile];
+                prolog = newProlog;
+            }
+        }
+        else
             NSLog(@"Couldn't load psImport.prolog file '%@'", path);
     }
 
@@ -295,13 +326,19 @@ static NSString	*prolog = nil;
         nsData = [data dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES];
         data = [[[NSString alloc] initWithData:nsData encoding:NSASCIIStringEncoding] autorelease];
     }
-    [data writeToFile:inFile atomically:NO];
+    [data writeToFile:inFile atomically:NO encoding:NSASCIIStringEncoding error:NULL];  // >= 10.5
+    //[data writeToFile:inFile atomically:NO];    // <= 10.4
 
     commandLine = [NSString stringWithFormat:@"%@ -dNODISPLAY -dBATCH -dNOSAFER -g1000x1000 '%@'", [self gsPath], inFile];
-    system([commandLine cString]);
+    system([commandLine UTF8String]);
 
     /* load generated file */
-    data = [[[NSString alloc] initWithContentsOfFile:outFile] autorelease];
+    //data = [[[NSString alloc] initWithContentsOfFile:outFile] autorelease];
+    data = [[[NSString alloc] initWithContentsOfFile:outFile usedEncoding:&enc error:NULL] autorelease];
+    if ( !data )    // files from Corel Draw, etc.
+        data = [[[NSString alloc] initWithContentsOfFile:outFile encoding:NSISOLatin1StringEncoding error:NULL] autorelease];
+    if ( !data )    // files from Next-Step (ex. Example file with Ducks)
+        data = [[[NSString alloc] initWithContentsOfFile:outFile encoding:NSNEXTSTEPStringEncoding error:NULL] autorelease];
 
     /* interpret data and send to receiving object */
     if ( !data || ![self interpret:data] )
@@ -334,20 +371,20 @@ static NSString	*prolog = nil;
 
 #define DESCALE	0.5
 - (BOOL)interpret:(NSString*)data
-{   NSString		*string;
-    float		value, c[MAXCOORDCNT];		// stack for coordinates
-    NSMutableArray	*s = [NSMutableArray array];	// stack for strings
-    int			cCnt = 0, poolCnt = 0;
-    NSPoint		p0, p1, p2, p3;
-    NSMutableArray	*cList = [NSMutableArray array];	// current list
-    NSRect		bounds;
-    PSState		states[MAXSTATES];	// array of states for gsave and grestore
-    int			stateCnt = 0;
-    NSScanner		*scanner = [NSScanner scannerWithString:data];
-    NSCharacterSet	*skipCharacters = [scanner charactersToBeSkipped];
-    NSCharacterSet	*noCharacters = [NSCharacterSet characterSetWithCharactersInString:@""];
-    NSAutoreleasePool	*pool = nil;
-    BOOL		ok = YES;
+{   NSString            *string;
+    float               value, c[MAXCOORDCNT];          // stack for coordinates
+    NSMutableArray      *s = [NSMutableArray array];    // stack for strings
+    int                 cCnt = 0, poolCnt = 0;
+    NSPoint             p0, p1, p2, p3;
+    NSMutableArray      *cList = [NSMutableArray array];    // current list
+    NSRect              bounds;
+    PSState             states[MAXSTATES];  // array of states for gsave and grestore
+    int                 stateCnt = 0;
+    NSScanner           *scanner = [NSScanner scannerWithString:data];
+    NSCharacterSet      *skipCharacters = [scanner charactersToBeSkipped];
+    NSCharacterSet      *noCharacters = [NSCharacterSet characterSetWithCharactersInString:@""];
+    NSAutoreleasePool   *pool = nil;
+    BOOL                ok = YES;
 
     coordinateSet = [NSCharacterSet characterSetWithCharactersInString:DIGITS];
     invCoordinateSet = [coordinateSet invertedSet];
