@@ -1,11 +1,13 @@
 /* VHFStringAdditions.m
  * vhf NSString additions
  *
- * Copyright (C) 1997-2008 by vhf interservice GmbH
+ * Copyright (C) 1997-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1997-07-08
- * modified: 2008-11-08 (inactive method for NSMutableArray parked here)
+ * modified: 2012-02-07 (-writeToFile:atomically:encoding:error: added for backward compatibility)
+ *           2011-09-01 (-stringWithContentsOfFile: loads flexible on Apple)
+ *           2008-11-08 (inactive method for NSMutableArray parked here)
  *
  * This file is part of the vhf Shared Library.
  *
@@ -37,6 +39,30 @@
 
 @implementation NSString(VHFStringAdditions)
 
+/* Methods to replace deprecated methods */
+#ifdef __APPLE__
++ (id)stringWithContentsOfFile:(NSString*)path
+{   NSStringEncoding	enc;
+    NSString            *string;
+
+    string = [self stringWithContentsOfFile:path usedEncoding:&enc error:NULL];
+    if ( !string )
+        string = [self stringWithContentsOfFile:path encoding:NSASCIIStringEncoding error:NULL];
+    return string;
+}
+
+/* Methods added in certain versions of Mac OS X */
+#   if MAC_OS_X_VERSION_MAX_ALLOWED < 1050 /*MAC_OS_X_VERSION_10_5*/
+    - (BOOL)writeToFile:(NSString *)path atomically:(BOOL)useAuxiliaryFile
+               encoding:(NSStringEncoding)enc error:(NSError **)error
+    {
+        //printf("writeToFile: replacement\n");
+        return [self writeToFile:path atomically:useAuxiliaryFile];    // <= 10.4
+    }
+#   endif
+#endif
+
+
 /* return string from float with 4 decimals
  */
 + (NSString*)stringWithFloat:(float)value
@@ -143,9 +169,9 @@
     return [self stringByReplacing:from by:to all:YES];
 }
 - (NSString*)stringByReplacing:(NSString*)from by:(NSString*)to all:(BOOL)replaceAll;
-{   NSRange		range, searchRange;
-    NSMutableString	*mutString = [NSMutableString string];
-    int			start = 0;
+{   NSRange         range, searchRange;
+    NSMutableString *mutString = [NSMutableString string];
+    int             start = 0;
 
     searchRange = NSMakeRange(0, [self length]);
     while ( searchRange.length )
@@ -258,9 +284,9 @@
 }
 
 - (NSString*)stringWithLength:(int)length fillCharacter:(NSString*)fillChar
-{   int			i, space;
-    NSMutableString	*newString;
-    BOOL		alignLeft = YES;
+{   int             i, space;
+    NSMutableString *newString;
+    BOOL            alignLeft = YES;
 
     if (length < 0)
     {
