@@ -1,11 +1,13 @@
 /* vhf2DFunctions.m
  * vhf 2-D graphic functions
  *
- * Copyright (C) 1996-2009 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-19
- * modified: 2008-02-17 (some clean-up)
+ * modified: 2012-01-25 (vhfConvert3DLineToLines() new)
+ *           2011-04-26 (vhfConvertLineToLines(), vhfBoundsOfCurve())
+ *           2008-02-17 (some clean-up)
  *           2008-08-18 (vhfIsIntValueInArray() added, valueInArray() is deprecated now)
  *
  * This program is free software; you can redistribute it and/or
@@ -29,6 +31,7 @@
 
 #include <math.h>
 #include "vhf2DFunctions.h"
+#include "vhfMath.h"
 
 /*
  * created:   04.09.93
@@ -595,9 +598,9 @@ float vhfAngleBetweenPoints(const NSPoint start, const NSPoint middle, const NSP
         angle0 = (dy >= 0) ? 90.0 : 270;
     else
     {	angle0 = RadToDeg(atan(dy / dx));
-        if (dx<0)		/* 2nd, 3rd quadrant */
+        if (dx<0)           /* 2nd, 3rd quadrant */
             angle0 += 180;
-        if (dx>0 && dy<0)	/* 4th quadrant */
+        if (dx>0 && dy<0)   /* 4th quadrant */
             angle0 += 360;
     }
 
@@ -609,9 +612,9 @@ float vhfAngleBetweenPoints(const NSPoint start, const NSPoint middle, const NSP
         angle1 = (dy >= 0) ? 90.0 : 270;
     else
     {	angle1 = RadToDeg(atan(dy / dx));
-        if (dx<0)		/* 2nd, 3rd quadrant */
+        if (dx<0)           /* 2nd, 3rd quadrant */
             angle1 += 180;
-        if (dx>0 && dy<0)	/* 4th quadrant */
+        if (dx>0 && dy<0)   /* 4th quadrant */
             angle1 += 360;
     }
 
@@ -907,6 +910,82 @@ BOOL vhfGetGradientOfCurveAt(NSPoint *p, NSPoint pc[4], float t)
     return YES;
 }
 
+/* flatten line
+ *
+ * return: number of points in pts
+ *         pts has to be freed using NSZoneFree(NSDefaultMallocZone(), pts)
+ *
+ * modified: 2011-04-23
+ */
+int vhfConvertLineToLines(NSPoint pc[2], float flatness, NSPoint **pts)
+{   NSZone  *zone = NSDefaultMallocZone();
+    double	dx = pc[1].x-pc[0].x, dy = pc[1].y -pc[0].y;
+    double  length2 = dx*dx + dy*dy;
+    NSPoint *ptsPtr;
+
+    if ( flatness < 0.02 )
+        flatness = 0.02;
+
+    if (length2 > flatness * flatness)
+    {   NSZone  *zone = NSDefaultMallocZone();
+        int     i, n = Max(1, (int)ceil(sqrt(length2 / (flatness*flatness))));
+        NSPoint	len = (NSPoint){dx / (double)n, dy / (double)n};
+
+        *pts = NSZoneMalloc(zone, (n+1) * sizeof(NSPoint));
+        ptsPtr = *pts;
+
+        ptsPtr[0] = pc[0];
+
+        for (i=1; i<=n; i++)
+        {   ptsPtr[i].x = pc[0].x + (double)i * len.x;
+            ptsPtr[i].y = pc[0].y + (double)i * len.y;
+        }
+        return n+1;
+    }
+    *pts = NSZoneMalloc(zone, 2 * sizeof(NSPoint));
+    (*pts)[0] = pc[0];
+    (*pts)[1] = pc[1];
+    return 2;
+}
+
+/* flatten 3D line
+ *
+ * return: number of points in pts
+ *         pts has to be freed using NSZoneFree(NSDefaultMallocZone(), pts)
+ *
+ * modified: 2011-12-19
+ */
+int vhfConvert3DLineToLines(V3Point pc[2], float flatness, V3Point **pts)
+{   NSZone  *zone = NSDefaultMallocZone();
+    double	dx = pc[1].x-pc[0].x, dy = pc[1].y -pc[0].y, dz = pc[1].z -pc[0].z;
+    double  length2 = dx*dx + dy*dy + dz*dz;
+    V3Point *ptsPtr;
+
+    if ( flatness < 0.02 )
+        flatness = 0.02;
+
+    if (length2 > flatness * flatness)
+    {   int     i, n = Max(1, (int)ceil(sqrt(length2 / (flatness*flatness))));
+        V3Point	len = (V3Point){dx / (double)n, dy / (double)n, dz / (double)n};
+
+        *pts = NSZoneMalloc(zone, (n+1) * sizeof(V3Point));
+        ptsPtr = *pts;
+
+        ptsPtr[0] = pc[0];
+
+        for (i=1; i<=n; i++)
+        {   ptsPtr[i].x = pc[0].x + (double)i * len.x;
+            ptsPtr[i].y = pc[0].y + (double)i * len.y;
+            ptsPtr[i].z = pc[0].z + (double)i * len.z;
+        }
+        return n+1;
+    }
+    *pts = NSZoneMalloc(zone, 2 * sizeof(V3Point));
+    (*pts)[0] = pc[0];
+    (*pts)[1] = pc[1];
+    return 2;
+}
+
 /* flatten curve
  *
  * return: number of points in pts
@@ -1124,6 +1203,74 @@ NSRect vhfBoundsOfArc(NSPoint center, float radius, float begAngle, float angle)
     return bounds;
 }
 
+/*
+ * modified:  01.08.93 30.04.96
+ * Author:    Martin Dietterle
+ * purpose:   get bounds of curve
+ * parameter: curve points, bounds
+ * return:    none
+ */
+NSRect vhfBoundsOfCurve( NSPoint pc[4] )
+{   double	max, min, ft;
+    double	ax, bx, cx, ay, by, cy, t, ts[2];
+    int		i, cnt;
+    NSRect	bounds;
+
+    /* Berechnung und Festlegung der Bezierkurve durch die Gleichungen
+     *  x(t) = ax*t^3 + bx*t^2 + cx*t + x(0)
+     *  y(t) = ay*t^3 + by*t^2 + cy*t + y(0)
+     */
+    cx = 3.0*(pc[1].x - pc[0].x);
+    bx = 3.0*(pc[2].x - pc[1].x) - cx;
+    ax = pc[3].x - pc[0].x - bx - cx;
+    cy = 3.0*(pc[1].y - pc[0].y);
+    by = 3.0*(pc[2].y - pc[1].y) - cy;
+    ay = pc[3].y - pc[0].y - by - cy;
+
+    /* Ermittlung des maximalen und minimalen X-Wertes der Bezier-Kurve. */
+    min = Min(pc[0].x, pc[3].x);
+    max = Max(pc[0].x, pc[3].x);
+
+    cnt = svPolynomial2(3.0 * ax, 2.0 * bx, cx, ts);
+    for (i=0; i<cnt; i++)
+    {
+        t = ts[i];
+        if ( t>0.0 && t<1.0)
+        {
+            ft = ax*t*t*t + bx*t*t + cx*t + pc[0].x;
+            if (min > ft)
+                min = ft;
+            else if (max < ft)
+                max = ft;
+        }
+    }
+
+    bounds.origin.x = min;
+    bounds.size.width = max - min;
+
+    /* Ermittlung des maximalen und minimalen Y-Wertes der Bezier-Kurve. */
+    min = Min(pc[0].y, pc[3].y);
+    max = Max(pc[0].y, pc[3].y);
+
+    cnt = svPolynomial2(3.0 * ay, 2.0 * by, cy, ts);
+    for (i=0; i<cnt; i++)
+    {
+        t = ts[i];
+        if ( t>0.0 && t<1.0)
+        {
+            ft = ay*t*t*t + by*t*t + cy*t + pc[0].y;
+            if (min > ft)
+                min = ft;
+            else if (max < ft)
+                max = ft;
+        }
+    }
+
+    bounds.origin.y = min;
+    bounds.size.height = max - min;
+    return bounds;
+}
+
 /* flatten arc into lines
  * return: number of points in pts
  *         pts has to be freed using NSZoneFree(NSDefaultMallocZone(), pts)
