@@ -1,11 +1,12 @@
 /*
  * vhfCommonFunctions.m
  *
- * Copyright (C) 1996-2008 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-25
- * modified: 2008-02-28
+ * modified: 2012-02-28 (vhfMOdulo() added)
+ *           2012-02-06 (vhfUserDocuments())
  *
  * This file is part of the vhf Shared Library.
  *
@@ -134,12 +135,15 @@ void checkPoint(NSPoint p)
         NSLog(@"point.y > maximum");
 }
 
+/* return library paths
+ * modified: 2008-02-28
+ */
 /* return library paths
  * modified: 2008-02-28
  */
 NSString *vhfLocalLibrary(NSString* append)
 {
-#if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__)
+#   if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__)
     NSFileManager	*fileManager = [NSFileManager defaultManager];
     NSString		*lPath, *sPath;
 
@@ -164,28 +168,47 @@ NSString *vhfLocalLibrary(NSString* append)
     if ( sPath && [fileManager fileExistsAtPath:sPath] )
         return sPath;
     return lPath;
-#else	// OpenStep
+#   else	// OpenStep: /LocalLibrary
     return vhfPathWithPathComponents(@"/LocalLibrary", append, nil);
-#endif
+#   endif
 }
 NSString *vhfUserLibrary(NSString* append)
 {
-#if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__)
+#   if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__)
     NSFileManager	*fileManager = [NSFileManager defaultManager];
     NSString		*lPath;
 
-    /* "Library/Application Support/..." */
+    /* "$HOME/Library/Application Support/..." */
     lPath = vhfPathWithPathComponents(
             [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) objectAtIndex:0],
             @"Application Support", append, nil );
     if ( lPath && [fileManager fileExistsAtPath:lPath] )
         return lPath;
-    /* "Library/..."  (Default) */
+    /* "$HOME/Library/..."  (Default) */
     return [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) objectAtIndex:0]
             stringByAppendingPathComponent:append];
-#else	// OpenStep
+#   else	// OpenStep: $HOME/Library
     return vhfPathWithPathComponents(NSHomeDirectory(), @"Library", append, nil);
-#endif
+#   endif
+}
+NSString *vhfUserDocuments(NSString* append)
+{
+#   if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__)
+    //NSFileManager	*fileManager = [NSFileManager defaultManager];
+
+    /* "$HOME/Documents/..." */
+    return [[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0]
+            stringByAppendingPathComponent:append];
+#   else	// OpenStep
+    return nil;
+#   endif
+}
+/* Cenon.app/...Resources/Library/append or CAM.bundle/...Resources/Library/append */
+NSString *vhfBundleLibrary(NSBundle *bundle, NSString* append)
+{   NSString    *rscPath = [bundle resourcePath], *path;
+
+    path = vhfPathWithPathComponents(rscPath, @"Library", append, nil);
+    return path;
 }
 /* return "path/" instead of "path" ()
  * DEPRECATED !, use vhfPathWithPathComponents(NSHomeDirectory(), ...) or the NSString method
@@ -203,9 +226,9 @@ NSString *vhfHomeDirectory(void)	// deprecated !
  * created: 2005-11-13
  */
 NSString *vhfPathWithPathComponents(NSString *seg1, ...)
-{   va_list	arg_p;
-    void	*arg;
-    NSString	*path = seg1;
+{   va_list     arg_p;
+    void        *arg;
+    NSString    *path = seg1;
 
     va_start(arg_p, seg1);
     while ( (arg = va_arg(arg_p, void*)) != nil )
@@ -216,6 +239,46 @@ NSString *vhfPathWithPathComponents(NSString *seg1, ...)
     return path;
 }
 
+/* return 1st path+name that fits a name alternative within a list of paths
+ * created: 2012-06-22
+ */
+NSString    *vhfFilePathForNamesInPaths(id name, NSString *seq1, ...)
+{   va_list         arg_p;
+    NSString        *path = seq1;
+    NSFileManager   *fileManager = [NSFileManager defaultManager];
+    NSArray         *names = ([name isKindOfClass:[NSArray class]]) ? name : nil;
+
+    va_start(arg_p, seq1);
+    while ( 1 )
+    {   NSString    *fileName;
+
+        if ( names )    // array of name choices
+        {   int i;
+
+            for ( i = 0; i < [names count]; i++)
+            {   fileName = [path stringByAppendingPathComponent:[names objectAtIndex:i]];
+                if ( [fileManager fileExistsAtPath:fileName] )
+                {   va_end(arg_p);
+                    return fileName;
+                }
+            }
+        }
+        else            // single string
+        {
+            fileName = [path stringByAppendingPathComponent:name];  // userLibrary() + "name.cenon"
+            if ( [fileManager fileExistsAtPath:fileName] )
+            {   va_end(arg_p);
+                return fileName;
+            }
+        }
+
+        if ( ! (path = va_arg(arg_p, void*)) )
+            break;
+    }
+    va_end(arg_p);
+    return nil;
+}
+
 #if 0
 /* scan up to sequence
  * a sequence is separated by space
@@ -347,10 +410,10 @@ NSString *buildRoundedString(float value, float limitL, float limitH)
     return string;
 }
 NSString *vhfStringWithFloat(float value)
-{   int			i;
-    NSMutableString	*string;
-    NSRange		range;
-    char		c;
+{   int             i;
+    NSMutableString *string;
+    NSRange         range;
+    char            c;
 
     value += 0.000005;
     string = [NSMutableString stringWithFormat:@"%.4f", value];
@@ -409,6 +472,14 @@ NSString *vhfStringWithDouble(double value)
     return string;
 }
 
+/* Modulo - return v % denom
+ * created: 2012-02-28
+ */
+double vhfModulo(double v, double denom)
+{
+    return (v - floor(v / denom) * denom);
+}
+
 /* created:   16.07.93
  * modified:  16.07.93
  * purpose:   compare function for qsort
