@@ -40,8 +40,8 @@
     NSRect	affectedBounds = NSZeroRect;
 
     for ( l=0, cnt = [slayList count]; l<cnt; l++ )
-    {	NSMutableArray *slist = [slayList objectAtIndex:l];
-        LayerObject	*layer = [layerList objectAtIndex:l];
+    {	NSMutableArray  *slist = [slayList  objectAtIndex:l];
+        LayerObject     *layer = [layerList objectAtIndex:l];
 
         if (![layer editable])
             continue;
@@ -101,9 +101,9 @@
     [change startChange];
         switch (colorNum)
         {
-            default: [self graphicsPerform:@selector(setColor:) with:color]; break;
+            default: [self graphicsPerform:@selector(setColor:)     with:color]; break;
             case 1:  [self graphicsPerform:@selector(setFillColor:) with:color]; break;
-            case 2:  [self graphicsPerform:@selector(setEndColor:) with:color];
+            case 2:  [self graphicsPerform:@selector(setEndColor:)  with:color];
         }
         [document setDirty:YES];
     [change endChange];
@@ -149,25 +149,49 @@
  */
 - (void)takeFillFrom:sender
 {   id		change;
+    int		l, cnt, i;
+    NSRect	affectedBounds = NSZeroRect;
     int		filled = [sender indexOfSelectedItem];
 
     change = [[FillGraphicsChange alloc] initGraphicView:self fill:filled];
     [change startChange];
-	[self graphicsPerform:@selector(setFilled:) with:(void*)filled];
-        [document setDirty:YES];
+        //[self graphicsPerform:@selector(setFilled:) with:(void*)filled];
+        //[document setDirty:YES];
+        for (l=0, cnt = [slayList count]; l<cnt; l++)
+        {   NSMutableArray	*slist = [slayList  objectAtIndex:l];
+            LayerObject		*layer = [layerList objectAtIndex:l];
+
+            if (![layer editable])
+                continue;
+            for (i=[slist count]-1; i>=0; i--)
+            {   VGraphic    *g = [slist objectAtIndex:i];
+
+                if ( [g respondsToSelector:@selector(setFilled:)] )
+                {
+                    affectedBounds = (affectedBounds.size.width)
+                                        ? NSUnionRect([g extendedBoundsWithScale:scale], affectedBounds)
+                                        : [g extendedBoundsWithScale:scale];
+                    [g setFilled:filled];
+                    [layer updateObject:g];
+                    affectedBounds = NSUnionRect([g extendedBoundsWithScale:scale], affectedBounds);
+                }
+            }
+        }
+    [document setDirty:YES];
+    [self cache:affectedBounds];
     [change endChange];
 }
 
 - (void)takeStepWidth:(float)stepWidth
 {   id		change;
     int		l, cnt, i;
-    NSRect	affectedBounds = NSMakeRect(0, 0, 0, 0);
+    NSRect	affectedBounds = NSZeroRect;
 
     change = [[StepWidthGraphicsChange alloc] initGraphicView:self stepWidth:stepWidth];
     [change startChange];
         cnt = [slayList count];
         for (l=0; l<cnt; l++)
-        {   NSMutableArray	*slist = [slayList objectAtIndex:l];
+        {   NSMutableArray	*slist = [slayList  objectAtIndex:l];
             LayerObject		*layer = [layerList objectAtIndex:l];
 
             if (![layer editable])
@@ -230,13 +254,13 @@
         //[self graphicsPerform:@selector(setWidth_ptr:) with:&width];
         cnt = [slayList count];
         for (l=0; l<cnt; l++)
-        {   NSMutableArray	*slist = [slayList objectAtIndex:l];
+        {   NSMutableArray	*slist = [slayList  objectAtIndex:l];
             LayerObject		*layer = [layerList objectAtIndex:l];
 
             if (![layer editable])
                 continue;
             for (i=[slist count]-1; i>=0; i--)
-            {   id	g = [slist objectAtIndex:i];
+            {   VGraphic    *g = [slist objectAtIndex:i];
 
                 affectedBounds = (affectedBounds.size.width) ? NSUnionRect([g extendedBoundsWithScale:scale], affectedBounds) : [g extendedBoundsWithScale:scale];
                 if ( [g respondsToSelector:@selector(setWidth:)] )
