@@ -1,10 +1,11 @@
 /* ContourGraphicsChange.h
  *
- * Copyright (C) 1993-2002 by vhf interservice GmbH
+ * Copyright (C) 1993-2011 by vhf interservice GmbH
  * Authors:  Georg Fleischmann
  *
  * created:  1993
- * modified: 2006-11-24
+ * modified: 2011-04-06 (-setRemoveSource: added)
+ *           2006-11-24
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -27,12 +28,13 @@
 
 @interface ContourGraphicsChange: GraphicsChange
 {
-    NSMutableArray	*newObjects;
-    BOOL		removeSource;
+    NSMutableArray  *newObjects;
+    BOOL            removeSource;
 }
 
 - (NSString *)changeName;
 - (void)saveBeforeChange;
+- (void)setRemoveSource:(BOOL)flag;
 - (void)saveAfterChange;
 - (Class)changeDetailClass;
 //- (void)noteNewObjects:(NSArray*)array;
