@@ -1,10 +1,10 @@
 /* SplitGraphicsChange.h
  *
- * Copyright (C) 1993-2002 by vhf interservice GmbH
+ * Copyright (C) 1993-2012 by vhf interservice GmbH
  * Authors:  Georg Fleischmann
  *
  * created:  1993
- * modified: 2002-07-15
+ * modified: 2012-02-24 (-noteList: added fixed with this)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -27,14 +27,16 @@
 
 @interface SplitGraphicsChange : Change
 {
-    id 			graphicView;
-    NSMutableArray	*changeDetails;
-    NSMutableArray	*groups;
+    id              graphicView;
+    NSMutableArray  *changeDetails;
+    NSMutableArray  *groups;
+    NSMutableArray  *splitList;
 }
 
 - initGraphicView:aGraphicView;
-- (NSString *)changeName;
+- (NSString*)changeName;
 - (void)saveBeforeChange;
+- (void)noteList:(NSArray*)aList;
 - (void)undoChange;
 - (void)redoChange;
 - (Class)changeDetailClass;
